/* Module for xsa   Apr.2005 */
/* For a given transition (A2 and A4 given) a slope for thet3 is returned */

/* Calculates anisotropy ratios using 4 detectors placed
 *  I (130/80/50/100) 
 * II (100/50/80/130) or close to max and min of ang.distribution
 * Input pattern: det1(high/low), det4 , det2 and det 3
 * in that order
 */

#include <stdio.h>
#include <stdlib.h>
#include <Xm/TextF.h>
#include <math.h>

#define    k_slope               29
#define    k_anis_box            23
#define    k_eta                 31

extern Widget   wdg_arr[];      /* all other widgets */

extern double Anm[][2];		/* angcor parameters A2 and A4 */
extern double dAnm[][2];	/* errors of typed coefficients */
extern double anm[];
double danm[2]; 		/* error of Acoefs */
extern double thet1, thet2;	/* anisotropy angles */
extern double thet3;		/* precession angle */
double lpar[2], lprime[2];	/* legendre polinomials */
extern double eta, deta;	/* eta parameter (err) */
extern double slope, dslope;	/* slope at precession angle */
double aniso, daniso;		/* anisotropy from peak ratio (for/back) */
extern int ispin;
extern int ntrs;
extern char *wptr[];
extern double q[];

void calcoeff()			/* calculate coef from eta */
{
  register int i,k;

  for(i=0; i<2; i++){
    k = 2*(i+1);
    anm[i]  = Anm[ispin][i]*q[i] * (1.0 - eta*(double)(k*(k+1)));
    danm[i] = (double)(k*(k+1))* fabs(anm[i]) * deta;
  }
}
void corslope()
{
  register int i;

  double w, dw, wp;

  w  = 1.0;
  dw = 0.0;
  wp = 0.0;
  dslope = 0.0;

  for(i=0;i<2;i++) {
    w  += Anm[ispin][i] *q[i] * lpar[i];
    dw += (lpar[i]*dAnm[ispin][i]*lpar[i]*dAnm[ispin][i]);

    wp += Anm[ispin][i] *q[i] * lprime[i];
  }
  dw = sqrt(dw);
  slope = wp / w;
  for(i=0;i<2;i++) {
    dslope  += (((w * lprime[i] - wp * lpar[i])*dAnm[ispin][i]) * 
      ((w * lprime[i] - wp * lpar[i])*dAnm[ispin][i]));
  } 
  dslope = sqrt(dslope)/(w*w);
}

void calslope()			/* calculate slope from eta */
{
  register int i;

  double w, dw, wp;
  double a,ap,b,bp;

  a = 1.0;
  ap = 0.0;
  b = 0.0;
  bp = 0.0;
  for(i=0;i<2;i++) {
    a  += Anm[ispin][i] *q[i] * lpar[i];
    ap += Anm[ispin][i] *q[i] * lprime[i];
    b  += Anm[ispin][i] *q[i] * lpar[i] * (double)(2*(i+1)*(2*(i+1) + 1));
    bp += Anm[ispin][i] *q[i] * lprime[i] * (double)(2*(i+1)*(2*(i+1) + 1));
  }
  w = a - b * eta;
  dw = fabs(b) * deta;
  wp = ap - bp * eta;
  slope = wp / w;
  dslope  = fabs(ap*b - a*bp) * deta / (w*w);
}

void legendre(double angle)           /* legendre polynomian 2&4 */
{
    double cosx, cosq;

    cosx = cos (0.01745329252 * angle);
    cosq = cosx*cosx;
    lpar[0] = 1.5*cosq -0.5;
    lpar[1] = (35.0*cosq*cosq - 30.0*cosq + 3.0) / 8.0;
}

void legendre_deriv(double angle)           /* derivatives of legendre pol */
{
    double costh, sinth, theta, costh3;

    theta = 0.01745329252 * angle;
    costh = cos (theta);
    sinth = -sin(theta);
    costh3 = pow(costh, (double)3.0);
    lprime[0] = 3.0 * costh * sinth;
    lprime[1] = sinth * (17.5*costh3 - 7.5 * costh);
}

void caleta()			/* calculate eta parameter from anisotropy */
{
  double a,b,c,d;
  double dd;
  int register i;

  a = 1.0;
  b = 1.0;
  c = 0.0;
  d = 0.0;

  legendre(thet1);
  for(i=0;i<2;i++) {
    a += Anm[ispin][i]*q[i] * lpar[i];
    c += Anm[ispin][i]*q[i] * lpar[i] * (double)(2*(i+1)*(2*(i+1) + 1));
  }
  legendre(thet2);
  for(i=0;i<2;i++) {
    b += Anm[ispin][i]*q[i] * lpar[i];
    d += Anm[ispin][i]*q[i] * lpar[i] * (double)(2*(i+1)*(2*(i+1) + 1));
  }
  dd = d*aniso - c;
  eta  = (b*aniso - a)  / dd;
  deta = daniso * fabs(a*d - b*c) / (dd*dd);
}


int anis()
{
  extern double pa[],dpa[],creff,dcreff;   /* peak areas and errors */
  extern char sadatfile[];
  int i,j,k;
  double nval, nvalerr;
  double res[2], rerr[2];
  double ferr[8], err;
  double sum, sumv;
  double weight;
  char chstr[64];

  FILE  *fopen(),*op;

  int order[8] = { 1, 1, 4, 4, 2, 2, 3, 3 };

  if(ntrs){
    aniso = atof(wptr[1]);
    daniso = (ntrs > 1) ? atof(wptr[2]) : 0.0;
  }
  else {

    for(i=0;i<8;i++) {		/*  check for missing data */
      printf("%d %10.1f %10.1f\n",order[i], pa[i], dpa[i]);
      ferr[i] = pow ((dpa[i]/pa[i]),2.0);
      if(pa[i] < 0.000001)
	break;
    }
    if (i == 2) {
      aniso = pa[0]*creff/pa[1];
      daniso = aniso*sqrt(ferr[0]+ferr[1]+dcreff);
      printf("  R (%.1f/%.1f)  =  %5.3f %5.3f\n",thet1,thet2,aniso,daniso);
    }
    else if (i == 8) {
      for(j=0;j<2;j++) {
      
	k = j*4;
	res[j] = sqrt((pa[k+0]/pa[k+1]) * (pa[k+2]/pa[k+3]));

	err = 0.0;
	for(i=0;i<4;i++)
	  err += ferr[k+i];
	rerr[j] = res[j]*sqrt(err)/2.0;

	for(i=0;i<4;i++)
	  printf("\n%d %10.1f %10.1f",order[k+i], pa[k+i], dpa[k+i]);
	printf("\n\n Ratio %1d/%1d = %f %f",order[k],order[k+2],res[j],rerr[j]);

      /* write to sa.dat */
	if ((op = fopen (sadatfile, "a")) != NULL) {
	  fprintf(op,"\n Angular Correlation information\n");
	  for(i=0;i<4;i++)
	    fprintf(op,"\n%d %10.1f %10.1f",order[k+i], pa[k+i], dpa[k+i]);
	  fprintf(op, "\n\n Ratio %1d/%1d = %f %f",
		  order[k],order[k+2],res[j],rerr[j]);
	  fclose(op);
	}
	else
	  printf("Warning:  No log!\n");
      }

      sum = 0.0;
      sumv = 0.0;
      for(i=0;i<2;i++) {
	weight = 1.0/(rerr[i]*rerr[i]);
	sum += weight;
	sumv += weight*res[i];
      }
      nval = sumv/sum;                        /* weighted mean */       
      nvalerr = sqrt(1.0/sum);                /* mean error */
      printf("    R  (forw/back)  =  %5.3f %5.3f\n",nval,nvalerr);
      if ((op = fopen (sadatfile, "a")) != NULL) {
	fprintf(op,"   R   (forw/back)  =  %5.3f %5.3f\n",nval,nvalerr);
	fclose(op);
      }
      aniso = nval;
      daniso = nvalerr;

      for(j=0;j<2;j++) {

	k = j*2;
	res[j] = sqrt((pa[k+0]/pa[k+1]) * (pa[k+4]/pa[k+5]));
	err = ferr[k+0] + ferr[k+1] + ferr[k+4] + ferr[k+5];
	rerr[j] = res[j]*sqrt(err)/2.0;
      }
      printf("\n Ratio 1/2 = %f %f",res[0],rerr[0]);
      printf("\n Ratio 3/4 = %f %f",res[1],rerr[1]);
      if ((op = fopen (sadatfile, "a")) != NULL) {
	fprintf(op,"\n Ratio 1/2 = %f %f",res[0],rerr[0]);
	fprintf(op,"\n Ratio 3/4 = %f %f",res[1],rerr[1]);
	fclose(op);
      }

      sum = 0.0;
      sumv = 0.0;
      for(i=0;i<2;i++) {
	weight = 1.0/(rerr[i]*rerr[i]);
	sum += weight;
	sumv += weight*res[i];
      }
      nval = sumv/sum;                        /* weighted mean */       
      nvalerr = sqrt(1.0/sum);                /* mean error */
      printf("    R (left/right)  =  %5.3f %5.3f\n",nval,nvalerr);
      if ((op = fopen (sadatfile, "a")) != NULL) {
	fprintf(op,"   R  (left/right)  =  %5.3f %5.3f\n",nval,nvalerr);
	fclose(op);
      }
      res[0] = sqrt((pa[0]/pa[1]) * (pa[7]/pa[6]));
      err = ferr[0] + ferr[1] + ferr[6] + ferr[7];
      rerr[0] = res[0]*sqrt(err)/2.0;
      res[1] = sqrt((pa[4]/pa[5]) * (pa[3]/pa[2]));
      err = ferr[2] + ferr[3] + ferr[4] + ferr[5];
      rerr[1] = res[1]*sqrt(err)/2.0;

      printf("\n Ratio 1/3 = %f %f",res[0],rerr[0]);
      printf("\n Ratio 2/4 = %f %f",res[1],rerr[1]);
      if ((op = fopen (sadatfile, "a")) != NULL) {
	fprintf(op,"\n Ratio 1/3 = %f %f",res[0],rerr[0]);
	fprintf(op,"\n Ratio 2/4 = %f %f",res[1],rerr[1]);
	fclose(op);
      }
      sum = 0.0;
      sumv = 0.0;
      for(i=0;i<2;i++) {
	weight = 1.0/(rerr[i]*rerr[i]);
	sum += weight;
	sumv += weight*res[i];
      }
      nval = sumv/sum;                        /* weighted mean */       
      nvalerr = sqrt(1.0/sum);                /* mean error */
      printf("    R   (cross)     =  %5.3f %5.3f\n\n",nval,nvalerr);
      if ((op = fopen (sadatfile, "a")) != NULL) {
	fprintf(op,"   R (cross check)  =  %5.3f %5.3f\n\n",nval,nvalerr);
	fclose(op);
      }
    }
    else
      return(1);
  }

  caleta();
  calcoeff();
  printf("             a2 = %f(%f)        a4 = %f(%f)\n",
	 anm[0],danm[0],anm[1],danm[1]);
  printf("         eta    =  %f(%f)\n",eta,deta);
  printf("             q2 = %f                 q4 = %f\n",q[0],q[1]);
  legendre_deriv(thet3);
  legendre(thet3);
  calslope();
  printf("    slope(%.1f) =  %5.3f(%.3f)\n",thet3, slope,dslope);
  if ((op = fopen (sadatfile, "a")) != NULL) {
    fprintf(op,"  R(%.1f,%.1f)  =  %f(%f)\n",thet1,thet2,aniso,daniso);
    fprintf(op,"         eta    =  %f(%f)\n",eta,deta);
    fprintf(op,"    slope(%.1f) =  %5.3f(%.3f)\n",thet3, slope,dslope);
    fprintf(op," Correlation coefficients:\n");
    fprintf(op," a2 = %f(%f)\n",anm[0],danm[0]);
    fprintf(op," a4 = %f(%f)\n",anm[1],danm[1]);
    fclose(op);
  }
  if((wdg_arr[k_anis_box] != NULL) && (XtIsManaged(wdg_arr[k_anis_box]))) {
    sprintf(chstr,"%8.5f(%.5f)",eta,deta);
    XmTextFieldSetString(wdg_arr[k_eta],chstr);
    sprintf(chstr,"%7.3f(%.3f)",slope, dslope);
    XmTextFieldSetString(wdg_arr[k_slope],chstr);
  }
  return(0);
}

