/*      DETERM and BKGD
 *	bkgd  (2.22.85)
 *
 *	background fit up to a polinomian of the 5. order
 *
 *	simlar to subroutine polfit
 *	Gerfried Kumbartzki
 */
#include <math.h>

#define MOR 6				/* max order of poly +1 */

extern float *pdat;
extern int *pmarks;
extern double bp[6];

static double determ(double mtx[][MOR],int norder)
{
    double det=1.0,fac;
    register int    k,i,j;

    for(k=0;k<norder;k++){
	for(j=k+1;j<norder;j++)
	  mtx[k][j] /= mtx[k][k];
	for(i=k+1;i<norder;i++)
	  for(j=k+1;j<norder;j++)
	    mtx[i][j] -= mtx[i][k] * mtx[k][j];
    }
    fac = pow((double)10,(double)norder);
    for(i=0;i<norder;i++)
      det = det * mtx[i][i]/fac;
    return(det);
}

void bkgd()
{
    register int i,k,l;
    int    ibkdc,ibkd;
    int    mid;
    double d[MOR][MOR],dx;
    double  b[MOR],c[2*MOR-1],x,a;

    ibkd = pmarks[10] + 1;
    for(i=0;i<MOR;i++)
      b[i] = 0;
    for(i=0;i<2*MOR-1;i++)
      c[i] = 0;
    mid = (pmarks[5] + pmarks[0]) >> 1;
    ibkdc = ibkd + ibkd - 1;

    for(k=0;k<5;k+=4){
	for(i=pmarks[k];i<=pmarks[k+1];i++){
	    a = i-mid;
	    x = pdat[i];
	    for(l=0;l<ibkd;l++){
		b[l] += x;
		x *= a;
	    }
	    x = 1.0;
	    for(l=0;l<ibkdc;l++){
		c[l] += x;
		x *= a;
	    }
	}
    }
    for(i=0;i<ibkd;i++)
      for(l=0;l<ibkd;l++)
	d[i][l] = c[i+l];

    dx = determ (d, ibkd);

    for(k=0;k<ibkd;k++){
	for(i=0;i<ibkd;i++)
	  for(l=0;l<ibkd;l++){
	      if(k==l)
		d[i][l] = b[i];
	      else	d[i][l] = c[i+l];
	  }
	bp[k] = determ(d,ibkd)/dx;
    }
}
/*
 *	background fit: calculates only bp[0] 
 *	for udsa
 */
void bkgd1()
{
  double   sum,smt,a;
  int      ibkd,m,mid;
  register int i,k,l;


  ibkd = pmarks[10] + 1;
  mid = (pmarks[5] + pmarks[0]) >> 1;

  sum = 0;
  m = 0;
  smt = 0;
  for(k=0;k<5;k+=4){
    for(i=pmarks[k];i<=pmarks[k+1];i++){
      m++;		/* total numb of points */
      a = i-mid;
      sum += pdat[i];
      for(l=1;l<ibkd;l++)
	smt += bp[l]*pow(a,(double)l);
    }
  }
  bp[0] = (sum - smt)/m;
}
