/* 
 *	file calib.c also contain calibration
 */
#define  TRUE  1
#define  FALSE 0

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "dacf.h"
#include <Xm/Command.h>
#include <Xm/Text.h>

#define    k_com_win               6

extern int  disp();
extern int  getid(int);

static char cll[] = 
          {"\nCALIBRATION:  centroid     energy     kev/ch      E0\n"};

extern struct hinfo *spc[], *hptr;
extern Widget   wdg_arr[];		/* all other widgets */

extern char   sadatfile[];
extern char  *wptr[];
extern float *pdat;
extern int    ntrs;
extern int    nhist;
extern int    repeat;
extern float  en[], cen[];	/* calibration points */
extern int    calmode;		/* used in inout */

float cent(int markn)		/* takes channel - returns centroid */
{
    float         anst,temp;
    float         y[3],x[3];

    if( (pdat[markn] == 0) || (pdat[markn-1] > pdat[markn] || 
			       pdat[markn+1] > pdat[markn]))
      return ((float)markn);

    y[0] = pdat[markn];		/* has to be max */
    y[1] = pdat[markn-1];
    y[2] = pdat[markn+1];	/* has to be min */

    x[0] = markn;
    x[1] = markn-1;
    x[2] = markn+1;

    if(y[2] > y[1]){		/* interchange */
	temp = y[1];
	y[1] = y[2];
	y[2] = temp;

	temp =x[1];
	x[1] = x[2];
	x[2] = temp;
    }
    if(y[0] - y[2] < 0.001)
      return ((float)markn);
    anst = (x[0]-x[2])/(y[0]-y[2]);
    return((x[1]+x[0]+(y[1]-y[0])*anst)*0.5);
}


int dalbrt(float enn)	/* takes energy - returns channel  */
{
  int chan;

  chan = hptr->cflag ?  (int)(enn - hptr->zr)/hptr->sl + 0.5 : (int) enn;
  return(chan);
}


float calbrt(double cem)	/* takes channel - returns energy */
{
  return(hptr->sl*(float)cem + hptr->zr);
}

int calibrate () 
{
  void docal();
  register int i;
  int     j, in;
  float   sl,zr;
  FILE    *fp;

    switch (ntrs) {

      case 0:		/* reset to channel numbers */
	hptr->cflag = FALSE;
	hptr->sl = 1.0;
	hptr->zr = 0.0;
	break;

      case 1:
	if(strcmp(wptr[1],"all") == 0) { /* reset all */
	  for (i=0; i<nhist; i++) {
	    spc[i]->cflag = FALSE;
	    spc[i]->sl = 1.0;
	    spc[i]->zr = 0.0;
	  }
	  break;
	}
	/* read from file */
	if ((fp = fopen (wptr[1], "r")) == NULL) {
	    printf ("%s not found\n", wptr[1]);
	    return(-1);
	}
	while (fscanf(fp,"%d %d %f %f\n",&j,&in,&sl,&zr) != EOF) {
	    spc[j]->cflag = TRUE;
	    spc[j]->sl = sl;
	    spc[j]->zr = zr;
	}
	fclose (fp);
	break;

      case 2:		/* read energies, get channels from display */
	en[0] = atof(wptr[1]);
	en[1] = atof(wptr[2]);
	calmode = 1;
	break;

      case 3:		/* get calibration for spectrum */
	if(strcmp(wptr[1],"all") == 0) {
	  sl = atof(wptr[2]);
	  zr = atof(wptr[3]);
	  for (i=0; i<nhist; i++) {
	    spc[i]->cflag = TRUE;
	    spc[i]->sl = sl;
	    spc[i]->zr = zr;
	  }
	  break;
	}
	else {
	  if((in = getid(atoi(wptr[1]))) < 0)
	    return(-1);
	  hptr = spc[in];
	  hptr->cflag = TRUE;
	  hptr->sl = atof(wptr[2]);
	  hptr->zr = atof(wptr[3]);
	}
	break;

      case 4:		/* calculate calibration for current spectrum */
	en[0] = atof(wptr[1]);
	en[1] = atof(wptr[2]);
	cen[0] = atof(wptr[3]);
	cen[1] = atof(wptr[4]);
	docal();
	break;

      default: 
	break;
    }
    return(0);
}

void docal()
{
  FILE  *op;

    hptr->sl = (en[1] - en[0]) / (cen[1] - cen[0]);
    hptr->zr = en[0] - hptr->sl * cen[0];
    hptr->cflag = TRUE;
    puts (cll);
    printf ("%21.3f %11.3f %20.3f\n", cen[0], en[0], hptr->zr);
    printf ("%21.3f %11.3f %10.3f\n", cen[1], en[1], hptr->sl);
    if ((op = fopen (sadatfile, "a")) != NULL) {
	fputs (cll, op);
	fprintf (op, "%23.3f %11.3f %20.3f\n", cen[0], en[0], hptr->zr);
	fprintf (op, "%23.3f %11.3f %20.3f\n", cen[1], en[1], hptr->sl);
	fclose (op);
	repeat = 1;
    }
    ntrs =0;
    disp();
}

int cpcal()				/* copy calibration of spectrum in
				 to current spectrum */
{
    int in;
    XmString text;


    if (ntrs < 1) {
	text = XmStringCreate("no valid input",XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }
    if((in = atoi(wptr[1])) <= NSPEC) {
	hptr->sl = spc[in]->sl;
	hptr->zr = spc[in]->zr;
	hptr->cflag = TRUE;
	return(0);
    }
    return(1);
}

int wcal()
{
  register int i;
  FILE  *fp;
  char str[32];

  if (ntrs <1) {
    wptr[1] = str;
    strcpy(wptr[1],"sa");
  }
  strcat (wptr[1], ".cal");
  if ((fp = fopen (wptr[1], "w")) == NULL) {
    perror (" ");
    return(1);
  }
  for(i=0;i<nhist;i++)
    if(spc[i]->sl != 1.00)
      fprintf(fp,"%3d %3d %9.3f %9.3f\n",
	      i,spc[i]->id,spc[i]->sl, spc[i]->zr);
  fclose (fp);
  return(0);
}

int rcal()
{
  char buf[80];
  FILE  *fp;
  int   i, id;
  float sl,zr;

  if (ntrs <1) {
    wptr[1] = buf;
    strcpy(wptr[1],"sa");
  }
  strcat (wptr[1], ".cal");
  if ((fp = fopen (wptr[1], "r")) == NULL) {
    perror (" ");
    return(1);
  }
  while (fgets(buf,80,fp) != NULL){
    if (buf[0] == '#')
      continue;
    if(sscanf(buf,"%d %d %f %f",&i,&id,&sl,&zr) != 4){
      printf("line ignored: %s\n", buf);
      continue;
    }
    spc[i]->sl = sl;
    spc[i]->zr = zr;
    spc[i]->cflag = TRUE;
  }
  fclose (fp);
  return(0);
}

int mcal()
{
    register int i, k, j;
    int narr[128];
    double sl,zr;
    XmString text;

    if (ntrs < 1) {
	text = XmStringCreate("no valid input",XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }
    k = 0;
    for (i = 1; i <= ntrs-2; i++) {	   /* fill number array */
      if(strcmp(wptr[i],"-") == 0) {
	do {
	  narr[k] = k ? narr[k-1]+ 1 : 1;
	  if(k>127) break;
	} while (narr[k++] < atoi(wptr[i+1]));
	k--;
	continue;
      }
      narr[k++] = atoi(wptr[i]);
      if(k>128) break;
    }
    sl = atof(wptr[ntrs-1]);
    zr = atof(wptr[ntrs]);

    for(i=0;i<k;i++) {
      j = narr[i];
      if(getid(j) >= 0) {
	spc[j]->cflag = TRUE;
	spc[j]->sl = (float)sl;
	spc[j]->zr = (float)zr;
      }
    }
    return(0);
}
