/*      SA subroutine
 *	module:        intgrt.c
 *	X - version
 */
#include <math.h>
#include <Xm/Text.h>
#include <stdio.h>
#include "dacf.h"

#define k_mess  20

extern float *pdat, *hdat;
extern float *pdr;		/* pointer to randoms spectrum */
extern int   *pmarks;
extern int    mrks;		/* number of marks */
extern int    npeak;
extern int    repeat;		/* print ledger in sa.dat */
extern int    ran;		/* randoms subtraction flag */
extern double bp[];
extern Widget wdg_arr[];
extern float  calbrt(double cem);
extern struct hinfo *hptr;
extern char   sadatfile[];

double pa[8], dpa[8];		/* store peak area and error for anis */
double rfac;


void intgrt(int mode)
{
    register int i, m2, m3;
    static int  oldid = 0;
    double psum, perr;		/* peak sum and errror */
    double centr,cenerr;	/* centroid and error */
    double sumt,terr;		/* total sum and error */
    double dat;
    double dsum;
    double tbd;
    double bd, sum, diff, bgd(double a);
    float *pd;
    char xstr[80];
    FILE *fopen(),*op,*fp;


    psum = 0.0;			   /* peak sum */
    sumt = 0.0;			   /* total counts */
    sum  = 0.0;
    dsum = 0.0;
    tbd  = 0.0;			   /* total back ground */

    
    if(mrks < 2)		   /* nothig should be done */
      return;

    if (pmarks[2] == -1) {	   /* no marker 2 set */
      m2 = pmarks[0];
      m3 = pmarks[1];
    }
    else {
      m2   = pmarks[2];		   /* peak limits */
      m3   = pmarks[3];
    }
    if (m3 < m2) {
      XmTextSetString(wdg_arr[k_mess],"Check marks!");
      return;
    }
      
    if(pmarks[10] == -1) {	   /* b- or no backgd */
      for(i=m2; i <= m3; i++) {
	dat = (double)pdat[i];
	dsum += dat * (double)i;
	sumt += dat;
      }
      psum = sumt;
    }
    else {			   /* normal integration */
      for(i=m2; i <= m3; i++) {
	bd = bgd((double)(i));
	tbd += bd;
	dat = (double)pdat[i] - bd;
	psum += dat;
	dsum += dat * (double)i;
	sumt += (double)pdat[i];
      }
    }
    centr = dsum/psum;		/* center of gravity */
    for(i=m2; i <= m3; i++) {
	diff = (double)i - centr;
	sum += (double)pdat[i] * diff * diff;
    }
    cenerr = sqrt(sum)/psum;	/* error in position */
    terr   = sqrt(sumt);	/* error of total */
    perr   = sqrt(sumt + tbd);	/* errror of peaksum */

    if (ran) {			/* set when 'sran' is used */
      sum = 0;
      for(i=m2;i <= m3; i++)
	sum += (double)pdr[i];
      rfac = (pdr[0] == 0.0) ? 1.0 : (double)pdr[0]; /* scale factor for error */
      perr = sqrt(sumt + tbd + 2.0*sum*rfac); /* error including randoms */
      pa[ran>>1] = psum;
      dpa[ran>>1] = perr;
    }

    if (pmarks[10] == -1)
      sprintf(xstr,"peak#: %d pos: %.2f    sum: %.1f(%.1f)",
	      npeak,calbrt(centr),sumt,perr);
    else
      sprintf(xstr,"peak#: %d pos: %.2f  sum: %.1f(%.1f)  total: %.1f(%.1f)",
	      npeak,calbrt(centr),psum,perr,sumt,terr);
    XmTextSetString(wdg_arr[k_mess],xstr);

    if(mode) {	    /* output to sa.dat and results.dat file */

      printf("%6d pos: %.2f sum: %.1f(%.1f) total: %.1f(%.1f) bp[0]: %.1f\n",
	     hptr->id,calbrt(centr), psum, perr, sumt,terr, bp[0]);
      if(ran) 
        printf("      %.2f randoms subtracted - scale factor %f\n", sum, rfac);

      if((op=fopen(sadatfile,"a")) != NULL) {

	if(hptr->id != oldid) {
	  fprintf(op,"\n%s:%d\n",hptr->title,hptr->id);
	  oldid = hptr->id;
	}

	if(repeat)  		/* repeat headline */
	  fprintf(op,
		  "\n     peak#    energy    centroid     peaksum       total        marker");
	fprintf(op,"\n\n    ");
	fprintf(op,"%4d%12.2f%11.2f%13.1f%11.1f%9d%6d%6d\n", npeak,
		calbrt(centr),centr,psum,sumt,pmarks[0],
		pmarks[1],pmarks[2]);
	fprintf(op,"%31.2f%13.1f%11.1f%9d%6d%6d\n",
		cenerr,perr,terr,pmarks[3],
		pmarks[4],pmarks[5]);
	fprintf(op,"     backgd:  polynom = %d     pars: %7.3f",
		pmarks[10],bp[0]);

	for(i=1;i<pmarks[10]+1;i++)
	  fprintf(op,"  %6.3f",bp[i]);
        fprintf(op,"  randoms: %.2f",sum);
	fprintf(op,"\n");
	repeat = 0;
	fclose(op);
      }

      if ((fp = fopen("results.dat","a")) != NULL){

	if (ran){
	  fprintf(fp,"%6d %10.2f %10.2f\n", hptr->id/2+1,psum,perr);
	}
	else {
	  fprintf(fp,"%6d%10.2f%10.2f%10.2f\n",
		  hptr->id,calbrt(centr),psum,perr);
	}
      }
      fclose(fp);
      ran = 0;			/* reset only after 'i' or 's' */
    }
}
