/* routines to match the gain of two spectra, shift and re-bin spectra
 * new idx is always the first entry
 */
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/Command.h>
#include <time.h>
#include <fcntl.h>
#include <unistd.h>
#include "dacf.h"

#define k_com_win   6
#define k_mess     20

extern  double gmfac, shft;	/* gain match factor and shift */
extern  int    ntrs;
extern  char  *wptr[];
extern  Widget wdg_arr[];
extern  int disp();

int set_gmf () {		/* s_hift, c_ontract spectra */
  double  soll[2], ist[2];
  char xstr[80];

    switch(ntrs) {

      case 1:			/* reset to default */
	if(strncmp(wptr[1],"reset",2)== 0) {
	  gmfac = 1.0;
	  shft = 0.0;
	}
	else			/* shift only */
	  shft = atof (wptr[1]);
	break;

      case 2:			/* new gain match info */
	gmfac = atof (wptr[1]);	
	shft  = atof (wptr[2]);
	break;

      case 4:			/* calc new gain match info */
	soll[0] = atof(wptr[1]);
	soll[1] = atof(wptr[2]);
	ist [0] = atof(wptr[3]);
	ist [1] = atof(wptr[4]);

	gmfac  = (soll[1] - soll[0])/(ist[1] - ist[0]);
        shft   = soll[0] - gmfac*ist[0];
	break;

      default:
	break;
    }
    sprintf(xstr,"gain factor = %4.3f  shift = %4.3f",gmfac,shft);
    XmTextSetString(wdg_arr[k_mess],xstr);
    return(0);
}
/* rebinning take current length and new bin number to calculate gmfac 
   either uses current id or id from input line */

int rebin()
{
  extern  int    getid(int);
  extern  int    kilo(char*);
  extern  int    idx;
  extern  float  *hdat, *pdat;
  extern  struct hinfo *spc[], *hptr;

  int    i,j;
  int    len;
  float  nbin;
  int    nshft;
  float  sum, rest;
  double ashft,bshft;
  float  tmp, fac, nfac;
  float  *pd;
  float  n;
  char   xstr[80];
  XmString text;


  if(ntrs)			/* new number of bin */
    nbin = kilo(wptr[1]);
  if(ntrs > 1) {		/* input id always first */
    if((idx = getid(atoi(wptr[1]))) < 0) {
      sprintf(xstr,"%s unknown id",wptr[2]);
      text = XmStringCreate(xstr,XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(-1);
    }
    nbin = kilo(wptr[2]);
  }
  hptr = spc[idx];
  pdat = hdat + hptr->offset; /* original data */
  len  = hptr->xma;
  gmfac = (ntrs>0) ? len/nbin : gmfac;
  fac = gmfac;
  pd = hdat + spc[0]->offset;	 /* destination work area */

  if (fabs(fac-1.0) < 0.0001) {	/* nothing should be done */
    for(i=0;i<len;i++)
      pd[i] = pdat[i];
  }
  else if(fac > 1.0) {		/* shrink spectrum */
    sum=0;
    tmp=0;
    i=j=0;

    while(i < len){
      do{
	if(fac > 1.0) fac = 1.0;
	nfac = 1.0 - fac;
	sum += fac * pdat[i++];	/* collect new bin content */
	tmp += fac;
      } while((fac = fabs(tmp-gmfac)) > 0.0001);

      pd[j] = (int)sum;
      sum -= pd[j];
      j++;
      tmp = 0.0;
      fac = nfac;
      if(fabs(nfac) < 0.0001) {
	fac = gmfac;
	continue;
      }
      i--;
    }
  }
  else if (fac < 1.0) {		/* expand spectrum */
    if(idx == 0) {		/* would over write needed data */
      sprintf(xstr,"need id other than 0");
      text = XmStringCreate(xstr,XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(-1);
    }
    rest = 0.0;
    n = 1.0;
    for (j=0,i=0;i<len;i++) {
      tmp = n * fac - (n - 1.0);
      if (tmp < 0.0001) {
	pd[j++] = rest;
	n = 1.0;
	rest = 0.0;
	i--;
	continue;
      }
      pd[j++] = pdat[i]*tmp + rest;
      rest = pdat[i]*(1.0 - tmp);
      n += 1.0;
    }
  }
  spc[0]->xma = j-1;		/* new length */
  printf("j=%d\n",j);
  strcpy(spc[0]->title,"w-");
  if (idx != 0)
    strncat(spc[0]->title,spc[idx]->title,13);
  ntrs = 1;
  strcpy(wptr[1],"0");
  disp();
  return(0);
}

int shift()			/* shift spectrum */
{				/* if result should be in work area */
                                /* put first data in work area */
  extern  int    getid(int);
  extern  int    idx;
  extern  float *hdat, *pdat;
  extern  struct hinfo *spc[];

  int    i,j;
  int    len;
  int    nshft;
  float  sum, rest;
  double ashft,bshft;
  float  *pd;
  float  n;
  char   xstr[80];
  XmString text;


  if (ntrs) {
    shft = atof(wptr[1]);
  }
  if(ntrs>1) {
    if((idx = getid(atoi(wptr[1]))) < 0) {
      sprintf(xstr,"%s unknown id",wptr[2]);
      text = XmStringCreate(xstr,XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(-1);
    }
    shft = atof(wptr[2]);
  }
  pd = hdat + spc[idx]->offset;	/* always ORIGINAL data are shifted */
  len  = spc[idx]->xma;

  if(shft != 0.0) {
    nshft = floor(shft);
    if(shft > 0) nshft++;

    if(nshft != 0) {
      for(i=0;i<=len;i++) {
	j = (nshft >= 1) ? len - i : i;
	if((j-nshft) < 1) continue;
	pd[j] = pd[j-nshft];
      }
    }

    if((ashft = (shft - (double)nshft)) < 0.0) ashft *= (-1.0);
    bshft = 1.0 - ashft;
    if(ashft > 0.0001) {	/* round to integer values */
      rest = 0.0;
      for(i=0;i<=len;i++) {
	sum = pd[i]*bshft + pd[i+1]*ashft+rest;
	pd[i] = (int)sum;
	rest = sum - pd[i];
      }
    }
  }
  disp();
  return(0);
}

int g_match()
{
  extern  int    getid(int);
  extern  int    idx;
  extern  float  *hdat, *pdat;
  extern  struct hinfo *spc[];

  int    i,j,k;
  int    len;
  double kf;
  double tmp, fac;
  float  *pd;
  char   xstr[80];
  XmString text;

  switch (ntrs) {
  case 0:			/* use default idx, gmfac, shft */
    break;
  case 1:
  case 2:
  case 3:
    if((idx = getid(atoi(wptr[1]))) < 0) {
      sprintf(xstr,"%s unknown id",wptr[1]);
      text = XmStringCreate(xstr,XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(-1);
    }
    if(ntrs == 2)
      gmfac = atof (wptr[2]);
    if(ntrs == 3)
      shft  = atof (wptr[3]);
    break;
  default:
    break;
  }
  if(idx == 0) {
    sprintf(xstr,"need id other than 0");
    text = XmStringCreate(xstr,XmSTRING_DEFAULT_CHARSET);
    XmCommandError(wdg_arr[k_com_win],text);
    XmStringFree(text);
    return(-1);
  }
  if((fabs(gmfac - 1.0) < 0.001) && shft < 0.001) {
    sprintf(xstr,"nothing to be done");
    text = XmStringCreate(xstr,XmSTRING_DEFAULT_CHARSET);
    XmCommandError(wdg_arr[k_com_win],text);
    XmStringFree(text);
    return(-1);
  }
  pdat = hdat + spc[idx]->offset;
  len  = spc[idx]->xma;

  fac = gmfac;
  pd = hdat + spc[0]->offset;	 /* new data in work area */
  for(i=0;i<spc[0]->xma;i++)
    pd[i]=0.0;
  srand((int)pdat[101]);
  shft -= 0.5;
  for(i=0;i<len;i++) {
    tmp = i*fac + shft;

    for(j=0;j<(int)pdat[i];j++) {
      kf = rand()/(double)RAND_MAX;
      k = (int)(tmp + kf);
      if(k && k<spc[0]->xma)
	pd[k]++;
    }
  }
  spc[0]->xma = k;
  strcpy(spc[0]->title,"w-");
  if (idx != 0)
    strncat(spc[0]->title,spc[idx]->title,13);
  ntrs = 1;
  strcpy(wptr[1],"0");
  disp();
  return(0);
}

int gmfile()			/* used to get gain match info for
				 16 clover segments used in evb */
{
  register int i;
  extern  int calmode, idx;
  extern  float cen[];
  extern  struct hinfo *spc[];
  extern  int tcnt;
  static int entries;		/* usual = idx */
  static int setentry = 0;	/* entry flag */
  char mesg[80];
  XmString text;
  static double val1, val2;	/* reference values 1 and 2 */
  static double pos[16][2];	/* two energies for each of the 16 det. */
  double ma[16],mb[16];
  time_t t;
  int fd;

  switch(ntrs) {
  case 0:			/* turn pp mode on */
    if(calmode == 2) {
      calmode = 0;
      sprintf(mesg,"pp mode off");
      text = XmStringCreate(mesg,XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
    }
    else {
      entries = idx;
      tcnt = 0;
      calmode = 2;
      sprintf(mesg,"pp mode on id = %d",entries);
      text = XmStringCreate(mesg,XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
    }
    return(0);
    break;

  case 1:			/* specify det or "out" = 0 for output */
    entries = atoi(wptr[1]);
    if (entries < 0 || entries > 16) {
      sprintf(mesg,"%d det 1 - 16 only",entries);
      text = XmStringCreate(mesg,XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(-1);
    }
    if(entries == 0) {		/* write out */
      t = time(0);
      strftime(mesg, sizeof(mesg),"%m%d%y_%H%M.gmf",localtime(&t));

      for (i=0;i<16;i++) {
	ma[i] = (val2 - val1) / (pos[i][1] - pos[i][0]);
	mb[i] = val1 - ma[i] * pos[i][0];
	printf("%2d %8.4f %8.4f\n",i, ma[i],mb[i]);
      }
      if((fd = creat(mesg,0644)) > 3) {
	write(fd,&ma,sizeof(ma));
	write(fd,&mb,sizeof(mb));
	close(fd);
      }
      calmode = 0;		/* after write reset mode */
      return(0);
    }
    setentry = 1;		/* provided entry is used */
    tcnt = 0;
    calmode = 2;
    return(0);
    break;
    
  case 2:			/* enter set values */
    val1 = atof(wptr[1]);
    val2 = atof(wptr[2]);
    tcnt = 0;
    calmode = 2;		/* enter calmode right away */
    return(0);
    break;

  case 3:
    entries = setentry ? entries : idx;
    if (entries > 0 && entries < 17) {
      pos[entries-1][0] = cen[0];
      pos[entries-1][1] = cen[1];
      printf("entry %d: %f %f\n",entries,pos[entries-1][0], pos[entries-1][1]);
      setentry = 0;
    }
    break;

  default:
    return(-1);
    break;
  }
  return(0);
}
