/*
$Header: /usr/local/cvsrep/xsa/new_ful.c,v 1.1.1.1 2001/09/25 19:07:52 kum Exp $
*/
#include <math.h>
#include "benlib.h"
#include <stdio.h>
#include "standardize.h"

double          narrow_chi_squared(), modified_chi_squared();
double          fabs(), sqrt();

/*========================================================================*/

 /*** FUL stands for "flat upper limit"
      Defined as the place where modified chi-square = 1/2
      Modified chi-square is defined as a function of k:
          (1/n)*sum ((k-data)/sqrt(k))**2
      where the sum is only over points with data<k.
      Use a binary search algorithm.
      For initial lower limit,  take the lowest data-point
      For initial upper limit,  take the value of ful where
      the contribution of the lowest data-point to the modified
      chi-square gives 1/2 all by itself. ***/


/*
The FUL2 statistic is defined as the average of all data-points
within s*sqrt(FUL1) of FUL1 (**** see modification, 12/10/92,  below ****),
where s is chosen such that the narrow chi-squared is
equal to one.  The narrow chi-squared is defined as the sum of d(i)**2,  where
d(i) is equal to |data(i)-FUL1| or |s*sqrt(FUL1)|,  whichever is lower.  The FUL2 is
supposed to work better than the FUL1 for sparse data,  since the FUL1 is systematically
too high for sparse data.
*/

/*
Modified 12/10/93 by B. Crowell:  There was a problem with the above definition:
Sometimes none of the data-points qualify to be included in the average that makes
up the FUL2.  Changed definition of FUL2 to include in the average *all* the data-
points *below* the FUL1,  in addition to those that were included in the original
definition.
*/

/*
Modified 1/18/94 by B. Crowell: Added FUL3.  This is the same as FUL2,  except
that instead starting with
	mean(est) = FUL1
	sigma(est) = sqrt(FUL1)
we use
	mean(est) = FUL1
	sigma(est) = sqrt(FUL1-bias)
where bias corrects approximately for the bias in the FUL1 algorithm.  With
FUL2,  I'd found that relative errors could be as big as ~5% for ~1 count/bin.
To make sure that some bins are actually included in the final sum,  the final
sum is over all bins with y(i)<y(max),  where
		y(max) = max(mean(est)+s*sigma(est),FUL1)
But actually,  on further thought,  it is not mathematically possible to have
mean(est)+s*sigma(est) < FUL1,  since we always have s>=1 and sigma(est)>=0.7,
and mean(est)>=FUL1-0.5.
Also,  merged all FUL stuff into this single file.  The file ful.c is now saved
as ful.c.save,  but should never be referenced in makefiles.
Tested FUL3 on a sample gator file,  difference between FUL2 and FUL3 is very
small for both gated and ungated 190Hg cubes.
****OOPS!****
The bias I was trying to correct for was slightly NEGATIVE!
So FUL3 as described above is slightly worse!
Changed def of FUL3 to
	mean(est) = FUL1
	sigma(est) = 2*sqrt(FUL)
This seems to work well,  but need more extensive numerical tests to be
sure.
OK,  did the following test on this new def of FUL3:
	Made spectra -- some difference in bgsub,  not much,  from ungated
	or gated cube.
	Did Monte Carlo -- with a flat Poisson background,  FUL3 ends up
	averaging all of the data,  so it is exactly the same as averaging
	all the bins.  This is fine.
Redefined do_new_ful so default is FUL3.
*/

#define DEBUG_FUL 0
 /* ...for FUL1 */
#define DEBUG 0
 /* ...for FUL2, FUL3 */

/*========================================================================*/

 /* The following routine is what should be called by programs. It will
  * always be the newest version of the FUL that has been tested. */
do_new_ful(new_ful, data, n)
  float          *new_ful;
  float          *data;
  int             n;
{
  do_ful_any_version(new_ful, data, n, 3);
}

do_ful1(new_ful, data, n)
  float          *new_ful;
  float          *data;
  int             n;
{
  do_ful_any_version(new_ful, data, n, 1);
}

do_ful2(new_ful, data, n)
  float          *new_ful;
  float          *data;
  int             n;
{
  do_ful_any_version(new_ful, data, n, 2);
}

do_ful3(new_ful, data, n)
  float          *new_ful;
  float          *data;
  int             n;
{
  do_ful_any_version(new_ful, data, n, 3);
}


/*========================================================================*/

do_ful_any_version(new_ful, data, n, version)
  float          *new_ful;
  float          *data;
  int             n;
  int             version;	/* = 1, 2 or 3 */
{
  float           old_ful;
  double          lo_s, hi_s, z, lo_ncs, hi_ncs, interp_s, interp_ncs,
                  y_max;
  double          est_mean, est_fluct;
  int             i, nn, niter;
  do_ful(&old_ful, data, n);
  if (DEBUG)
    printf("Old ful= %10.5lf\n", old_ful);
  if (old_ful < .5)
    old_ful = .5;
  switch (version)
  {
  case 1:
    *new_ful = old_ful;
    return;
  case 2:
    est_mean = old_ful;
    est_fluct = sqrt(old_ful);
    break;
  case 3:
    est_mean = old_ful;
    est_fluct = 2 * sqrt(old_ful);
    break;
  }
  lo_s = 0.;
  z = 0;
  for (i = 0; i <= n - 1; i++)
  {
    if (fabs(data[i] - old_ful) > z)
      z = fabs(data[i] - est_mean);
  }
  hi_s = z / est_fluct + 1e-3;	/* Note that old_ful is always at least 0.5,
				 * so there is no danger of division by zero. */
  lo_ncs = narrow_chi_squared(data, n, est_mean, lo_s * est_fluct, est_fluct);
  hi_ncs = narrow_chi_squared(data, n, est_mean, hi_s * est_fluct, est_fluct);
  if (hi_ncs < 1.01)
  {
    interp_s = hi_s;
    interp_ncs = hi_ncs;
  } else
  {
    niter = 0;
    while (hi_ncs - lo_ncs > .01)
    {
      if (DEBUG)
	printf("Iteration %d, s=%10.5lf %10.5lf,  ncs=%10.5lf %10.5lf\n",
	       niter, lo_s, hi_s, lo_ncs, hi_ncs);
      /* On each iteration,  first look at the point that should work by
       * linear interpolation,  then at the half-way point: */
      interp_s = lo_s + (hi_s - lo_s) * (1. - lo_ncs) / (hi_ncs - lo_ncs);
      interp_ncs = narrow_chi_squared(data, n, est_mean, interp_s * est_fluct, est_fluct);
      if (interp_ncs > 1.)
      {
	hi_s = interp_s;
	hi_ncs = interp_ncs;
      } else
      {
	lo_s = interp_s;
	lo_ncs = interp_ncs;
      }
      interp_s = .5 * (lo_s + hi_s);
      interp_ncs = narrow_chi_squared(data, n, est_mean, interp_s * est_fluct, est_fluct);
      if (interp_ncs > 1.)
      {
	hi_s = interp_s;
	hi_ncs = interp_ncs;
      } else
      {
	lo_s = interp_s;
	lo_ncs = interp_ncs;
      }
      ++niter;
    }
  }
  if (DEBUG)
    printf("Determined s=%10.5lf, ncs=%10.5lf, max diff=%10.5lf\n",
	   interp_s, interp_ncs, interp_s * est_fluct);
  /* Now do the final averaging step: */
  y_max = est_mean + interp_s * est_fluct;
  if (y_max < old_ful)
    y_max = old_ful;
  *new_ful = 0.;
  nn = 0;
  for (i = 0; i <= n - 1; i++)
  {
    if (data[i] <= y_max)
    {
      *new_ful += data[i];
      ++nn;
    }
  }
  *new_ful = *new_ful / nn;
  if (DEBUG)
    printf("new_ful = %10.5lf, pts included=%d\n",
	   *new_ful, (int) nn);
  if (IS_INVALID_NUMBER(*new_ful))
  {
    printf("Error in new FUL: result is invalid floating-point number\n");
    {
      int             i;
      for (i = 0; i <= n - 1; i++)
      {
	printf("data[%d] = %10.3lf\n", i, data[i]);
      }
      printf("FUL version 1 = %10.3lf\n", old_ful);
      printf("new FUL = %10.3lf\n", *new_ful);
      printf("s=%10.5lf, ncs=%10.5lf, max diff=%10.5lf\n",
	     interp_s, interp_ncs, interp_s * est_mean);

    }
    exit(-1);
  }
}

/*========================================================================*/

double
narrow_chi_squared(data, n, k, max_diff, est_fluct)
  float          *data;
  int             n;
  double          k, max_diff, est_fluct;
{
  int             i;
  double          u, d;
  u = 0;
  for (i = 0; i <= n - 1; i++)
  {
    d = fabs(data[i] - k);
    if (d > max_diff)
      d = max_diff;
    u = u + d * d;
  }
  return u / (n * est_fluct * est_fluct);
}

/*========================================================================*/

do_ful(ful, data, n)
  float          *ful;
  float          *data;
  int             n;
{
  int             i;
  float           lowest, lo_ful, hi_ful, lo_mc, hi_mc, interp, x_range,
                  new_mc, max_allowed_err;
  /* Find lowest data-point: */
  lowest = data[0];
  for (i = 1; i <= n - 1; i++)
  {
    if (data[i] < lowest)
      lowest = data[i];
  }

  /* Initial upper and lower limits: */

  lo_ful = lowest;
  hi_ful = lowest + sqrt(n * lowest / 2.);
  if (hi_ful < .5)
    hi_ful = .5;
  while ((1. / n) * square(hi_ful - lowest) / fabs(hi_ful) < .5)
    hi_ful = 2 * (fabs(hi_ful) + 1);
  if (lo_ful <= 0.)
    lo_ful = 1.e-10;

  /* Binary search for the correct value of ful What is acceptable accuracy?
   * Well,  if we have n data-points and statistics/bin is about =ful,  then
   * error bars on ful should be about sqrt(ful)/sqrt(n).  We will demand .1
   * times this error,  or a relative error of .01,  whichever is lower. */

  lo_mc = modified_chi_squared(lo_ful, data, n);
  hi_mc = modified_chi_squared(hi_ful, data, n);
  for (;;)
  {
    /* Do linear interpolation to look for the best guess as to where the
     * solution is,  but if linear interpolation yields something within 10%
     * of the ends of the interval,  don't trust it, because we might get
     * stuck. */

    if (lo_mc > .5)
    {
      interp = lo_mc;
      break;
    }

    x_range = hi_ful - lo_ful;
    interp = lo_ful + (.5 - lo_mc) * x_range / (hi_mc - lo_mc);
    if (interp > hi_ful - .1 * x_range)
      interp = hi_ful - .1 * x_range;
    if (interp < lo_ful + .1 * x_range)
      interp = lo_ful + .1 * x_range;
    new_mc = modified_chi_squared(interp, data, n);
    if (DEBUG_FUL)
    {
      printf("low ful=%f mc=%f\n", lo_ful, lo_mc);
      printf("    ful=%f mc=%f\n", interp, new_mc);
      printf("hi  ful=%f mc=%f\n", hi_ful, hi_mc);
      printf("-------------------------------------\n");
    }
    max_allowed_err = .1 * sqrt(interp / n);
    if (max_allowed_err > .01 * interp)
      max_allowed_err = .01 * interp;
    if (x_range < max_allowed_err)
      break;
    if (new_mc > .5)
    {
      hi_ful = interp;
      hi_mc = new_mc;
    } else
    {
      lo_ful = interp;
      lo_mc = new_mc;
    }
  }
  *ful = interp;
}

/*========================================================================*/

double
modified_chi_squared(flat_value, data, n)
  double          flat_value;
  float          *data;
  int             n;
{
  int             i;
  float           mc;
  mc = 0;
  for (i = 0; i <= n - 1; i++)
  {
    if (data[i] < flat_value)
      mc = mc + square(flat_value - data[i]);
  }
  mc = mc / (n * flat_value);
  return mc;
}
