#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "dacf.h"
#include <X11/Xlib.h>
#include <Xm/Command.h>

				/* in portrait layout */
#define xL 2*72			/* x left corner */
#define xR 8*72			/* x right corner */
#define yB 2*72			/* y bottom */
#define yT 10*72		/* y top */
#define sfac 0.020		/* 2.5 % scale factor for psize */

#define tcH  10*72		/* two column height = 10" */
#define tcW  216		/* two column width  */

#define k_com_win               6

static char *pcolor[8] = { "0.0 0.0 0.0 ","1.0 0.0 0.0 ",
			   "0.0 1.0 0.0 ","0.0 0.0 1.0 ",
                           "0.5 0.3 0.0 ","0.0 0.5 0.5 ",
                           "0.3 0.5 0.3 ","1.0 0.5 0.0 "};

float ysc, xsc;
float ymx, xmx;
float xb, yb, ybb;
float psize;

int   lsize;			/* number of layers == ntrs */
int   yoff;
int   nomax;

FILE *fopen(), *fps, *fpp;

extern Widget wdg_arr[];		/* all other widgets */
extern float  yscale(float max,float min,float hght);
extern float  mkround(float in);
extern void   ylimits(int beg,int end);
extern int getid(int);

extern int    nhist;
extern float *hdat;
extern float *pdat;
extern float  deca;
extern float  ymin, ymax;
extern float  ysmax[],ysmin[];
extern int    yzero;
extern int    fchan;
extern int    dsr, dsl;		/* display range */
extern int    len;
extern int    ntrs;
extern int    sntrs;
extern int    numbs[];
extern int    yfix;
extern int    do_index;
extern int    dsl, dsr;
extern int    mlog;
extern int    mrks,marks[];
extern char  *wptr[];
extern char   strPS[];
extern char   filNam[];
extern char   psname[];
extern char   cdirec[];
extern struct hinfo *spc[], *hptr;
extern int dalbrt(float a);

void pslabel(),ps1label(),pklabel();
void wr_txt(char *str);			/* sup scripting */


int psfile(int option)

				/* 0:  normal mode */
				/* 1:  overlay mode */
				/* 2:  several spectra separated */
				/* 3:  two column spectra separated */
{

    register int i, k, j;

    int rflag;
    int separator;

    char buf[80];
    void psaxis(), psplot();

    rflag = 0;
    nomax = 0;
    ymax = 0.0;

    fps = NULL;

    if(filNam[0] != '\0')       /* file name provided in command line */
      strcpy(buf,filNam);
    else
      strcpy(buf,"sagraph.ps");	/* default filename */

    if((fps = fopen(buf, "w")) == NULL) {
      fprintf(stderr,"cannot open ps output file %s\n",buf);
      return(1);
    }
				/* add preample */
    if((fpp = fopen(psname, "r")) == NULL) {
	fprintf(stderr,"cannot open ps preamble file\n");
	return(-1);
    }
    while(fgets(buf,sizeof (buf),fpp) != NULL) /* copy preamble */
      fputs(buf,fps);
    fclose(fpp);

    if(ntrs<1) {		/* information from screen */
      ntrs = sntrs;
      rflag = 1;
      ymax = ysmax[0];	/* screen values */
      ymin = ysmin[0];
      if(do_index > 0) { 	/* overlay mode */
	ntrs = (do_index + 1 < 10) ? do_index + 1 : 8;
	option = 1;
      }
    }
    else {
      yfix = 0;
      k = 0;
      if(ntrs > 10) ntrs = 10;
      for (i = 1; i <= ntrs; i++) {	   /* fill number array */
	if(strcmp(wptr[i],"-") == 0) {
	  do {
	    numbs[k] = k ? numbs[k-1]+ 1 : 1;
	    if(k > 16) break;
	  } while (numbs[k++] < atoi(wptr[i+1]));
	  k--;
	  continue;
	}
	numbs[k++] = atoi(wptr[i]);
      }
      ntrs = k;
      if(ntrs <= 0) return(-1);		   /* no valid entry */

      if (numbs[k-1] < 0) {		   /* last entry treated as YMAX */
	ntrs--;
	if (numbs[k-1] == -1 && option == 1)
	  nomax = 1;
	else {
	  ymax =  -numbs[k-1];
	  yfix = 1;
	}
      }
      for(i=0;i<ntrs;i++) {		   /* check for validity */
	if(getid(numbs[i])<0) {
	  for(j=i;j<ntrs;j++)
	    numbs[j] = numbs[j+1];
	  ntrs--;
	}
      }
      if (!ntrs) {
	XmCommandError(wdg_arr[k_com_win],
		       XmStringCreate("? input",XmSTRING_DEFAULT_CHARSET));
	return(-1);
      }
    }

    switch(option) {

    case 0:			/* normal mode */

      if(ntrs == 1) {		/* LANDSCAPE:special case if one spectrum */

	ymx = 432.0;		/* YMAX 6 inch */
	xmx = 572.0;		/* XMAX 8 inch */
	psize = ymx*sfac;	/* 2% of height */

	hptr = spc[getid(numbs[0])];
	pdat = hdat + hptr->offset;

	if(!yfix)
	  ylimits(fchan,len);
	ysc = yscale(ymax,ymin,ymx);

	fprintf(fps,"/w %.2f def\n",ymx);    /* here xmx and */
	fprintf(fps,"/h %.2f def\n",xmx);    /* ymx interchanged */
	fprintf(fps,"newpath\n");
	fprintf(fps,"1.5 inch 2.0 inch box\n");
	fprintf(fps,"stroke\n");
	fprintf(fps,"gs\n");
	fprintf(fps,"1.45 inch 2.5 inch translate\n");
	fprintf(fps,"10 deffont\n");
	fprintf(fps,"90 rotate\n");
	fprintf(fps,"(%s:%d) rightshow\n",hptr->title,hptr->id);
	fprintf(fps,"gr\n");
	fprintf(fps,"7.5 inch 2.0 inch translate\n");
	fprintf(fps,"90 rotate\n");
	fprintf(fps,"16 deffont\n");
	psaxis(0);		/* draw channel numbers */
	psplot();
	pklabel();
	ps1label();
      }
      else {		  /* PORTRAIT: multiple spectra */
	if (ntrs > 8)
	  ntrs = 8;		/* not more than 8 */
	xmx = xR - xL;
	ymx = yT - yB + 72.0;	/* added 1 inch, no separations */
	ymx /= ntrs;
	psize = ymx*sfac;
	lsize = ntrs;	/* decrease with number of spectra */
	fprintf(fps,"/w %.2f def\n",xmx);
	fprintf(fps,"/h %.2f def\n",ymx);
	fprintf(fps,"gs\n");

	for (i = 0; i < ntrs; i++) {

	  hptr = spc[getid(numbs[i])];
	  pdat = hdat + hptr->offset;
	  if(i == 0) {	/* all should be the same length */
	    fchan = (dalbrt((float)dsr) <= 0) ? 0 : dalbrt((float)dsr);
	    len = dsl ? dalbrt((float)dsl) : hptr->xma;
	  }
	  if(!yfix)
	    ylimits(fchan,len);
	  ysc = yscale(ymax,ymin,ymx);
	  yb = yT - (i+1)*ymx;       /* start at top go down */
	  fprintf(fps,"newpath\n");
	  fprintf(fps,"1.5 inch %.2f box\n",yb);
	  fprintf(fps,"stroke\n");
	  fprintf(fps,"0 inch %.2f translate\n",yb+ymx-10.0);
	  fprintf(fps,"10 deffont\n");
	  fprintf(fps,"(%s:%d) rightshow\n",hptr->title,hptr->id);
	  fprintf(fps,"gr\n");
	  fprintf(fps,"gs\n");
	  fprintf(fps,"1.5 inch %.2f translate\n",yb); /* set new origin */
	  fprintf(fps,"%d deffont\n", 17-lsize);
	  psaxis(ntrs-1-i);
	  psplot();
	  pklabel(); 
	  fprintf(fps,"gr\n");
	  fprintf(fps,"gs\n");
 	}
	pslabel();
      }
      break;

    case 1:			/* overlay mode */
				/* only LANDSCAPE */

      ymx = 432.0;		/* YMAX 6 inch */
      xmx = 572.0;		/* XMAX 8 inch */
      psize = ymx*sfac;	        /* 2% of height */

      fprintf(fps,"/w %.2f def\n",ymx);    /* here xmx and */
      fprintf(fps,"/h %.2f def\n",xmx);    /* ymx interchanged */
      fprintf(fps,"newpath\n");
      fprintf(fps,"1.5 inch 2.0 inch box\n");
      fprintf(fps,"stroke\n");
      fprintf(fps,"gs\n");
      fprintf(fps,"1.45 inch 2.5 inch translate\n");
      fprintf(fps,"10 deffont\n");
      fprintf(fps,"90 rotate\n");
      fprintf(fps,"(%s:%d) rightshow\n",hptr->title,hptr->id);
      fprintf(fps,"gr\n");

      fprintf(fps,"7.5 inch 2.0 inch translate\n");
      fprintf(fps,"90 rotate\n");

      for(i=0;i<ntrs;i++) {
	hptr = spc[getid(numbs[i])];
	pdat = hdat + hptr->offset;
	if(rflag) {		/* data taken from sreen */
	  ymax = ysmax[i];
	  ymin = ysmin[i];
	}
	if(i == 0) {		/* scale done only once */
	  if (!yfix)
	    ylimits(fchan,len);
	  ysc = yscale(ymax,ymin,ymx);
	  fprintf(fps,"16 deffont\n");
	  psaxis(0);
	  pklabel();
	}
	else if(nomax) {	/* autoscaling */
	  ylimits(fchan,len);
	}
	ysc = yscale(ymax,ymin,ymx);
	fprintf(fps,"%s setrgbcolor\n",pcolor[i]);
	psplot();
      }
      fprintf(fps,"%s setrgbcolor\n",pcolor[0]);
      ps1label();
      break;

    case 2:		/* multiple, separated spectra  */

      if (ntrs > 8)
	ntrs = 8;			   /* not more than 8 */
      separator = 1.5*72.0/ntrs;   /* 1.5 inch for separations */
      xmx = xR - xL;
      ymx = yT - yB;		/* 5 inch */
      ymx /= ntrs;
      psize = ymx*sfac;
      lsize = ntrs;
      fprintf(fps,"/w %.2f def\n",xmx);
      fprintf(fps,"/h %.2f def\n",ymx);
      fprintf(fps,"gs\n");

      for (i = 0; i < ntrs; i++) {

	hptr = spc[getid(numbs[i])];
	pdat = hdat + hptr->offset;
	fchan = (dalbrt((float)dsr) <= 0) ? 0 : dalbrt((float)dsr);
	len = dsl ? dalbrt((float)dsl) : hptr->xma;

	if(!yfix) 
	  ylimits(fchan,len);
	ysc = yscale(ymax,ymin,ymx);
	ybb = yT - (i+1)*(ymx+separator);     /* start at top go down */
	yb = ybb + separator;                 /* for labels */
	fprintf(fps,"newpath\n");
	fprintf(fps,"1.5 inch %.2f box\n",yb);
	fprintf(fps,"stroke\n");
	fprintf(fps,"0 inch %.2f translate\n",yb+ymx-10.0);
	fprintf(fps,"10 deffont\n");
	fprintf(fps,"(%s:%d) rightshow\n",hptr->title,hptr->id);
	fprintf(fps,"gr\n");
	fprintf(fps,"gs\n");
	fprintf(fps,"1.5 inch %.2f translate\n",yb); /* set new origin */
	fprintf(fps,"%d deffont\n", 17-lsize); /* reduce fontsize */
	psaxis(0);
	psplot();
	pklabel(); 
	fprintf(fps,"gr\n");
	fprintf(fps,"gs\n");
      }
      pslabel();
      break;

    case 3:		/* multiple spectra in 2 column */

      xmx = tcW;		/* two column width */
      if(ntrs >= 16)		/* not more than 16 */
	ntrs = 16;
      lsize = (ntrs+1)/2;
      ymx = 576.0/lsize;
      separator = 1.5*72.0/lsize;
      psize = ymx*sfac;

      ybb = tcH;		/* 10.0 in from bottom */
      for (i = 0; i < ntrs; i++) {
	if (i%2 == 0) {
	  ybb -= (ymx + separator);     /* start at top go down */
	  yb = ybb + separator;                 /* for labels */
	}
	hptr = spc[getid(numbs[i])];
	pdat = hdat + hptr->offset;
	fchan = (dalbrt((float)dsr) <= 0) ? 0 : dalbrt((float)dsr);
	len = dsl ? dalbrt((float)dsl) : hptr->xma;
	if(!yfix) 
	  ylimits(fchan,len);
	ysc = yscale(ymax,ymin,ymx);
	xb = (i%2 == 0) ? 80 : 350;
	fprintf(fps,"/w %.2f def\n",xmx);
	fprintf(fps,"/h %.2f def\n",ymx);
	fprintf(fps,"gs\n");
	fprintf(fps,"newpath\n");
	fprintf(fps,"%.2f %.2f box\n",xb,yb);
	fprintf(fps,"stroke\n");
	fprintf(fps,"10 deffont\n");
	fprintf(fps,"%.1f %.1f (%s:%d) cshow\n",
		xb+180,yb+ymx-10,hptr->title,hptr->id);
	fprintf(fps,"gr\n");
	fprintf(fps,"gs\n");
	fprintf(fps,"%.2f %.2f translate\n",xb,yb); /* set new origin */
	fprintf(fps,"%d deffont\n", 17-lsize);
	psaxis(0);
	psplot();
	pklabel(); 
	fprintf(fps,"gr\n");
	fprintf(fps,"gs\n");
      }
      pslabel();
      break;

    default:
      break;
    }
    fprintf(fps,"showpage\n");
    fprintf(fps,"%%%%Trailer\n");
    fclose(fps);
    return(0);
}

void psaxis(int mode)
{
    register float x, y, i;
    float      ny;
    float      max;
    float      step;
    float      roff;
    int        deltx;
    float      delty;
    float      tic;
    float      ytic;
    float      fch, dlen;
    float      divis;
    float      savexsc;

    fch = fchan;
    dlen = (float)len;
    xsc = xmx / dlen;		/* 8 inch wide */
    roff = xsc/2;		/* 1/2 of channel width */
    if(hptr->cflag) {		/* use energy scale */
        savexsc = xsc;
	fch = fch*hptr->sl + hptr->zr;
	dlen *= hptr->sl;
	xsc /= hptr->sl;
    }
    divis = xmx > tcW ? 11.0 : 5.5;
    step = mkround (dlen/divis);
    if (step < 1.0) step = 1.0;
    deltx = fch / step;
    deltx *= step;
				/* x-scale */
    while ((x = (deltx - fch) * xsc + roff) < xmx) {

      if(x > 5) {
	fprintf(fps,"%.2f %.2f m\n",x,ymx); /* top tick marks */
	fprintf(fps,"%.2f %.2f l\n",x,ymx-psize);
	fprintf(fps,"%.2f 0.0 m\n",x);	   /* bottom tick mark */
	fprintf(fps,"%.2f %.2f l\n",x,psize);
	if(!mode)
	  fprintf(fps,"%.2f 0.0 (%d) numdc\n",x, deltx);
      }
      deltx += step;
    }
    if(hptr->cflag)		/* restore scale for psplot */
      xsc = savexsc;

    delty = 0;			/* y-scale */
    y = 0.0;

    if (mlog) {
      tic = 1.0;
      ytic = tic*ysc;
      delty += deca;		/* deca = 0 is default */
      yoff = fabs(LOGD(0.0)*ysc);
      y += yoff;
      do {
	fprintf(fps,"%.1f %.2f m\n",xmx-psize,y); /* right tick */
	fprintf(fps,"%.1f %.2f l\n",xmx,y);
	fprintf(fps,"%.2f %.2f m\n",psize,y);	   /* left tick */
	fprintf(fps,"0.0 %.2f l\n",y);
	fprintf(fps,"-7.0 %.2f (%d) log10ur\n",y-3.0, (int)delty);
	for(i=2;i<10;i++) {	/* minor ticks */
	  if((ny = y + ytic*log10((double)i)) < ymx) {
	    fprintf(fps,"%.1f %.2f m\n",xmx-0.8*psize,ny); /* right tick */
	    fprintf(fps,"%.1f %.2f l\n",xmx,ny);
	    fprintf(fps,"%.2f %.2f m\n",0.8*psize,ny);     /* left tick */
	    fprintf(fps,"0.0 %.2f l\n",ny);
	  }
	}
	fprintf(fps,"stroke\n");
	delty += tic;
      } while ((y += ytic) < ymx);
    }
    else {
      tic = 0.1;
      max = (ymin > ymax) ? ymin : ymax;
      for (;;) {
	step = tic * 5;
	if (step > max)
	  break;
	tic = tic * 5.0;
	step = step * 5;
	if (step > max)
	  break;
	tic = tic * 2.0;
      }
      ytic = tic*ysc;
      if (yzero) {
	y = yzero;
	fprintf(fps,"gs\n");
	fprintf(fps,"[2] 0 setdash\n");
	fprintf(fps,"0.0 %.2f m\n",y);
	fprintf(fps,"%0.2f %0.2f l\n",xmx,y);
	fprintf(fps,"stroke\n");
	fprintf(fps,"gr\n");
	while ((y - ytic) > 0.0) {
	  y -= ytic;
	  delty -= tic;
	}
      }
      do {
	fprintf(fps,"%.1f %.2f m\n",xmx-psize,y); /* right tick */
	fprintf(fps,"%.1f %.2f l\n",xmx,y);
	fprintf(fps,"%.2f %.2f m\n",psize,y);	   /* left tick */
	fprintf(fps,"0.0 %.2f l\n",y);
	if(delty > 0.05 && delty<5.0)
	  fprintf(fps,"-7.0 %.2f (%.1f) numur\n",y-3.0, delty);
	else
	  fprintf(fps,"-7.0 %.2f (%.0f) numur\n",y-3.0, delty);
	fprintf(fps,"stroke\n");
	delty += tic;
      } while ((y += ytic) < ymx);
    }
}

#define Lim(a) (a < 0.0 ? 0.0 : (a > ymx ? ymx : a))

void psplot()			/* show histogram */
{
    register int  i;
    register float *p;
    float     x,y;		/* current coordinates */
    float     ybf;		/* y-before */

    p = pdat + fchan;
    ybf = (mlog ? (LOGD (p[0])-deca) : p[0]) * ysc;
    ybf += (mlog) ? yoff : yzero;

    fprintf(fps,"0.5 setlinewidth\n");
    fprintf(fps,"newpath\n");
    fprintf(fps,"0 %.2f m\n",Lim(ybf));

    for(i=0;i<len;i++) {

      x = i*xsc;
      y = (mlog ? (LOGD(p[i])-deca) : p[i]) * ysc;
      y += (mlog) ? yoff : yzero;
      if(fabs(y - ybf) < 0.01)
	continue;

      fprintf(fps,"%.2f %.2f l\n",x,Lim(ybf));
      fprintf(fps,"%.2f %.2f l\n",x,Lim(y));
      ybf = y;
      if(i%100 == 0) {
	fprintf(fps,"stroke\n");
	fprintf(fps,"newpath\n");
	fprintf(fps,"%.2f %.2f m\n",x,Lim(y));
      }
    }
    fprintf(fps,"%.1f %.2f l\n", xmx,Lim(ybf));
    fprintf(fps,"stroke\n");
}

void pslabel()			/* Portrait mode */
{
  fprintf(fps,"%d deffont\n", 21-lsize);
    if(hptr->cflag)		    /* use energy scale */
      fprintf(fps,"300 %d (Energy [keV]) cshow\n", 40 -(3*lsize));
    else 
      fprintf(fps,"300 %d (Channel number) cshow\n", 40 - (3*lsize));
    fprintf(fps,"300 740 (%s) center\n",strPS);
    wr_txt(strPS);
    fprintf(fps,"470 10 m\n");
    fprintf(fps,"8 deffont\n");
    fprintf(fps,"(%s) show\n", cdirec);
    fprintf(fps,"%d deffont\n", 21-lsize);
    fprintf(fps,"60 380 m\n");
    fprintf(fps,"90 rotate\n");
    fprintf(fps,"0 0 (Counts) show\n");
}

void ps1label()			/* LANDSCAPE mode */
{
    fprintf(fps,"20 deffont\n");
    if(hptr->cflag)		    /* use energy scale */
      fprintf(fps,"288.0 -36.0 (Energy [keV]) cshow\n");
    else 
      fprintf(fps,"288.0 -36.0 (Channel number) cshow\n");
    fprintf(fps,"288.0 450.0 (%s) center\n",strPS);
    wr_txt(strPS);
    fprintf(fps,"480 -50 m\n");
    fprintf(fps,"8 deffont\n");
    fprintf(fps,"(%s) show\n", cdirec);
    fprintf(fps,"20 deffont\n");
    fprintf(fps,"-60 200 m\n");
    fprintf(fps,"90 rotate\n");
    fprintf(fps,"0 0(Counts) show\n");
}

void pklabel()
{
    register int i,j,num;
    register float *p;
    float x, y, cen;
    float max;
    extern float cent(int n), calbrt(double cem);

    p = hdat + hptr->offset;
    fprintf(fps,"10 deffont\n");
    fprintf(fps,"newpath\n");

    for(i=0;i<mrks;i++) {
      num = marks[i];
      if(INSIDE(num)) {
	x = (num - fchan + 0.5)*xsc;
	max = p[num-2];
	for(j=1; j<5; j++)
	  if(p[num-2+j] > max)
	    max =p[num-2+j];
	y = (mlog ? (LOGD(max)-deca) : max) * ysc + 3;
	y += mlog ? yoff : yzero;
	if(y+30 > ymx) {
	  y = ymx -35;
	  x -= 8;
	}
	else {
	  fprintf(fps,"%.2f %.2f m\n",x,y);
	  fprintf(fps,"%.2f %.2f l\n",x,y+5);
	}
	fprintf(fps,"%.2f %.2f m\n",x+3,y+8);
	fprintf(fps,"gs\n");
	fprintf(fps,"90 rotate\n");
	if(p[num-1] < p[num] && p[num+1] < p[num])
	  cen = cent(num);
	else cen = num;
	if(hptr->cflag)
	  fprintf(fps,"(%.2f) show\n",calbrt((double)cen));
	else
	  fprintf(fps,"(%.2f) show\n",cen);
	fprintf(fps,"gr\n");
      }
    }
    fprintf(fps,"stroke\n");
}

void wr_txt(char *str)			/* sup scripting */
{
  register int i, j;
  int n;
  int norm = 1;
  char text[80];

  n = strlen(str);

  for(i=0,j=0;i<n;i++) {
    switch(str[i]) {

    case '^':
      if (j != 0) {
	text[j] = '\0';
	fprintf(fps,"(%s) show\n",text);
	j = 0;
      }
      if(norm) {
	fprintf(fps,"0 fontsize 0.6 mul rm smallfont\n");
	norm = 0;
      }
      else {
	fprintf(fps,"0 fontsize 0.3 mul rm normalfont\n");
	norm = 1;
      }
      break;

    case '_':
      if (j != 0) {
	text[j] = '\0';
	fprintf(fps,"0 0 (%s) show\n",text);
	j = 0;
      }
      if(norm) {
	fprintf(fps,"0 fontsize 0.3 mul neg rm smallfont\n");
	norm = 0;
      }
      else {
	fprintf(fps,"0 fontsize 0.6 mul neg rm normalfont\n");
	norm = 1;
      }
      break;

    default:
      if(str[i] == '\\')
	i++;
      text[j++] = str[i];
      if(i>78) {
	text[j] ='\0';
	fprintf(fps,"0 0 (%s) show\n",text);
      }
      break;
    }
   }
  text[j] ='\0';
  fprintf(fps,"0 0 (%s) show\n",text);
}
