/* Atomatic peak search for SA.
 * called in xsa.c 
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "dacf.h"

extern int    mrks, marks[];
extern float *pdat;
extern int    fchan, len;
extern int    ntrs;
extern int    peakmode;
extern char  *wptr[];
extern char   sadatfile[];
extern struct hinfo *hptr;
extern int    prtf;			/* print flag */

extern float cent(int);
extern float calbrt(double);
extern int   checkmarks();
extern void  setmark(int,int,int,float);

float   hwid = 2.0;		/* peak search parameter */
int     nwid = 2;		/* peaks search parameter*/

int  set_aps()
{
    if(ntrs == 0) {
        printf("Peak search parameter h = %.1f n= %d\n",hwid,nwid);
	return(0);
    }
    if(ntrs)
        hwid = atof(wptr[1]);
    if(ntrs>1)
        nwid = atoi(wptr[2]);
    return(0);
}

void search () {
    register int    i, jim, jum;
    int     jtail, peak;
    FILE   *op;         	/* sa.dat */

    float    amp, ampsq;
    float   isum, max, knc;
    float   icon;
    double  var, conv;

    jim = fchan + nwid;
    if (hwid < 4.0) {
	jtail = 2 * nwid;
	amp = 4;
    }
    else {
	jtail = nwid;
	amp = 2;
    }
    ampsq = amp * amp;

loop: 
    jum = jim + nwid;
    icon = 0;
    isum = 0;
    for (i = jim; i < jum; i++)
	icon += pdat[i];
    for (i = jim - nwid; i < jim; i++)
	isum = isum + pdat[i] + pdat[i + jtail];

    while (jim < fchan+len) {
	var = ampsq * icon + isum;
	if (var < 1.0)
	    var = 1.0;
	conv = (2 * icon - isum) / sqrt (var);
	if (conv > hwid) {	/* peak found */
	    max = 0;
	    jum = jum + nwid / 2;
	    for (i = jim; i <= jum; i++) {
		if (pdat[i] < max)
		    continue;
		max = pdat[i];
		peak = i;
	    }
	    jim = peak + nwid;
	    if(hptr->cflag || peakmode) {
	      setmark (peak, -1, WP, calbrt((double)cent(peak)));
	    }
	    else {
	      if(checkmarks()) return;
	      marks[mrks-1] = peak;
	      setmark (peak, mrks-1, WP, 0.0);
	    }
        	/* this is only for peak pos file with 'c' */
	    if(prtf && ((op = fopen (sadatfile, "a")) != NULL)) {
		fprintf(op," %7.3f",cent(peak));
		fclose (op);
	    }
	    goto loop;
	}
	knc = pdat[jum] - pdat[jim];
	icon += knc;
	isum = isum + pdat[jum + nwid] - pdat[jim - nwid] - knc;
	jim++;
	jum++;
    }
}
