/* display events for new data acquisition system at YALE 
 *
 * gkum 8.18.99
 * implemented new evt file header
 */

#include <stdio.h>
#include <sys/types.h>

int fd, nb, blocks, nd;
int evc=3;
int evhl=2;
short unsigned int dbuf[8*1024];

struct evnthead {
  char name[32];		/* text */
  int  byteorder;		/* byteorder indicator == 7 ok */
  int  nrun;			/* run number */
  int  type;			/* = 0 default
				   = 1 clover version */
  int  ndet;			/* number of detectors */
  int  mon;			/* monitor detector */
  int  npar;			/* number of parametes in event */
  int  parl;			/* particle spectrum lenght */
  int  gaml;			/* gamma spectrum lenght */
  int  monl;			/* monitor spectrum lenght */
  int  tdcl;			/* tac spectrum lenght */
  int  date;			/* time stamp */
  int  unused;
}evh;


main(argc,argv)
int argc;
char **argv[];
{

  int i,j;
  int fld;
  int ngam;
  int ndet;
  int sblock;
  char *filename;
  char   *cptr, tmp;
 

    if(argc < 1){
      printf("need file name\n");
      exit(1);
    }
    if((fd=open(*++argv,0)) < 0){
      printf("file not found\n");
      exit(1);
    }
    sblock = 0;
    if(argc == 3)
      sblock = atoi(*++argv);

    if(read(fd,&evh,sizeof(evh)) < 0) {
      perror("reading event header");
      exit(1);
    }
    if(strncmp(evh.name,"evt/",4) != 0) {
      printf("Not an Event data file\n");
      exit(1);
    }
    if(evh.byteorder != 7) {
      cptr = (char *) &evh.nrun;
      for(i=0;i<44;i+=4) {
	tmp = cptr[i];
	cptr[i] =cptr[i+3];
	cptr[i+3] = tmp;
	tmp = cptr[i+1];
	cptr[i+1] = cptr[i+2];
	cptr[i+2] = tmp;
      }
    }
    ndet = evh.ndet;
    printf("Run number: %d %s\n",evh.nrun,&evh.name[4]);
    printf("Time stamp: %d\n",evh.date);
    printf("Byte order: %s\n",(evh.byteorder==7)?"ok":"reversed");
    printf("Event format: npar = %2d\n",evh.npar);
    printf("              par  = %2d   %4d\n",1,evh.parl);
    printf("              ngam = %2d   %4d\n",evh.ndet,evh.gaml);
    if(evh.mon){
      printf("              mon  = %2d   %4d\n",evh.mon,evh.monl);
      ndet++;
    }
    printf("              ntdc = %2d   %4d\n",ndet,evh.tdcl);

    if(sblock) {
      if(lseek(fd,sblock*16384,SEEK_CUR) == (off_t) -1){
	perror("stepping forward");
	exit(1);
      }
    }
    blocks = sblock;
    do {
      
	if ((nb = read (fd, dbuf, sizeof (dbuf))) < 0) {
	    puts ("error: read error");
	    exit (1);
	}

	nd = nb >>1;
	printf ("\n%4d   Block length = %d\n\n", blocks++, nd);

	i = 0;

	if (nb) {

	    if(evh.byteorder != 7)
	      swapbuf(dbuf,nd);

	    while (dbuf[i] < 0170000) i++; /* not a header word */

	    printf("   i  ngam   part    det     en    time    fld\n\n");

	    for (   ; i < nd; i += evc*ngam+evhl) {
	      if(dbuf[i] == 0xffff)
		break;
	      ngam = dbuf[i]&255;	/* get number of detectors */
	      fld = dbuf[i]>>8 &0xf;
	      printf("%4d %5d ",i,ngam);
	      printf ("%6u %6u %6u  %6u %6u\n", 
		      dbuf[i+1],dbuf[i+2],dbuf[i+3],dbuf[i+4],fld);
		for (j = 1; j < ngam; j++)
		  printf ("%24u %6u  %6u\n",
			  dbuf[i+j*3+2],dbuf[i+j*3+3],dbuf[i+j*3+4]);
	    }
	}
    } while (nb != 0);
    close(fd);
}

swapbuf (t, n)
char   *t;
int     n;
{
    register int    i, j;
    register char   temp;

    for (i = 0; i < n; i++) {
	j = i << 1;
	temp = t[j];
	t[j] = t[j + 1];
	t[j + 1] = temp;
    }
}

