#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <fcntl.h>

#include "dacf.h"

struct filehead flhd;		/* data file header */
struct evnthead evh;		/* event file header */

static char data_type_list[3][15] = {
    "32 bit integer", "16 bit integer","real"};

void printout(), shevhd();

int main(argc,argv)
     int argc;
     char *argv[];
{
  int fd;
    
  if (argc == 1){
    printf("%s shows file header of data file\n",*argv);
    printf("Argument: file name\n");
    exit(1);
  }
  while (--argc>0) {
    if ((fd = open(*++argv,0)) <= 0){
      printf("can't open %s\n", *argv);
      continue;
    }
    if (read(fd,&evh,sizeof(evh))<=0){
      printf("Read error in %s\n",*argv);
      continue;
    }
    if(strncmp(&evh.name[0],"evt",3) == 0) {
      shevhd();
      close(fd);
      continue;
    }
    if(!strncmp(&evh.name[0],"data/*",6) || !strncmp(&evh.name[0],"his/",4)){

      lseek(fd,0,0);
      if((read(fd,&flhd,1024))<=0)
	printf("Read error in %s\n",*argv);
      printout();
      close(fd);
      continue;
    }
    if(!strncmp(&evh.name[0],"data/",5)) { /* old short file header */
      lseek(fd,0,0);
      if((read(fd,&flhd,512))<=0)
	printf("Read error in %s\n",*argv);
      printout();
      close(fd);
      continue;
    }
    if(evh.name[4] == '!') { /* old header */
      printf("Could be very old file header\n");
      close(fd);
      continue;
    }
    printf("%s  not a data file\n",*argv);
    close(fd);
    continue;
  }
}

void printout()
{
    register int i;
    int dlen, dtype;
    int hdflag;

    hdflag = 0;
    printf("\n%s",&flhd.name[0]);

    printf("\nRun number: %d\n",flhd.nrun);
    dlen = flhd.len;
    dtype = flhd.dtype&0xF;
    hdflag = flhd.dtype >> 4;
    dtype = dtype%3;

    printf("Data type : (%d) %s\n",flhd.dtype, data_type_list[dtype]);
    printf("Spectra   : %d\n",flhd.maxid);
    printf("Data words: %d\n",flhd.len);

    if(flhd.maxid > 2000) {
      printf("Possible byte order conflict --\n");
      return;
    }
    if(flhd.dtype > 2 && flhd.dtype < 10) {
      printf("Matrix: %d x %d\n",flhd.sinfo[0].id,flhd.sinfo[0].offs);
    }
    else if (!hdflag) {
      flhd.sinfo[flhd.maxid].offs = flhd.len;
      for(i=0;i<flhd.maxid;i++) {
	printf("%4d %4d %6d %6d\n",i, flhd.sinfo[i].id,
	       flhd.sinfo[i].offs,flhd.sinfo[i+1].offs -
	       flhd.sinfo[i].offs);
      }
    }
}

void shevhd()
{
  register int i;
  int npar;
  const struct tm *tms;
  time_t etime;
  char   *cptr, tmp;
  char   ostr[32];
  
  npar = 0;
  if (evh.byteorder != 7) {
    cptr = (char*) &evh.nrun;
    for (i = 0; i < 44; i += 4) {
      tmp = cptr[i];
      cptr[i] = cptr[i + 3];
      cptr[i + 3] = tmp;
      tmp = cptr[i + 1];
      cptr[i + 1] = cptr[i + 2];
      cptr[i + 2] = tmp;
    }
  }
  printf("\n  Run number: %d\n",evh.nrun);
  if(evh.date > 4)
    printf("  Date stamp: %7.4f\n",(float)evh.date/10000.0);
  else
    npar = evh.date;
  if(evh.time) {		/* for backward compatibility */
    etime = evh.time;
    tms = localtime(&etime);
    printf("  Time stamp: %s",asctime(tms));
  }
  printf("  Events:     %d\n",evh.npar);
  if(npar)
    printf("  Format:     %d*%d; %d*%d; %d*%d; %d*%d\n",
	   npar,evh.parl,evh.ndet,evh.gaml,evh.mon,evh.monl,
	   (evh.ndet+evh.mon),evh.tdcl);
  else
    printf("  Format:     %d; %d*%d; %d*%d; %d*%d\n",
	   evh.parl,evh.ndet,evh.gaml,evh.mon,evh.monl,
	   (evh.ndet+evh.mon),evh.tdcl);
  switch(evh.type) {
    case 0:
      strcpy(ostr,"Ge");
      break;
    case 3:
      strcpy(ostr,"NaI");
      break;
    case 1:
      strcpy(ostr,"Clovers");
      break;
    case 2:
      strcpy(ostr,"Clovers gainmatched");
      break;
  }
  printf("  Byte order: %s\n",
	 (evh.byteorder == 7)? "ok" : "reversed");
  printf("  Setup:      %s\n", ostr);
  printf("  Comment:    %s\n",&evh.name[4]);
}
