/*   Sample file to write data in SA format */

#include <stdio.h>
#include <stdlib.h> 
#include <string.h>
#include <unistd.h>
#include <fcntl.h>

#include "dacf.h"		/* SA definitions */

#define N  20			/* set proper size for data array */
#define SPECL 4096		/* Length of individual spectrum */


float hdata[N*1024];		/* data array */
char *chbuf;
char  filnam[64];
char  spname[32];		/* output file name */

struct filehead flhd = {"his/"};	/* file head structure */

int main(argc,argv)
int argc;
char *argv[];
{
  register int i,j,n;
  int fd;
  int len;
  int nb;
  int swap;
  int record;

  struct header {		/* this is the unix header format */
    int    rlen;		/* record length = 24 */
    char   namesp[8];		/* file name */
    int    numch;		/* number of channnels  = idim1*/
    int    idim2;		/* second dimension for matrixes */
    int    cc[3];		/* two more numbers + rlen again */
  } inh;

  union {
    struct header inh;
    char cbuf[0];
  } head;

  char erstr[80];
  char csave;
  char filext[16];
  void bsname();

  if(argc == 1){
    printf("%s translates GF2.spe data format to XSA.his\n",*argv);
    printf("Argument: file name\n");
    exit(1);
  }
  chbuf = (char *) hdata;
  i = 1;
  do {
    if(argc > 1 && (fd = open(argv[i],0)) == EOF){
      sprintf(erstr,"cannot open file %s\n",argv[i]);
      perror(erstr);
      continue;
    }
    j = 0;
    while((csave = argv[i][j]) != '.')
      filnam[j++] = csave;
    filnam[j] = '\0';
    n = 0;
    while((csave = argv[i][j++]) != '\0')
      filext[n++] = csave;
    filext[n] = '\0';
    if(strcmp(filext,".spe") != 0) {
      fprintf(stderr,"%s has wrong file ext %s\n",argv[i],filext);
      continue;
    }
    bsname(filnam,spname);
    strcat(spname,".his");	/* out file name has ext ".his" */

    if(read(fd,&head,sizeof(struct header))<=0){
      fprintf(stderr,"read error in input file %s\n",argv[i]);
      continue;
    }

    if (head.inh.rlen != 24) {
      swap = 1;
    /* file has wrong byte order - swap bytes */
      printf("*** Swapping bytes read from file %s\n", argv[i]);
      for (j = 3; j < 5; ++j) {
	csave              = head.cbuf[j*4 + 3];
	head.cbuf[j*4 + 3] = head.cbuf[j*4];
	head.cbuf[j*4]     = csave;
	csave              = head.cbuf[j*4 + 2];
	head.cbuf[j*4 + 2] = head.cbuf[j*4 + 1];
	head.cbuf[j*4 + 1] = csave;
      }
    }
    else {
      swap = 0;
    }
    len = head.inh.numch;

    if(read(fd,&record,4) != 4 || len*4 != record) {
      fprintf(stderr,"error in input file %s\n",argv[i]);
      continue;
    }
    if((nb= read(fd,chbuf,record)) != record ) {
      fprintf(stderr,"error reading %d data in file %s\n",nb,argv[i]);
      continue;
    }

    if(swap) {
    /* file has wrong byte order - swap bytes */
      for (j = 0; j < len; ++j) {
	csave          = chbuf[j*4 + 3];
	chbuf[j*4 + 3] = chbuf[j*4];
	chbuf[j*4]     = csave;
	csave          = chbuf[j*4 + 2];
	chbuf[j*4 + 2] = chbuf[j*4 + 1];
	chbuf[j*4 + 1] = csave;
      }
    }

    if ((fd = creat (spname, 0644)) < 0) {  /* open disk file */
      fprintf (stderr,"error opening %s\n", filnam);
      exit(1);
    }
    strncpy (&flhd.name[4], spname, 12);	/* original data file name */
    flhd.name[15] = '\0';			/* allways zero */

    flhd.nrun = 1;		                /* can be any number */
    flhd.dtype = 2;				/* int */
    flhd.len = len;			        /* total length */
    flhd.maxid = 1;			        /* # of spectra (id's) */

    for (j = 0; j < flhd.maxid; j++) {
      flhd.sinfo[j].id = j+1;
      flhd.sinfo[j].offs = j * len;
    }

    for (j++ ; j< NSPEC; j++) {
      flhd.sinfo[j].id = 0;
      flhd.sinfo[j].offs = 0;
    }
    write (fd, &flhd, sizeof (flhd));
    write (fd, chbuf, flhd.len<<2);
    close (fd);
  } while(++i < argc);
}

void bsname(a,b)
char *a, *b;
{
    char *tb = b;

    while( *a != '\0') {
        *tb++ = *a;
        if(*a++ =='/')
          tb = b;
    }
    *tb = '\0';
}
