/*      SA subroutine
 *	module:        sumlines.c
 *	X - version
 *
 *      sum                     (without arguments) 
 *                              sums counts between marker pairs in
 *                              current spectrum
 *      sum id                  (one argument)
 *                              id is the spectrum id for summing
 *      sum id addr             (two arguments)
 *                              the second argument is the start addr
 *                              for a back ground or error spectrum
 *                              addr can be w, w+#k, #k. Default is
 *                              for use after ful command.
 *      an optional fourth argument can be
 *      s(special), n(no), b(bgd), p(marks in peak mode),ps, pn, pb
 */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include "dacf.h"
#include <Xm/Xm.h>
#include <Xm/Command.h>

#define    k_com_win  6		/* command window */

extern float *pdat;
extern float *hdat;
extern int    len;
extern int    idx;		/* current index */
extern int   *pmarks;
extern float  calbrt(double);
extern int    getid(int);
extern int    kilo(char*);
extern int    bktg[][11];
extern int    mrks;
extern char  *wptr[];		/* token word pointers */
extern int    ntrs;
extern struct hinfo *hptr, *spc[];

extern Widget wdg_arr[];	/* all other widgets */

int sumlines()
{
    register int i,j;

    int    nf[NMARKS/2];
    int    nl[NMARKS/2];
    int    max;
    int    out;			/* output file open */
    int    nx,mx;		/* spectra indices */
    static int       m = 0;	/* peak mode off */
    static int    addr = 0;	/* start of bkgd data */
    static int     bkf = 0;	/* bkgd treatement flag */
    double perr;
    double centr;
    double dsum;
    double bd, sum;
    float *pd;			/* pointer of bkgd data */
    XmString text;
    char errstr[32];
    FILE *fp;


    nx = idx;			/* current index is default */
    if (ntrs) {
      if((nx = getid(atoi(wptr[1]))) < 0) {
	sprintf(errstr,"%s unknown",wptr[1]);
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win],text);
	XmStringFree(text);
	return(-1);
      }
      if(ntrs > 1) {
	if(wptr[2][0] == 'w') {
	  addr = MKNR*1024;
	  if(wptr[2][1] == '+') {
	    strcpy(errstr,&wptr[2][2]);
	    addr += kilo(errstr);
	  }
	}
	else {
	  addr = kilo(wptr[2]);
	  if(addr < MKNR && (mx = getid(addr)) > 0)
	    addr = spc[mx]->offset;
	  printf("add = %d\n",addr);
	}
      }
      if(ntrs > 2) {
	if (wptr[3][0] == 's')
	  bkf = 1;		        /* special spectrum */
	else if (wptr[3][0] == 'b')
	  bkf = 2;			/* bkgd spectrum */
	else if (wptr[3][0] == 'n')
	  bkf = 0;			/* no bkgd (sqrt(psum)) */
	else if (wptr[3][0] == 'p') {
	  m = 1;
	  if (wptr[3][1] == 's')
	    bkf = 1;
	  else if (wptr[3][1] == 'b')
	    bkf = 2;
	  else if (wptr[3][1] == 'n')
	    bkf = 0;
	}
	else {
	  bkf = 0;
	  m = 0;
	}
      }
    }
    printf("idx = %d nx = %d addr = %d sqr = %d m = %d\n",idx,nx,addr,bkf,m);
    pd = hdat + addr;
    pdat = hdat + spc[nx]->offset;
    out = 0;
    if ((fp = fopen("results.dat","a")) != NULL){
      fprintf(fp,"%s:%d\n",hptr->title,hptr->id);
      out = 1;
    }

    if(m == 0) {
      for (i = 0, j = 0; i < mrks; i+=2) {
	nf[j]   = pmarks[i];
	nl[j++] = pmarks[i+1];
      }
      max = j;
    }
    else {
      for (i = 0, j = 0; i < mrks; i++) {
	nf[j]   = bktg[i][0];
	nl[j++] = bktg[i][1];
      }
      max = mrks;
    }

    for(i = 0; i < max; i++) {
      sum = 0.0;
      bd = 0.0;
      dsum = 0.0;
      for (j = nf[i]; j <= nl[i]; j++) {
	sum += pdat[j];
	bd  += pd[j] ;
	dsum+= pdat[j]*j;
      }
      if(sum > 0.01) {
	centr = dsum/sum;
	if(bkf == 0)
	  bd = 0.0;
	perr  = sqrt(sum + 2.0*bd);
	if(bkf == 1)
	  perr = sqrt(bd);
      }
      printf("%9.2f %9.2f %9.2f %9.2f\n",calbrt(centr),sum,perr,bd);
      if(out) 
	fprintf (fp, "%9.2f %9.2f %9.2f\n",calbrt(centr),sum,perr);
    } 
    if(out)
      fclose (fp);
    return (0);
}
