/* udselect.c --- 
 * 
 * functions used for dealing with transient field data analysis
 * in Xsa
 *
 * G.Kumbartzki
 * date 4-14-96
 * modified input order (det, data id) and error reporting (3/7/05).
 * changed true to tru because of keyword true (1.14.08)
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <X11/Xlib.h>
#include <Xm/Command.h>
#include <Xm/Text.h>
#include "dacf.h"

#define    k_com_win               6
#define    k_mess                 20

int    udlen;			/* length of ud spectrum */
int    m2,m3;			/* limits of peak region */
int    m0,m5;			/* limits of bkgd region */
int    np,nd;			/* peak and detector numbers */
int    srflag;			/* print single det ratios */
int    dbf;			/* do bkgd ratios */
double sumr;			/* adjusted randoms sum */
double rsum;			/* randoms sum */
double sumt;
double psum;
double centr,cenerr;		/* peak position  & error */
double tru[UDPNR][4][2];	/* results of ud analysis */
double truerr[UDPNR][4][2];
double st[UDPNR][4][2];		/* sum total saved */
double bks[UDPNR][4][2];	/* total backgroud 0-5 */
double ppos[UDPNR];		/* peak positions */
float *pup;			/* pointer to field up */
float *pdown;			/* pointer to field down */
float *pran;			/* pointer to randoms */

extern Widget   wdg_arr[];	/* all other widgets */
extern Display *dpy;
extern Window   gwin;
extern GC       gc;
extern Pixmap   pixmap;
extern int width,height;
extern struct colorlist color_vec[];
extern double pa[8], dpa[8];  /* store peak area and error for anis */
extern int peak();
		     /* defined in sa.c */

extern char    sadatfile[];	         /* name od sa.dat file */
extern char    cmd_line[];	         /* typed command line */
extern char   *wptr[];			 /* word pointer array */
extern int     ntrs;			 /* number entries */
extern int     idx;		         /* current spectra id */
extern int     fchan;			 /* first channel */
extern int     lexp;		         /* level count of expansion */
extern int     sfchan[],slen[];	         /* expansion buffer */
extern int     len;		         /* display lenght */
extern int     yfix;		         /* fixed yscale */
extern int     ytop,ybot,ylen,yzero;     /* y Display field values */
extern int     xzero, xmax;              /* x Display field values */
extern int    *pmarks;          	 /* pointer to current marker arrays */
extern int     marks[];		         /* marker array */
extern int     npeak;			 /* current peak number */
extern int     npks;		         /* number of peaks in fit */
extern int     pwidth;			 /* 1/2 display width in peak() */
extern int     dsl, dsr;		 /* display  length and range */
extern int     mrks;		         /* current number of markers */
extern int     peakmode;	         /* peak mode select */
extern int     randoms;		         /* randoms flag */
extern int     udflag;			 /* up/down mode */
extern int     bmode;		         /* individual or averaged BKGD */
extern int     det;
extern int     repeat;
extern float  *pdat,*hdat;    	         /* his data pointer */
extern float   xinc, yinc;	         /* scale factors */
extern struct  hinfo *spc[], *hptr;
extern struct  filehead flhd;
extern double  bp[], bgd(double a);
extern double  gpar[],sigm[];
extern void    displa(), xy_axis(), hplot(), axisbox();
extern float   yscale (float max,float min,float hght);
extern float   calbrt(double cem);
extern void    setmark(int nchan,int i,int mod,float en);
extern void    clear_screen();
extern int     getid(int),dalbrt(float);
extern void    gfit(), fplot();
void sratio(int);
void bkratios(int);

int udselect()
{
    register int i;
    static float *psm;		/* pointer to sum of data */
    static int initial_id = 1;	/* start of ud sequence */
    int id;
    XmString text;

    psm  = hdat + MKNR*1024;	/* pointer to work space */
    if (ntrs > 0)
      det = atoi(wptr[1]);
    if (det < 1 || det > 4) {
      XmTextSetString(wdg_arr[k_mess],"Check detector number");
      putchar(07);
      fflush(stdout);
      return(-1);
    }
    if(det == 1 && ntrs > 1) {
      initial_id = atoi(wptr[2]);
      np = (wptr[3] == NULL) ? 0 : atoi(wptr[3]);
    }
    nd = det-1;
    i = 4*nd + initial_id;
    if((id = getid(i)) == -1) {
      XmTextSetString(wdg_arr[k_mess],"Data ID not valid!");
      putchar(07);
      fflush(stdout);
      return(-1);
    }
    hptr = spc[id];
    pdat = hdat + hptr->offset;
    udlen = hptr->xma;
    spc[0]->xma = udlen;
    spc[0]->id  = hptr->id;
    strcpy(spc[0]->title,spc[id]->title);
    if(udlen*3 > WRK*1024) {
      XmTextSetString(wdg_arr[k_mess],"ERROR: workspace exceeded");
      putchar(07);
      fflush(stdout);
      return(-1);
    }
    pup = psm + udlen;
    pdown = pup + udlen;
    pran = pdat +2*udlen;	/* always right after up/down data */
	
    if(randoms) {
	for(i=0;i<2*udlen;i++)		 /* copy TRUE - RANDOM of up/down */
	  pup[i] = pdat[i] - pran[i];	          /* data into work space */
    }
    else {
	for (i=0;i<2*udlen;i++)
	  pup[i] = pdat[i];
    }

    for(i=0;i<udlen;i++)		 /* average of up & down */
      psm[i] = (pup[i] + pdown[i])/2.0;

    pdat = psm;
    fchan = 0;
    fchan += dalbrt((float)dsr);
    len = dsl ? dalbrt((float)dsl) : hptr->xma;
    lexp = 0;
    sfchan[0] = fchan;
    slen[0] = len;
    peakmode = 0;
    /*    yfix = 0; */
    idx = 0;
    hptr = spc[0];
    udflag = 1;
    pmarks = marks;

    if(pmarks[4*np+nd] != -1) { /* if np != 0 calculate peak number */
      if (wptr[1] == NULL) wptr[1] = &cmd_line[0];
      sprintf(wptr[1],"%d",4*np+nd);
      ntrs = 1;
      peak();
    }
    else {
      displa();			/* show <ud> in work area */
    }
    return(0);
}
/*
 *      result is stored tru[npeak][det][up/dn]
 *      npeak can be up to 50
 */

void dsp(int mode)
			/* various opertions */
{
    extern void bkgd1();
    extern int bgdplt();
    double sumran(int k,int a,int b);
    void   udoutput(int n,int mode);

    register int k,i;
    int    li,lf;
    int    yp;
    int    savytop, savybot;
    float  fylen;
    float *pd;
    char   str[32];
    double dsum;
    double tbd;
    double bd;
    double sum;
    double diff;
    double dat;
    double bt[2];
    double sumbk;
    char xstr[120];
    FILE  *op;



    savytop = ytop;			   /* save temporary display par */
    savybot = ybot;

    clear_screen();

    fylen = (float) ylen/2.0;

    hptr = spc[0];
    nd = det-1;
    dbf = 0;

    yinc /= 2.0;  /* yscale(fchan,len,fylen); */

    for(k = 0; k < 2; k++) {	        /* k = 0 field up
					   k = 1 field down */
	ytop = savytop + k*fylen;         /* to prevent round off */
	ybot = ytop + (int)fylen;
	sprintf(str,"%d %s",hptr->id,k?"DOWN":"UP");
			     /* scale is taken from averaged spectrum */
	pdat = hdat + MKNR*1024 + (k+1)*udlen;
	hplot();
	xy_axis();
	if(k == 0)			 /* draw dividing line */
	  XDrawLine(dpy,pixmap,gc,xzero,ybot,xmax,ybot);
	for(i=0;i<mrks;i++)
	  if(INSIDE(pmarks[i]))
	    setmark(pmarks[i],i,P,0.0);
	XSetForeground (dpy,gc,color_vec[7].pixel); /* black */
	yp = k ? ytop+26 : ytop+13;
	XDrawString (dpy,pixmap,gc,xzero+3,yp,str,strlen(str));
	XSetForeground (dpy,gc,color_vec[1].pixel);

	bt[k] = 0.0;
	if (pmarks[10] == -1)
	  mode = 3;
	else  {
	  if(bmode) {	        	 /* keep bgd from averaged data */
	    bgdplt();
	  }
	  else {
	    bkgd1();		 /* new bgp[0] */
	    bgdplt();
	  }
	  bt[k] = bp[0];
	  XSetForeground (dpy,gc,color_vec[1].pixel);
	}

	if(mode==1){		     /* integrate */

	  srflag = 1;
	  if (pmarks[2] != -1 && pmarks[3] != -1) {
	    m2 = pmarks[2];
	    m3 = pmarks[3];
	    m0 = pmarks[0];
	    m5 = pmarks[5];
	  }
	  else {
	    m2 = pmarks[0];
	    m3 = pmarks[1];
	    m0 = m2;
	    m5 = m3;
	  }
	  tbd = 0.0;
	  psum = 0.0;
	  dsum = 0.0;
	  sumt = 0.0;

	  for(i=m2; i <= m3; i++) {
	    bd = bgd((double)(i));
	    tbd += bd;
	    dat = (double)pdat[i] - bd;
	    dsum += dat * (double)i;
	    sumt += (double)pdat[i];
	  }
	  psum = sumt - tbd;
	  centr = dsum/psum;
	  sum = 0.0;
	  for(i=m2; i <= m3; i++) {
	    diff = (double)i - centr;
	    sum += (double)pdat[i] * diff * diff;
	  }
	  cenerr = sqrt(sum)/psum;	/* error in position */
	  sumr = sumran(k,m2,m3);     /* get randoms summed */
	  ppos[np] = centr;
	  tru[np][nd][k] = psum;
	  truerr[np][nd][k] = sqrt(sumt + tbd + 2.0*sumr);
	  st[np][nd][k] = sumt;
	  sumbk = 0.0;
	  for (i = m0; i <= m5; i++)
	    sumbk += bgd((double)(i));
	  bks[np][nd][k] = sumbk;
	  dbf = TRUE;
	}

	else if(mode == 2){	     /* gauss fit */

	    gfit();		     /* fit area, pos, and fhwm */
	    fplot();
	    XSetForeground (dpy,gc,color_vec[1].pixel);
	    for (i = 0; i < npks; i++) {
		lf = 0.8*gpar[3*i+2];
		li = gpar[3*i+1] - lf;
		lf = li +2*lf;
		sumr = sumran(k,li,lf);    /* get randoms summed */
		sigm[3*i] = sqrt(sigm[3*i]*sigm[3*i]+sumr);
		truerr[np+i][nd][k] = sigm[3*i];
		tru[np+i][nd][k] = gpar[3*i];
		ppos[np+i] = gpar[3*i];
	    }
	}
	else if (mode == 3) {	/* no bkgd */

	  srflag = 1;		/* print single det effects */
	  if (pmarks[2] != -1 && pmarks[3] != -1) {
	    m2 = pmarks[2];
	    m3 = pmarks[3];
	  }
	  else {
	    m2 = pmarks[0];
	    m3 = pmarks[1];
	  }
	  psum = 0;
	  dsum = 0;
	  for(i=m2;i <= m3; i++) {
	    psum += (double)pdat[i];
	    dsum += (double)pdat[i] * (double)i;
	  }
	  centr = dsum/psum;
	  sum = 0.0;
	  for(i=m2; i <= m3; i++) {
	    diff = (double)i - centr;
	    sum += (double)pdat[i] * diff * diff;
	  }
	  cenerr = sqrt(sum)/psum;	/* error in position */
	  sumr = sumran(k,m2,m3);     /* get randoms summed */
	  ppos[np] = centr;
	  tru[np][nd][k] = psum;
	  truerr[np][nd][k] = sqrt(psum + 2.0*sumr);
	  bt[k] = 0.0;
	  st[np][nd][k] = psum;
	}
	udoutput(k,mode);
    } /* end of for loop */
    if (mode != 2) {
      sprintf(xstr,
	      "det%d  up p: %.1f(%.1f) t: %.1f(%.1f) b: %.1f       down p: %.1f(%.1f) t: %.1f(%.1f) b: %.1f",
	      nd+1, tru[np][nd][0],truerr[np][nd][0],st[np][nd][0],
	      sqrt(st[np][nd][0]),bt[0],
	      tru[np][nd][1],truerr[np][nd][1],st[np][nd][1],
	      sqrt(st[np][nd][1]),bt[1]);
      XmTextSetString(wdg_arr[k_mess],xstr);
      fflush(stdout);
    }
    
    if ((op = fopen ("results.dat", "a")) != NULL) {
      fprintf(op,"%5d %10.2f %10.1f %8.1f %10.1f %8.1f\n",
	      hptr->id,calbrt(ppos[np]),
	      tru[np][nd][0],truerr[np][nd][0],
	      tru[np][nd][1],truerr[np][nd][1]);
      fclose (op);
    }
    ytop = savytop;		/* restore display parameters */
    ybot = savybot;
    yinc *= 2.0;
    axisbox();
    XCopyArea(dpy,pixmap,gwin,gc,0,0,width,height,0,0);
    pdat = hdat + MKNR*1024;	/* stay in work area */
}

double sumran(int k,int a,int b)
{
    register int i;
    double sum = 0;
    float *pd;

    rsum = 0;
    if(randoms) {
      pd = pran + k*udlen;
      for(i=a;i <= b; i++)
	sum += (double)pd[i];
      rsum = sum; 			 /* randoms sum for output only */
      sum *= (pd[0] == 0) ? 1.0 : pd[0]; /* scale factor for error */
    }
    return(sum);
}

int dratio4()			/* four detector double ratio */
{
    register int    i,j;
    static int  startpnr = 0;
    static int  nr = 1;
    static char outfile[64] = {"rlog.dat"};
    void dratio(int n);
    double sigm;
    double sum, sumv, weight;

    FILE  *op;
    extern char filNam[];
    extern int inda;

    if(filNam[0] != '\0')
      strcpy(outfile,filNam);

    if(ntrs) {
      startpnr = atoi (wptr[1]);
      if (ntrs > 1)
	nr = atoi (wptr[2]);
    }
    if (startpnr < 0)
      startpnr = 0;
    if (nr > UDPNR)
      nr = UDPNR;

    for (i = startpnr; i < startpnr + nr; i++) {
      dratio (i);
      if ((op = fopen (outfile, "a")) != NULL) {
	fprintf (op, "run %d\n",flhd.nrun);
	fprintf (op, "peak %.2f\n",calbrt(ppos[i]));
	for(j=0;j<4;j++) {
	  fprintf(op,"%.1f  %.3f  %.3f\n",
		  (float)j,tru[i][j][0],truerr[i][j][0]);
	  fprintf(op,"%.1f  %.3f  %.3f\n",
		  (float)j,tru[i][j][1],truerr[i][j][1]);
	}
	fclose (op);
      }
      sum = 0.0;
      sumv = 0.0;
      for(j=0;j<4;j++) {
	weight = (truerr[i][j][0]*truerr[i][j][0]);
	sumv += weight;
	weight = (truerr[i][j][1]*truerr[i][j][1]);
	sumv += weight;
	sum += tru[i][j][0];
	sum += tru[i][j][1];
      }
      sigm = sqrt(sumv);		 /* error */
      fprintf(stderr,"\nsum = %10.6f %10.6f\n",sum,sigm);
      pa[inda] = sum;		/* used in anis to calculate W() */
      dpa[inda] = sigm;		/* inda read in mkud */
    }
    return(0);
}

void dratio(int n)
{   
    register int i,j;
    extern double slope, dslope;
    extern double eta,deta;
    double temp, sltmp;
    double fi, dfi;		/* precession angle */
    double ro[4],dro[4],dsqr[4][2];
    double rho,drho,rhochk,drhochk;
    double eps,epschk,deps,depschk;
    FILE  *op;

    printf("\nFile: %s  Peak: %.2f\n",spc[1]->title,calbrt(ppos[0]));
    printf("                 up                      down\n");
    printf("det        sum        sumerr        sum        sumerr\n");

    for(i=0;i<4;i++)
      printf(" %d      %8.2f    %8.2f     %8.2f    %8.2f\n",
	     i+1,tru[n][i][0],truerr[n][i][0],tru[n][i][1],truerr[n][i][1]);
    for(i=0;i<4;i++)
      for(j=0;j<2;j++)
	if(tru[n][i][j] == 0.0 || truerr[n][i][j] == 0.0)
	  return;

/* ratio 1 4 */
    ro[0] = sqrt((tru[n][0][0]*tru[n][3][1])/(tru[n][0][1]*tru[n][3][0]));
/* ratio 2 3 */
    ro[1] = sqrt((tru[n][1][0]*tru[n][2][1])/(tru[n][1][1]*tru[n][2][0]));

    rho = sqrt(ro[0]/ro[1]);	/* 14/23 */

/* ratio 1 3 */
    ro[2] = sqrt((tru[n][0][0]*tru[n][2][1])/(tru[n][0][1]*tru[n][2][0]));
/* ratio 2 4 */
    ro[3] = sqrt((tru[n][1][0]*tru[n][3][1])/(tru[n][1][1]*tru[n][3][0]));

    rhochk = sqrt(ro[2]/ro[3]);	/* 13/24 */

    eps=(rho-1.0)/(rho+1.0);
    epschk=(rhochk-1.0)/(rhochk+1.0);
    
/* errors */

    for (i=0;i<4;i++)
      for (j=0;j<2;j++) 
	dsqr[i][j] = pow((truerr[n][i][j]/tru[n][i][j]),(double)2);

    dro[0] = 0.5 * ro[0] * sqrt(dsqr[0][0] + dsqr[0][1]
				+ dsqr[3][0] + dsqr[3][1]);

    dro[1] = 0.5 * ro[1] * sqrt(dsqr[1][0] + dsqr[1][1]
				+ dsqr[2][0] + dsqr[2][1]);

    dro[2] = 0.5 * ro[2] * sqrt(dsqr[0][0] + dsqr[0][1]
				+ dsqr[2][0] + dsqr[2][1]);

    dro[3] = 0.5 * ro[3] * sqrt(dsqr[1][0] + dsqr[1][1]
				+ dsqr[3][0] + dsqr[3][1]);


    drho = 0.5* rho * sqrt(pow(dro[0]/ro[0],(double) 2)
			   + pow(dro[1]/ro[1],(double) 2));

    drhochk = 0.5 * rhochk * sqrt(pow(dro[2]/ro[2],(double) 2)
				  + pow(dro[3]/ro[3],(double) 2));

    deps = 2*drho/((rho + 1.0)*(rho + 1.0));
    depschk = 2*drhochk/((rhochk + 1.0)*(rhochk + 1.0));

    printf("\n\tro 1,4   = %9.6f  +- %f\n",ro[0],dro[0]);
    printf("\tro 2,3   = %9.6f  +- %f\n",ro[1],dro[1]);
    printf("\tro 1,3   = %9.6f  +- %f\n",ro[2],dro[2]);
    printf("\tro 2,4   = %9.6f  +- %f\n\n",ro[3],dro[3]);
    printf("\tRO TOTAL = %9.6f  +- %f\n",rho,drho);
    printf("\tepsilon  = %9.6f  +- %f\n",eps,deps);
    printf("\tepscheck = %9.6f  +- %f\n\n",epschk,depschk);

    if (fabs(slope) > 0.0001) {	/* calculate precession angle fi */
      fi = (rho - 1.0) / (rho + 1.0) / slope;
      temp = 2.0*drho/(rho*rho - 1);
      sltmp = (dslope / slope);
      dfi = fabs(fi) * sqrt(temp*temp + sltmp*sltmp);
      printf("\tslope            = %10.5f(%f) \n",slope,dslope);
      printf("\tPrecession angle = %10.5f(%f) rad\n",fi,dfi);
    }

/* output to disk */

    if((op=fopen(sadatfile,"a")) == 0) {
	printf("cannot open sa.dat\n");
	return;
    }
    fprintf(op,"\nFile: %s  ",spc[1]->title);
    fprintf(op,"Peak: %.2f\n",calbrt(ppos[0]));
    fprintf(op,"                 up                      down\n");
    fprintf(op,"det        sum        sumerr        sum        sumerr\n");

    for(i=0;i<4;i++)
      fprintf(op," %d      %8.2f    %8.2f     %8.2f    %8.2f\n",
	      i+1,tru[n][i][0],truerr[n][i][0],tru[n][i][1],truerr[n][i][1]);

    fprintf(op,"\n\tro 1,4   = %9.6f  +- %f\n",ro[0],dro[0]);
    fprintf(op,"\tro 2,3   = %9.6f  +- %f\n",ro[1],dro[1]);
    fprintf(op,"\tro 1,3   = %9.6f  +- %f\n",ro[2],dro[2]);
    fprintf(op,"\tro 2,4   = %9.6f  +- %f\n\n",ro[3],dro[3]);
    fprintf(op,"\tRO TOTAL = %9.6f  +- %f\n",rho,drho);
    fprintf(op,"\tepsilon  = %9.6f  +- %f\n",eps,deps);
    fprintf(op,"\tepscheck = %9.6f  +- %f\n\n",epschk,depschk);
    if(fabs(slope) > 0.001) {
      fprintf(op,"\tEta parameter    = %10.5f(%f)\n",eta,deta);
      fprintf(op,"\tslope            = %10.5f(%f) \n",slope,dslope);
      fprintf(op,"\tPrecession angle = %10.5f(%f) rad\n",fi,dfi);
    }
    repeat = 1;
    fclose(op);
    if (srflag)
      sratio(n);
    if(dbf)
      bkratios(n);
}


int dratio2()			/* two detector double ratio */
{
  register int i,j,n;
  int x[2];			/* detector number array */
  XmString text;
  double  dsqr[2][2];
  double  rho, drho;
  double  eps, deps;

  FILE  *op;

  if(ntrs < 2){
    text = XmStringCreate("input mmissing",XmSTRING_DEFAULT_CHARSET);
    XmCommandError(wdg_arr[k_com_win], text);
    XmStringFree(text);
    return(-1);
  }
  if(ntrs > 2)
    n = atoi(wptr[3]);
  else
    n = 0;
    
  x[0] = atoi(wptr[1]) - 1;
  x[1] = atoi(wptr[2]) - 1;
  if (x[0] < 0 || x[0] > 3 || x[1] < 0 || x[1] > 3) {
    XmTextSetString(wdg_arr[k_mess],"Check detector numbers");
    putchar(07);
    fflush(stdout);
    return(-1);
  }
  printf("\nFile: %s  Peak: %.2f\n",spc[1]->title,calbrt(ppos[0]));
  printf("                 up                      down\n");
  printf("det        sum        sumerr        sum        sumerr\n");

  for(i=0;i<2;i++)
    printf(" %d      %8.2f    %8.2f     %8.2f    %8.2f\n",
	   x[i]+1,tru[n][x[i]][0],truerr[n][x[i]][0],
	          tru[n][x[i]][1],truerr[n][x[i]][1]);
  for(i=0;i<2;i++)
    for(j=0;j<2;j++)
      if(tru[n][x[i]][j] == 0.0 || truerr[n][x[i]][j] == 0.0)
	return(-1);
    
/* ratio 1 2 */
    rho = sqrt ((tru[n][x[0]][0] * tru[n][x[1]][1]) /
		(tru[n][x[0]][1] * tru[n][x[1]][0]));

    eps = (rho - 1.0) / (rho + 1.0);

/* errors */

    for (i = 0; i < 2; i++)
      for (j = 0; j < 2; j++)
	dsqr[i][j] = pow ((truerr[n][x[i]][j] / tru[n][x[i]][j]), (double) 2);

    drho = 0.5 * rho * sqrt (dsqr[0][0] + dsqr[0][1]
			     + dsqr[1][0] + dsqr[1][1]);

    deps = 2*drho / ((rho + 1.0)*(rho + 1.0));

    printf("\n\tRO TOTAL = %9.6f  +- %f\n",rho,drho);
    printf("\tepsilon  = %9.6f  +- %f\n",eps,deps);

/* output to disk */

    if((op=fopen(sadatfile,"a")) == 0) {
	printf("cannot open sa.dat\n");
	return(0);
    }
    fprintf(op,"\nFile: %s  Peak: %.2f\n",spc[1]->title,calbrt(ppos[0]));
    fprintf(op,"                 up                      down\n");
    fprintf(op,"det        sum        sumerr        sum        sumerr\n");

    for(i=0;i<2;i++)
      fprintf(op," %d      %8.2f    %8.2f     %8.2f    %8.2f\n",
	     x[i]+1,tru[n][x[i]][0],truerr[n][x[i]][0],
	            tru[n][x[i]][1],truerr[n][x[i]][1]);

    fprintf(op,"\n\tRO TOTAL = %9.6f  +- %f\n",rho,drho);
    fprintf(op,"\tepsilon  = %9.6f  +- %f\n",eps,deps);
    repeat = 1;
    fclose(op);
    return(0);
}


void sratio(int n)
{   
    register int i,j;
    extern double slope, dslope;
    extern double eta,deta;
    double temp, sltmp;
    double fi, dfi;		/* precession angle */
    double ep[4],dep[4];
    double sum, dif,sume,sumv,weight;
    double eps,deps;
    FILE  *op;

    /* calculate effect for each detector */
    /* Random counts are not included in st */

    for(i=0;i<4;i++) {
      dif = (i % 2) ? st[n][i][1]-st[n][i][0] : st[n][i][0]-st[n][i][1]; 
      sum =  st[n][i][0]+st[n][i][1];
      ep[i]  = dif/sum;
      dep[i] = fabs(ep[i])*sqrt(sum/(dif*dif) + 1/sum);
      printf("\n   eps(%d) = %f(%f)",i+1,ep[i],dep[i]);
    }
    sume = 0.0;
    sumv = 0.0;
    for(i=0;i<4;i++) {
	weight = 1.0/(dep[i]*dep[i]);
	sume += weight;
	sumv += weight*ep[i];
    }
    eps = sumv/sume;			 /* weighted mean */       
    deps = sqrt(1.0/sume);		 /* mean error */
    printf("\n\tepsilon  = %9.6f  +- %f\n",eps,deps);

    if (fabs(slope) > 0.0001) {	/* calculate precession angle fi */
      fi = eps / slope;
      temp = deps/eps;
      sltmp = (dslope / slope);
      dfi = fabs(fi) * sqrt(temp*temp + sltmp*sltmp);
      printf("\tslope            = %10.5f(%f) \n",slope,dslope);
      printf("\tPrecession angle = %10.5f(%f) rad\n",fi,dfi);
    }

/* output to disk */

    if((op=fopen(sadatfile,"a")) == 0) {
	printf("cannot open sa.dat\n");
	return;
    }
    for(i=0;i<4;i++)
      fprintf(op,"\n   eps(%d) =  %10.5f (%f)",i+1, ep[i],dep[i]);
	      
    fprintf(op,"\n\tepsilon  = %9.6f  +- %f\n",eps,deps);

    if(fabs(slope) > 0.001) {
      fprintf(op,"\tEta parameter    = %10.5f(%f)\n",eta,deta);
      fprintf(op,"\tslope            = %10.5f(%f) \n",slope,dslope);
      fprintf(op,"\tPrecession angle = %10.5f(%f) rad\n",fi,dfi);
    }
    repeat = 1;
    fclose(op);
    srflag = 0;
}

/*
 * n == 0 output for up
 * n == 1 output for down
 *
 *  06/15/96  G.Kumbartzki
 */

void udoutput(int n,int mode)
{
    extern double chi;
    register int i;
    static char mfold[4][11] = {"singlet","doublet","triplet",
				  "quadruplet"};
    FILE  *op;

    if(mode == 1) {		          /* integration */

      printf("%s pos: %.2f  sum: %.1f(%.1f)  total: %.1f(%.1f)  bp[0]: %.1f\n",
	     n?"DOWN":"UP  ",calbrt(ppos[np]),
	     tru[np][nd][n],truerr[np][nd][n],sumt,sqrt(sumt), bp[0]);
      
      if((op=fopen(sadatfile,"a")) != NULL ) {

	if(repeat)			   /* repeat headline */
	  fprintf(op,
		  "\n     peak#    energy    centroid     peaksum       total        marker\n");
	if(n == 0) fprintf(op,"\nup  ");
	if(n == 1) fprintf(op,"down");
	fprintf(op,"%4d%12.2f%11.2f%13.1f%11.1f%9d%6d%6d\n",
		npeak,calbrt(ppos[np]),ppos[np],tru[np][nd][n],sumt,
		pmarks[0],pmarks[1],
		pmarks[2]);
	fprintf(op,"%31.2f%13.1f%11.1f%9d%6d%6d\n",
		cenerr,truerr[np][nd][n],sqrt(sumt),pmarks[3],
		pmarks[4],pmarks[5]);
	fprintf(op,"     randoms subtracted: %.1f ",rsum);
	fprintf(op,"     background: %7.3f",bp[0]);
	for(i=1;i<pmarks[10]+1;i++)
	  fprintf(op,"  %6.3f",bp[i]);
	fprintf(op,"\n");
	repeat = 0;
      }
    }
    else if(mode == 2) {		  /* gauss fit */
    
      for(i=0;i<npks;i++) {
	printf("%s pos: %.2f    sum:  %.1f(%.1f)    fwhm:  %.2f(%.2f)\n",
	       n?"DOWN":"UP  ",calbrt(gpar[3*i+1]), gpar[3*i],sigm[3*i],
	       hptr->sl*gpar[3*i+2],hptr->sl*sigm[3*i+2]);
      }
      printf("chisq: %.3f\n",chi);

      if((op=fopen(sadatfile,"a")) != NULL ) {
	fprintf(op,"\n\n     GAUSS fit: %s\tchi = %2.3f\n",
		mfold[npks-1],chi);
	fprintf(op,"     Peak marker ");
	for(i=6;i<10;i++) {
	    if(pmarks[i] != -1)
	      fprintf(op," %6d",pmarks[i]);
	}
	fprintf(op,"     background: %7.3f",bp[0]);
	for(i=1;i<pmarks[10]+1;i++)
	  fprintf(op,"  %6.3f",bp[i]);
	fputs("\n\n     peak #  energy    centroid   peaksum    fwhm    en_fwhm",op);
	if(n == 0) fprintf(op,"\n\nup  ");
	if(n == 1) fprintf(op,"down");

	for(i=0;i<npks;i++) {
	  if(i > 0) fprintf(op,"    ");
	  fprintf(op," %3d   %8.2f   %8.2f   %8.2f   %6.2f%9.2f\n",
		  i+1,calbrt(gpar[3*i+1]),gpar[3*i+1],
		  gpar[3*i],gpar[3*i+2],hptr->sl*gpar[3*i+2]);
	  fprintf(op,"%19.2f   %8.2f   %8.2f   %6.2f%9.2f\n",
		  hptr->sl*sigm[3*i+1],sigm[3*i+1],sigm[3*i],
		  sigm[3*i+2],hptr->sl*sigm[3*i+2]);
	}					/* end of for */
	repeat = 1;
      }
    }

    else if(mode == 3) {
      printf("%s pos: %.2f    sum: %.1f(%.1f)  randoms: %.1f\n",
	     n?"DOWN":"UP  ",calbrt(ppos[np]),
	     tru[np][nd][n],truerr[np][nd][n],rsum);

      if((op=fopen(sadatfile,"a")) != NULL ) {
	if(repeat)			   /* repeat headline */
	  fprintf(op,
		  "\n     peak#    energy    centroid       peaksum      randoms        marker\n");
	fprintf(op,"%s %4d%12.2f%11.2f%11.1f(%.2f)%7.1f%9d%6d\n",
		n?"down":"up  ",
		npeak,calbrt(ppos[np]),ppos[np],
		tru[np][nd][n],truerr[np][nd][n],rsum,
		pmarks[0],pmarks[1]);
	repeat = 0;
      }
    }
    fclose(op);
}

void bkratios(int n)
{
    register int   i, j, k;

    double  ro[4], dro[4], dsqr[4][2];
    double  rho, drho, rhochk, drhochk;
    double  eps, epschk, deps, depschk;
    FILE  *op;

    printf ("\n Back Ground of region between marks 0 - 5 only\n");

    for (i = 0; i < 4; i++)
      printf (" %d      %8.2f    %8.2f     %8.2f    %8.2f\n",
	      i + 1, bks[n][i][0], sqrt(bks[n][i][0]),
	      bks[n][i][1], sqrt(bks[n][i][1]));
    /* ratio 1 4 */
    ro[0] = sqrt ((bks[n][0][0] * bks[n][3][1]) /
		  (bks[n][0][1] * bks[n][3][0]));
    /* ratio 2 3 */
    ro[1] = sqrt ((bks[n][1][0] * bks[n][2][1]) /
		  (bks[n][1][1] * bks[n][2][0]));

    rho = sqrt (ro[0] / ro[1]);/* 14/23 */

    /* ratio 1 3 */
    ro[2] = sqrt ((bks[n][0][0] * bks[n][2][1]) /
		  (bks[n][0][1] * bks[n][2][0]));
    /* ratio 2 4 */
    ro[3] = sqrt ((bks[n][1][0] * bks[n][3][1]) /
		  (bks[n][1][1] * bks[n][3][0]));
    
    rhochk = sqrt (ro[2] / ro[3]);/* 13/24 */

    eps = (rho - 1.0) / (rho + 1.0);
    epschk = (rhochk - 1.0) / (rhochk + 1.0);

    /* errors */

    for (i = 0; i < 4; i++)
      for (j = 0; j < 2; j++)
	dsqr[i][j] = 1.0/bks[n][i][j];

    dro[0] = 0.5 * ro[0] * sqrt (dsqr[0][0] + dsqr[0][1]
				 + dsqr[3][0] + dsqr[3][1]);
    
    dro[1] = 0.5 * ro[1] * sqrt (dsqr[1][0] + dsqr[1][1]
				 + dsqr[2][0] + dsqr[2][1]);

    dro[2] = 0.5 * ro[2] * sqrt (dsqr[0][0] + dsqr[0][1]
				 + dsqr[2][0] + dsqr[2][1]);

    dro[3] = 0.5 * ro[3] * sqrt (dsqr[1][0] + dsqr[1][1]
				 + dsqr[3][0] + dsqr[3][1]);


    drho = 0.5 * rho * sqrt (pow (dro[0] / ro[0], (double) 2)
			     + pow (dro[1] / ro[1], (double) 2));

    drhochk = 0.5 * rhochk * sqrt (pow (dro[2] / ro[2], (double) 2)
				   + pow (dro[3] / ro[3], (double) 2));

    deps = 2*drho / ((rho + 1.0)*(rho + 1.0));
    depschk = 2*drhochk / ((rhochk + 1.0)*(rhochk + 1.0));

    printf ("\n\tro 1,4   = %8.5f  +- %8.5f\n", ro[0], dro[0]);
    printf ("\tro 2,3   = %8.5f  +- %8.5f\n", ro[1], dro[1]);
    printf ("\tro 1,3   = %8.5f  +- %8.5f\n", ro[2], dro[2]);
    printf ("\tro 2,4   = %8.5f  +- %8.5f\n\n", ro[3], dro[3]);
    printf ("\tRO TOTAL = %8.5f  +- %8.5f\n\n", rho, drho);
    printf ("\tepsilon  = %8.5f  +- %8.5f\n", eps, deps);
    printf ("\tepscheck = %8.5f  +- %8.5f\n\n", epschk, depschk);

    /* output to disk */

/* output to disk */

    if((op=fopen(sadatfile,"a")) == 0) {
	printf("cannot open sa.dat\n");
	return;
    }
    fprintf(op,"\nFile: %s  ",spc[1]->title);
    fprintf(op,"Peak: %.2f Back ground of region\n",calbrt(ppos[0]));
    fprintf(op,"                 up                      down\n");
    fprintf(op,"det        sum        sumerr        sum        sumerr\n");

    for(i=0;i<4;i++)
      fprintf(op," %d      %8.2f    %8.2f     %8.2f    %8.2f\n",
	      i+1,bks[n][i][0],sqrt(bks[n][i][0]),
	      bks[n][i][1],sqrt(bks[n][i][1]));

    fprintf(op,"\n\tro 1,4   = %9.6f  +- %f\n",ro[0],dro[0]);
    fprintf(op,"\tro 2,3   = %9.6f  +- %f\n",ro[1],dro[1]);
    fprintf(op,"\tro 1,3   = %9.6f  +- %f\n",ro[2],dro[2]);
    fprintf(op,"\tro 2,4   = %9.6f  +- %f\n\n",ro[3],dro[3]);
    fprintf(op,"\tRO TOTAL = %9.6f  +- %f\n",rho,drho);
    fprintf(op,"\tepsilon  = %9.6f  +- %f\n",eps,deps);
    fprintf(op,"\tepscheck = %9.6f  +- %f\n\n",epschk,depschk);
    fclose(op);
}
