/* Data file: input/output of histogram data for SA
 * 
 * input data type short, int, float
 * output as int or float
 *
 * G.Kumbartzki
 * Jan. 95
 * New histogram structure, dynamic data space allocation
 *
 * 8-20-97
 * to accommodate gamma sphere data with more than 60 spectra
 * size(flhd) is now 1024, changed "data/" to "data/\*" 
 *
 * 5-10-00 added reading of .spe (radware) files
 * 3-20-01 in flhd "his is now eqivalent to data/\*
 * 3-13-04 added read of ascii data files (file ext = .asc)
*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <math.h>
#include "dacf.h"
#include <X11/Xlib.h>
#include <Xm/Xm.h>
#include <Mrm/MrmAppl.h>
#include <Xm/MessageB.h>
#include <Xm/Command.h>
#include <Xm/List.h>

#define SEEK_SET 0      /* Set file pointer to "offset" */
#define SEEK_CUR 1      /* Set file pointer to current plus "offset" */
#define SEEK_END 2      /* Set file pointer to EOF plus "offset" */

#define    k_com_win               6
#define    k_question             14
#define    k_listing              21
#define    k_list_box             22

extern Widget   toplevel;
extern Widget   wdg_arr[];		/* all other widgets */
extern MrmHierarchy s_MrmHierarchy;     /* Mrm database hierarchy ID */
extern MrmType  dummy_class;            /* and class variable. */

extern  char    sadatfile[];	         /* name of sa.dat file */
extern  char   *wptr[];		         /* token word pointers */
extern  char    direc[];	         /* directory path */
extern  int     ntrs;			 /* number of entries */
extern  int     repeat;			 /* make headline in sa.dat */
extern  int     mrks, marks[];	         /* marker information */
extern  int     nhist;			 /* number of histograms in memory */
extern  int     knr;		         /* next free address in data array */
extern  int     idx;                     /* index for spectrum */
extern  float  *hdat;   		 /* data */
extern  struct  hinfo *spc[], *hptr;     /* histogram information */

int dtypelist[] = { sizeof(int),sizeof(short),sizeof(float)};

static char    sperr[] = {"Insufficient data space"};
static char    rerr[]  = {"Data count mismatch"};

extern char    fname[80];      		 /* data file name */
float  *pd;			         /* general data pointer */

extern int  kilo(char*);
extern int  disp();
extern void s_error(char *s);
extern void bsname(char *a, char *b);

struct filehead flhd;			 /* input fileheader */
struct filehead oflhd = {"his/"};        /* output fileheader */

int    spefile;			         /* radware file */
int    ascifile;                         /* asci file */
enum readmode {r,ra,rs,rc,rr} flag;

int readin(enum readmode);
int wfile(int);
int checkname(char*, int);
int set_knr();
int matrix_read();


int readfile() { return(readin(r));}	 /* calls in main */
int readapnd() { return(readin(ra));}	 /* calls in main */
int readspec() { return(readin(rs));}
int readcol()  { return(readin(rc));}
int readrow()  { return(readin(rr));}

int opfile (char *filename)
{
    int fd;
    char errstr[80];
    XmString text;
    char *filext(char*);

    spefile = 0;
    if ((fd = open (filename, 0)) < 0) {
	sprintf (errstr,"cannot open %s",filename);
	perror("");
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	if(wdg_arr[k_com_win] != NULL)
	  XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	fprintf(stderr,"%c",07);
	fflush (stderr);
	return(-1);
    }
    if (filext(filename) != NULL && 
	 strcmp(filext(filename),"spe") == 0 ) {
      spefile = 1;
      return(fd);
    }
    if (filext(filename) != NULL && 
         strcmp(filext(filename),"asc") == 0 ) {
      ascifile = 1;
      return(fd);
    }

				/* read and process fileheader */
    if (read (fd, &flhd, sizeof (flhd)) <= 0) {
	sprintf (errstr,"cannot read file header");
	perror("");
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	if(wdg_arr[k_com_win] != NULL)
	  XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
    }
    if (strncmp (&flhd.name[0], "his/", 4) &&
	strncmp (&flhd.name[0], "data/", 5)) {
      sprintf (errstr,"%s is not a data file", filename);
      text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
      if(wdg_arr[k_com_win] != NULL)
	XmCommandError(wdg_arr[k_com_win], text);
      XmStringFree(text);
      fprintf(stderr,"%c",07);
      fflush (stderr);
      return(-1);
    }
    if (flhd.name[0] == 'd' && flhd.name[5] != '*') {/* old head block size */
        lseek(fd,512,SEEK_SET);	/* reset read pointer */
    }
    return(fd);
}

int readin (enum readmode flag)				 /* read data file */
{
    register int    i, j, k;

    XmString text;
    FILE *fp, *op;

    static  int  nf[NMARKS/2];		 /* begin of cut (matrix) */
    static  int  nl[NMARKS/2];		 /* length of cut (matrix) */
    char    spnam[30];		         /* data file basename */
    char    errstr[80];			 /* error string */
    int     errf;			 /* error flag */
    int     fd;			         /* data file descriptor */
    long    rptr;			 /* read pointer */
    int     dlen;			 /* data length (row length) */
    int     clen;		         /* column lenght (matrix) */
    int     nspec;			 /* number of spectrum */
    int     nx;				 /* last spec index */
    int     nid;		         /* new id */
    int     temp, temp1;	         /* temporary for sorting */
    int     swap;
    int     rec,nb;
    char    chstr[64];
    char    csave;
    char    *chbuf;
    XmString list[NSPEC];

    struct header {     /* this is the unix header format of radware */
      int    rlen;                /* record length = 24 */
      char   namesp[8];           /* file name */
      int    numch;               /* number of channnels  = idim1*/
      int    idim2;               /* second dimension for matrixes */
      int    cc[3];               /* two more numbers + rlen again */
    } inh;

    union {
      struct header inh;
      char cbuf[32];
    } head;

    int actual_read(int,int,int);

    nspec = 0;
    errf = 0;
    nid = 0;
    if(nhist == 1)
      knr = 0;

    switch (flag) {

      case r: 			 /* read whole file */
	if (ntrs < 1)
	  break;
	strcpy (fname, direc);
	strcat (fname, wptr[1]);
	if (ntrs > 1) {
          if (strncmp(wptr[2],"a",1) != 0) /* a for append */
            knr = kilo(wptr[2]);
	}
        else
          knr = 0;
	if(ntrs > 2)
	  nid = atoi(wptr[3]);
	else
	  nid = 0;
	break;

      case ra: 			 /* read whole file and append */
	if (ntrs < 1)
	  break;
	strcpy (fname, direc);
	strcat (fname, wptr[1]);
	nid = 0;
	break;

      case rs: 			 /* read one spectrum at a time */
	if (ntrs < 1)
	  break;
	if (ntrs >= 3) {
	    strcpy (fname, direc);
	    strcat (fname, wptr[1]);
	    nspec  = atoi (wptr[2]);
	    knr = kilo(wptr[3]);
	    if(ntrs > 3)
	      nid = atoi(wptr[4]);
	    else
	      nid = 0;
	}
	else {
	    nspec  = atoi (wptr[1]);
	    if(ntrs > 1)
	      knr = kilo(wptr[2]);
	}
	break;


      case rr: 			 /* read rows of 2-dim file */
      case rc: 			 /* read columns of 2-dim file */

	if (ntrs < 1) {		         /* pick range from marks */
	    if(mrks > 1) {
		j = 0;
		for(i=0;i<mrks;i +=2) {
		    nf[j] = marks[i];
		    nl[j] = marks[i+1] - marks[i] + 1;
		    j++;
		}
		for(i=0;i<j-1;i++)	 /* sort windows */
		  for(k = i+1; k < j; k++)
		    if(nf[i] > nf[k]) {
			temp = nf[i];
			temp1 = nl[i];
			nf[i] = nf[k];
			nl[i] = nl[k];
			nf[k] = temp;
			nl[k] = temp1;
		    }
		nf[j] = -1;		 /* termination */
	    }
	    else {
		sprintf(errstr,"no marks set");
		text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
		XmCommandError(wdg_arr[k_com_win], text);
		XmStringFree(text);
		return(-1);
	    }
	    break;
	}
	if (ntrs == 4) {		 /* complete input */
	    strcpy (fname, direc);
	    strcat (fname, wptr[1]);
	    nf[0] = atoi (wptr[2]);
	    nl[0] = atoi (wptr[3]);
	    nf[1] = -1;
	    knr = kilo(wptr[4]);
	}
	else {				 /* use previous filename */
	    nf[0]  = atoi (wptr[1]);	 /* start */
	    if(ntrs > 1) 
	      nl[0] = atoi (wptr[2]);	 /* stop or how many */
	    else
	      nl[0] = 1;
	    if(ntrs > 2)
	      knr = kilo(wptr[3]);
	    nf[1] = - 1;
	}
	break;

      default: 
	break;
    }
    if(knr < 0 || knr > HSIZ) {	/* check valid k number */
	sprintf (errstr,"Invalid address %d",knr);
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }
    pd = hdat + knr;		/* where the data go */

    nx = 1;
    if(knr == MKNR * 1024)	/* work space */
      nx = 0;
    else if (knr) {
      for(nx=1;nx<nhist;nx++)	/* find next available nx */
	if((spc[nx]->offset + spc[nx]->xma) > knr)
	  break;
    }
    if (nx > NSPEC) {	/* check maximum of spectra */
      sprintf(errstr,"number of spectra > maximum %d",NSPEC);
      text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win], text);
      XmStringFree(text);
      return(-1);
    }
    idx = nx;			/* save for first display */

    if((fd = opfile(fname)) < 0)
      return(-1);
    bsname (fname, spnam);
    j = strlen(spnam);
    i = 0;
    while(spnam[i] != '.' && i <= j) i++;
    spnam[i] = '\0';		/* cut extension */

    if(spefile) {		/* histogram in radware format */
      if(read(fd,&head,sizeof(struct header)) <= 0){
	sprintf(errstr,"read error in input file");
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	close(fd);
	return(-1);
      }
      if (head.inh.rlen != 24) {
	swap = 1;
	for (j = 3; j < 5; ++j) {
	  csave              = head.cbuf[j*4 + 3];
	  head.cbuf[j*4 + 3] = head.cbuf[j*4];
	  head.cbuf[j*4]     = csave;
	  csave              = head.cbuf[j*4 + 2];
	  head.cbuf[j*4 + 2] = head.cbuf[j*4 + 1];
	  head.cbuf[j*4 + 1] = csave;
	}
      }
      else {
	swap = 0;
      }
      dlen = head.inh.numch;
      if(read(fd,&rec,4) != 4 || dlen*4 != rec) {
	sprintf(errstr,"error in input file");
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	close(fd);
	return(-1);
      }
      if (dlen > HSIZ - knr) {	/* check for available data space */
	XmCommandError(wdg_arr[k_com_win],
		       XmStringCreate(sperr,XmSTRING_DEFAULT_CHARSET));
	dlen = HSIZ - knr;
      }
      chbuf = (char *) pd;	/* read spectrum */
      if((nb= read(fd,chbuf,dlen*4)) != dlen*4) {
	sprintf(errstr,"error reading %d data in file",nb);
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	close(fd);
	return(-1);
      }
      if(swap) {
		       /* file has wrong byte order - swap bytes */
	for (j = 0; j < dlen; ++j) {
	  csave          = chbuf[j*4 + 3];
	  chbuf[j*4 + 3] = chbuf[j*4];
	  chbuf[j*4]     = csave;
	  csave          = chbuf[j*4 + 2];
	  chbuf[j*4 + 2] = chbuf[j*4 + 1];
	  chbuf[j*4 + 1] = csave;
	}
      }
      hptr = spc[nx];
      strcpy(hptr->title,spnam);
      hptr->id = nid;
      hptr->offset = knr;
      hptr->xma = dlen;
      if(nx >= nhist)
	nhist++;
    }
    else if ( ascifile ) {
      if((fp = fdopen(fd,"r")) != NULL){
        dlen = 0;
        do {
          nb = fscanf(fp,"%f",&pd[dlen]);
          dlen++;
        } while(nb == 1);
        hptr = spc[nx];
        strcpy(hptr->title,spnam);
        hptr->id = nid;
        hptr->offset = knr;
        hptr->xma = dlen - 1;
        if(nx >= nhist)
          nhist++;
        fclose(fp);
      }
    }
    else {				   /* not .spe file */

      dlen = flhd.len;
      flhd.sinfo[flhd.maxid].offs = dlen;

      switch (flag) {

        case rs:			   /* read one spectrum only */
	  for(j=0;j<flhd.maxid;j++)        /* check whether spectrum exists */
	    if(flhd.sinfo[j].id == nspec)
	      break;
	  if(j == flhd.maxid) {
	    sprintf (errstr,"No spectrum %d in file %s",nspec,fname);
	    text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	    XmCommandError(wdg_arr[k_com_win], text);
	    XmStringFree(text);
	    close(fd);
	    return(-1);
	  }
	  dlen = flhd.sinfo[j+1].offs - flhd.sinfo[j].offs;
	  if (dlen > HSIZ - knr) {	/* check for available data space */
	    XmCommandError(wdg_arr[k_com_win],
			   XmStringCreate(sperr,XmSTRING_DEFAULT_CHARSET));
	    dlen = HSIZ - knr;
	  }
	  hptr = spc[nx];
	  if(nid)
	    hptr->id = nid;
	  else
	    hptr->id = nspec;
	  strcpy(hptr->title,spnam);	   /* copy name into title */
	  hptr->offset = knr;
	  hptr->xma = dlen;
	  if(nx >= nhist)
	    nhist++;

	  rptr = flhd.sinfo[j].offs;
	  rptr *= dtypelist[flhd.dtype];   /* convert to byte count */
	  if (rptr)
	    lseek (fd, rptr, SEEK_CUR);    /* step over spectra */
	  if ( actual_read(fd,flhd.dtype,dlen) != dlen ) {
	    text = XmStringCreate(rerr,XmSTRING_DEFAULT_CHARSET);
	    XmCommandError(wdg_arr[k_com_win], text);
	    XmStringFree(text);
	  }
	  break;
	  
        case r:
        case ra:

	  for (j = 0;j < flhd.maxid && nx <= NSPEC; j++,nx++){
	    hptr = spc[nx];
	    strcpy(hptr->title,spnam);
	    hptr->id = flhd.sinfo[j].id;
	    hptr->offset = flhd.sinfo[j].offs + knr;
	    hptr->xma = flhd.sinfo[j+1].offs - flhd.sinfo[j].offs;
	  }
	  if(nx > nhist)
	    nhist = nx;
	  dlen = flhd.len;
	  if (dlen > HSIZ - knr) {
	    XmCommandError(wdg_arr[k_com_win],
			   XmStringCreate(sperr,XmSTRING_DEFAULT_CHARSET));
	    dlen = HSIZ - knr;
	  }
	  if ( actual_read(fd,flhd.dtype,dlen) != dlen ) {
	    text = XmStringCreate(rerr,XmSTRING_DEFAULT_CHARSET);
	    XmCommandError(wdg_arr[k_com_win], text);
	    XmStringFree(text);
	    close (fd);
	    return(-1);
	  }
	  break;

        case rr: 			 /* read row of matrix */
        case rc: 			 /* read column of matrix */

	  if(flhd.dtype < 3) {
	    sprintf(errstr,"data file is not matrix");
	    text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	    XmCommandError(wdg_arr[k_com_win], text);
	    XmStringFree(text);
	    close(fd);
	    return(-1);
	  }
	  dlen =  flhd.sinfo[0].offs;
	  clen =  flhd.sinfo[0].id;
	  if(dlen == 0){		/* try symmetric matrix */
	    dlen = sqrt((double)flhd.len);
	    clen = dlen;
	  }

	  i = 0;
	  while (nf[i] != -1) {
	    if (nf[i] < 0 || nf[i] > clen) {
	      errf++;
	      sprintf (errstr," Numbers %d out of range",nf[i]);
	    }
	    i++;
	  }
	  if(errf) {
	    text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	    XmCommandError(wdg_arr[k_com_win], text);
	    XmStringFree(text);
	    close(fd);
	    return(-1);
	  }
	  if(matrix_read(fd,flhd.dtype,dlen,clen,flag,nf,nl) < 0) {
	    text = XmStringCreate(rerr,XmSTRING_DEFAULT_CHARSET);
	    XmCommandError(wdg_arr[k_com_win], text);
	    XmStringFree(text);
	    close(fd);
	    return(-1);
	  }
	  hptr = spc[nx];
	  hptr->id = nf[0];
	  strcpy(hptr->title,spnam);
	  hptr->offset = knr;
	  if(flag == rc)
	    dlen = clen;
	  hptr->xma = dlen;
	  if(nx >= nhist)
	  nhist = nx + 1;
	  break;

        default: 
	  break;
      }
    }
    knr += dlen;
    close (fd);

    if ((op = fopen (sadatfile, "a")) != NULL) {
	fprintf (op, "\n\nFILE: %s ",fname);
	nspec ? fprintf (op, "SPEC:%d\n", nspec) : fprintf (op, "\n");
	repeat = 1;
	fclose (op);
    }
    if(wdg_arr[k_list_box] != NULL && XtIsManaged(wdg_arr[k_list_box])) {
      for (i=1;i<nhist;i++){
	sprintf(chstr,"%3d %-4d  %-12s %4dk",
		i,spc[i]->id,spc[i]->title,spc[i]->offset/1024);
	list[i-1] = XmStringCreateLocalized(chstr);
      }
      i--;
      XtVaSetValues (wdg_arr[k_listing],XmNitems, list,
		     XmNitemCount, i,
		     XmNmustMatch, True,
		     NULL);
      while (--i >= 0)
	XmStringFree(list[i]);
    }
    sprintf(chstr,"%d",idx);
    strcpy(wptr[1],chstr);
    ntrs = 1;
    disp();
    return(0);
}

int actual_read(int fld,int dtyp,int num)
{
    register int i;
    char   dbuf[BUFSIZ];
    int   *intptr;
    short *shptr;
    int    nb, nd;
    int    total;
    FILE * op, *fopen ();		 /* sa.dat */

    total = 0;
    dtyp %= 3;
    switch (dtyp) {
      case 0:			/* int */
	intptr = (int *)dbuf;
	do {
	    nb = read(fld,dbuf,BUFSIZ);
	    if(nb == 0) break;
	    nd = nb>>2;
	    if((total + nd) > num)
	      nd = num - total;
	    for(i=0;i<nd;i++)
	      *pd++ = (float) intptr[i];
	} while ((total += nd) < num);
	return(total);

      case 1:			/* short */
	shptr = (short *)dbuf;
	do {
	    nb = read(fld,dbuf,BUFSIZ);
	    if(nb == 0) break;
	    nd = nb>>1;
	    if((total + nd) > num)
	      nd = num - total;
	    for(i=0;i<nd;i++)
	      *pd++ = (float) shptr[i];
	} while ((total += nd) < num);
	return(total);

      case 2:
	nb = read(fld,pd,num<<2);
	return(nb>>2);

      default:
	return(-1);
    }
}

int matrix_read(int fld,int dtyp,int numx,int numy,enum readmode flag, int *nf,int *nl)
{
    register int i,j;
    long   rptr;
    int   *intptr;
    unsigned short *shptr;
    float *flptr;
    float *ps;			/* data pointer duplicate */
    int    nb;
    int   *snf, *snl;
    static char sp[] = {"could not allocate space"};

    ps = pd;			/* save pointer */
    dtyp %= 3;
    snf = nf;			/* save begin of array */
    snl = nl;

    switch (dtyp) {

      case 0:			/* int */
	if ((intptr = (int *)calloc(numx,sizeof(int))) == (int*)0) {
	  printf("%s\n",sp);
	  return(-1);
	}
	nb = numx * sizeof(int);
	for(i=0;i<numx;i++)
	  pd[i] = 0.0;

	if(flag == rr) {
	  while (*nf != -1) {
	    rptr = *nf * nb + sizeof(flhd);
	    lseek(fld,rptr,SEEK_SET);
	    if (read(fld,intptr,nb) != nb) {
	      printf("%s\n",rerr);
	      return(-1);
	    }
	    pd = ps;
	    for(i=0;i<numx;i++)
	      *pd++ += (float) intptr[i];
	    while(--(*nl)) {
	      if (read(fld,intptr,nb) != nb) {
		printf("%s\n",rerr);
		return(-1);
	      }
	      pd = ps;
	      for(i=0;i<numx;i++)
		*pd++ += (float) intptr[i];
	    }
	    nf++;
	    nl++;
	  }
	}
	else if (flag == rc) {
	  for(i=0;i<numy;i++) {
	    if (read(fld,intptr,nb) != nb) {
	      printf("%s\n",rerr);
	      return(-1);
	    }
	    nf = snf;
	    nl = snl;
	    while (*nf != -1) {
	      for(j = *nf;j < *nf + *nl; j++)
		pd[i] += (float)intptr[j];
	      nf++;
	      nl++;
	    }
	  }
	}
	free(intptr);
	break;

      case 1:			/* short */

	if ((shptr = (unsigned short int *)calloc(numx,sizeof(int))) == 
	    (unsigned short int*)0) {
	    printf("%s\n",sp);
	    return(-1);
	}
	nb = numx * sizeof(short);
	for(i=0;i<numx;i++)
	  pd[i] = 0.0;

	if(flag == rr) {
	  while (*nf != -1) {
	    rptr = *nf * nb + sizeof(flhd);
	    if(lseek(fld,rptr,SEEK_SET) < 0)
	      perror("lseek");
	    if (read(fld,shptr,nb) != nb) {
	      printf("%s\n",rerr);
	      return(-1);
	    }
	    pd = ps;
	    for(i=0;i<numx;i++)
	      *pd++ += (float) shptr[i];
	    while(--(*nl)) {
	      if (read(fld,shptr,nb) != nb) {
		printf("%s\n",rerr);
		return(-1);
	      }
	      pd = ps;
	      for(i=0;i<numx;i++)
		*pd++ += (float) shptr[i];
	    }
	    nf++;
	    nl++;
	  }
	}
	else if(flag == rc) {
	  for(i=0;i<numy;i++) {
	    if (read(fld,shptr,nb) != nb) {
	      printf("%s\n",rerr);
	      return(-1);
	    }
	    nf = snf;
	    nl = snl;
	    while (*nf != -1) {
	      for(j = *nf;j < *nf + *nl; j++)
		pd[i] += (float)shptr[j];
	      nf++;
	      nl++;
	    }
	  }
	}
	free(shptr);
	break;

      case 2:
	if ((flptr = (float *)calloc(numx,sizeof(float))) == (float*)0) {
	  printf("%s\n",sp);
	  return(-1);
	}
	nb = numx * sizeof(float);
	for(i=0;i<numx;i++)
	  pd[i] = 0.0;

 	if(flag == rr) {
	  while (*nf != -1) {
	    rptr = *nf * nb + sizeof(flhd);
	    lseek(fld,rptr,SEEK_SET);
	    if (read(fld,flptr,nb) != nb) {
	      printf("%s\n",rerr);
	      return(-1);
	    }
	    pd = ps;
	    for(i=0;i<numx;i++)
	      *pd++ += flptr[i];
	    while(--(*nl)) {
	      if (read(fld,flptr,nb) != nb) {
		printf("%s\n",rerr);
		return(-1);
	      }
	      pd = ps;
	      for(i=0;i<numx;i++)
		*pd++ += flptr[i];
	    }
	    nf++;
	    nl++;
	  }
	}
	else if(flag == rc) {
	  for(i=0;i<numy;i++) {
	    if (read(fld,flptr,nb) != nb) {
	      printf("%s\n",rerr);
	      return(-1);
	    }
	    nf = snf;
	    nl = snl;
	    while (*nf != -1) {
	      for(j = *nf;j < *nf + *nl; j++)
		pd[i] += flptr[j];
	      nf++;
	      nl++;
	    }
	  }
	}
	free(flptr);
	break;
      default:
	return(-1);
    }
    return(0);
}

int wrtf0() {return(checkname(wptr[1],0));}	/* w */
int wrtf2() {return(checkname(wptr[1],2));}	/* wf */

int checkname(char* filnam, int wmode)
{
  int ffd;

    if((ffd = open(filnam,2)) > 3) {
      if(wdg_arr[k_question] == NULL) {
	if (MrmFetchWidget(s_MrmHierarchy, "question",
			   toplevel, &wdg_arr[k_question],
			   &dummy_class) != MrmSUCCESS) {
	  s_error("can't fetch Question Message Box");
	  return(-1);
	}
      }
      close(ffd);
      XtManageChild(wdg_arr[k_question]);
      return(-1);
    }
    else {
      wfile(wmode);
    }
    return(0);
}
 
int wfile (int mode)
{				 /* write data into specified file */
  register int    i, j;
  int         fd;
  int         dbuf[512];
  char        str[32];
  static int  dlen = 1024;
  static int  start;
  XmString text;


  if(ntrs < 1) {
    text = XmStringCreate("? cmd",XmSTRING_DEFAULT_CHARSET);
    XmCommandError(wdg_arr[k_com_win], text);
    XmStringFree(text);
    return(-1);
  }

  if(ntrs == 1) {		/* file name only default is workarea */
    hptr = spc[0];
    pd = hdat + hptr->offset;
    oflhd.maxid = 1;
    dlen  = hptr->xma;
    oflhd.nrun = 0;
    oflhd.sinfo[0].id = hptr->id;
    start = MKNR*1024;
  }
  else {
    start = kilo(wptr[2]);	 /* start addr */
    pd = hdat + start;
    if (ntrs > 2)
      dlen = kilo(wptr[3]);	 /* length */
    if (pd + dlen > &hdat[HSIZ]) {
      dlen = (&hdat[HSIZ] - pd) >> 2;
      printf ("Data count reduced to %d\n", dlen);
    }
    if(ntrs > 3)		/* run number */
      oflhd.nrun = atoi(wptr[4]);
  }
  if((fd = creat(wptr[1],0644)) < 0) {
    sprintf(str,"cannot create file %s",wptr[1]);
    perror(str);
    return(-1);
  }
  bsname(wptr[1],str);
  strncpy(&oflhd.name[4],str,11);
  flhd.name[15] = '\0';
  oflhd.len = dlen;
  oflhd.sinfo[0].offs = 0;

  if(start != MKNR*1024) {
    for(i=1;i<MKNR;i++)	         /* get i */
      if(start == spc[i]->offset)
	break;

    j = 0;
    do {
      oflhd.sinfo[j].id = spc[i]->id;
      oflhd.sinfo[j+1].offs = oflhd.sinfo[j].offs + spc[i]->xma;
      j++;
      i++;
    } while(spc[i]->xma > 0 && oflhd.sinfo[j].offs < dlen && j < SPEC);
    oflhd.maxid = j;
  }

  for(i=oflhd.maxid; i < SPEC; i++) {
    oflhd.sinfo[i].id = 0;
    oflhd.sinfo[i].offs = 0;
  }

  oflhd.dtype = mode;
  write (fd, &oflhd, sizeof (oflhd));

  if(mode == 2) {		 	 /* float */
    write (fd, pd, dlen << 2);
  }
  else if (mode == 0) {		 /* int */
    for(i=0; i<dlen; i += 512) {
      for(j=0; j<512; j++)
	dbuf[j] = (int) pd[i+j];
      write (fd, dbuf, 2048);
    }
    for(j=0;j+i<dlen;j++)
      dbuf[j] = (int) pd[i+j];
    write (fd, dbuf, j<<2);
  }
  close (fd);
  printf ("\n%d (%s) data in %d spectra written to disk\n",
	  dlen,mode?"real":"int",oflhd.maxid);
  return(0);
}

int wspe ()
{
   FILE    *fp;
   int     tmp[6];   
   char    err[64];
   float  *pd;
   char    filename[64];
   char    str[16], *s;
   int     flngth;
   XmString text;

   if(ntrs < 1) {
     text = XmStringCreate("? cmd",XmSTRING_DEFAULT_CHARSET);
     XmCommandError(wdg_arr[k_com_win], text);
     XmStringFree(text);
     return(-1);
   }
   strcpy(filename,wptr[1]);
   bsname(wptr[1],str);
   s = &str[strlen(str)-4];
   if(strcmp(s,".spe"))
     strcat(filename,".spe");
				/* output length given */
   flngth = (ntrs == 2) ? kilo(wptr[2]) : hptr->xma;
   if(flngth == 0 || flngth > WRKSIZ)
     flngth = hptr->xma;

   hptr = spc[0];
   pd = hdat + hptr->offset;

   if ( ( fp = fopen ( filename , "w+b" ) ) == NULL ) {
     sprintf(err, "open new .spe file : %s\n" , filename );
     perror(err);
     return (1);
   } 
   sprintf (err,"writing to .spe file : %s\n" , filename );
   tmp[0] = 24;			/* record length */
   if ( fwrite ( tmp , 4 , 1 , fp ) < 1 ) {
     perror(err);
     return (1);
   }   
   if (fwrite (str , 8 , 1 , fp ) < 1 ) { /* filename */
     perror (err);
     return (1);
   }   
   tmp[0] = flngth;		/* idim1 */
   tmp[1] = 1;			/* idim2 */
   tmp[2] = 1;
   tmp[3] = 1;
   tmp[4] = 24;			/* recl */
   tmp[5] = flngth*4;   	/* nbytes */

   if ( fwrite (tmp , 24 , 1 , fp ) < 1 ) {
     perror(err);
     return (1);
   }  
   if ( fwrite ( pd , tmp[5] , 1 , fp ) < 1 ) {
     perror(err);
     return (1);
   }
   if ( fwrite ( &tmp[5] , 4 , 1 , fp ) < 1 ) {
     perror(err);
     return (1);
   }   
   fclose ( fp );
   return (0);
} 

int wasci ()
{
   FILE    *fp;
   int     i;
   char    err[64];
   float  *pd;
   char    filename[64];
   int     flngth;		/* # of data written */
   char    str[16];
   XmString text;

   if(ntrs < 1) {
     text = XmStringCreate("? cmd",XmSTRING_DEFAULT_CHARSET);
     XmCommandError(wdg_arr[k_com_win], text);
     XmStringFree(text);
     return(-1);
   }
   strcpy(filename,wptr[1]);
   /* output length given */
   flngth = (ntrs == 2) ? kilo(wptr[2]) : hptr->xma;
   if(flngth == 0 || flngth > WRKSIZ)
     flngth = hptr->xma;


   hptr = spc[0];
   pd = hdat + hptr->offset;

   if ( ( fp = fopen ( filename , "w+b" ) ) == NULL ) {
     sprintf(err, "open new ascii file : %s" , filename );
     perror(err);
     return (1);
   } 
   for(i=0;i<flngth;i++)
     fprintf(fp,"%.1f\n", pd[i]);
   fclose ( fp );
   printf("%d data written to %s\n",flngth,filename);
   return (0);
} 

int set_flhd()
{
  register int i;
  int inc;

  if(ntrs == 0) {
    oflhd.maxid = 1;
  }
  else {
    if((oflhd.maxid = atoi(wptr[1])) <= SPEC) {
      inc = spc[0]->xma / oflhd.maxid;
      for(i=1;i<oflhd.maxid;i++) {
	oflhd.sinfo[i].id = oflhd.sinfo[i].id + 1;
	oflhd.sinfo[i].offs = oflhd.sinfo[i-1].offs + inc;
      }
    }
  }
  return(0);
}

int set_hist()
{
  register int i;

  if(ntrs == 0) {
    for(i=1;i <= NSPEC;i++)
      if(spc[i]->xma == 0)
	break;
    nhist = i;
  }
  else if((nhist = atoi(wptr[1])) >= NSPEC)
    return(-1);
  return(0);
}

int set_knr()
{
  register int i;
   char    mesg[80];
   XmString text;

   if(ntrs == 0) {
     sprintf(mesg,"Free memory begins at %dk",knr/1024);
     text = XmStringCreate(mesg,XmSTRING_DEFAULT_CHARSET);
     XmCommandError(wdg_arr[k_com_win], text);
     XmStringFree(text);
     return(0);
   }
   else if((knr = kilo(wptr[1])) >= MKNR*1024) {
     text = XmStringCreate("knr limit 440",XmSTRING_DEFAULT_CHARSET);
     XmCommandError(wdg_arr[k_com_win], text);
     XmStringFree(text);
     return(-1);
   }
   else {
     sprintf(mesg,"New knr = %d",knr);   
     text = XmStringCreate(mesg,XmSTRING_DEFAULT_CHARSET);
     XmCommandError(wdg_arr[k_com_win], text);
     XmStringFree(text);
   }
   return(0);
}
