/* xdisp.c --- 
 * 
 * functions used for setup of display
 * in Xsa
 *
 * g.kumbartzki
 *
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <X11/Xlib.h>

#include "dacf.h"
#include <Xm/Command.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>
#include <Xm/List.h>

#define    k_com_win               6
#define    k_esc                  18
#define    k_setid                19
#define    k_mess                 20

int     sntrs=1;	                /* exported for psfile */

char   chn[] = {"Channels"};
char   eng[] = {"Energy (keV)"};

extern  XFontStruct  *font1,*font2;

extern  char          sxstr[80];        /* Message text line */
extern  int      chnwidth, engwidth;    /* text positions */

extern  int     ylog;
extern  int     yset;
extern  int     yfix;
extern  int     do_index;
extern  int     ov_flag;
extern  int     numbs[];

extern Widget   wdg_arr[];		/* all other widgets */

extern Display *dpy;
extern Window   gwin;
extern GC gc;
extern Pixmap  pixmap;
extern int width,height;
extern struct colorlist color_vec[];

		     /* defined in sa.c */

extern char   *wptr[];			 /* word pointer array */
extern int     ntrs;			 /* number entries */
extern int     idx;		         /* current spectrum id */
extern int     fchan;			 /* first channel */
extern int     lexp;		         /* level count of expansion */
extern int     sfchan[],slen[];	         /* expansion buffer */
extern int     len;		         /* display lenght */
extern int     ytop,ybot,ylen,yzero;
extern int     xzero, xmax, xlen;
extern int     over;		         /* overlay plot */
extern int    *pmarks;          	 /* pointer to current marker arrays */
extern int     marks[];			 /* marker array */
extern int     npks;		         /* number of peaks for gaus fit */
extern int     bktg[][11];		 /* peak marker array */
extern int     npeak;			 /* current peak number */
extern int     pwidth;			 /* 1/2 display width in peak() */
extern int     dsl, dsr;		 /* display  length and range */
extern int     nhist;		         /* number of histograms */
extern int     mrks;		         /* current number of markers */
extern int     mlog;		         /* log scale */
extern int     doflag;		         /* overlay display mode */
extern int     peakmode;	         /* peak mode select */
extern int     udflag;		         /* up/down mode */
extern int     picture;
extern int     noid;
extern int     kilo(char *s);
extern float  *pdat,*opdat,*hdat;        /* his data pointer */
extern float   xinc, yinc;	         /* scale factors */
extern float   ymin, ymax;		 /* maximum of yscale */
extern float   ysmax[],ysmin[];		 /* yscale for overlay spectra */
extern float   deca;		         /* log display  */
extern struct  hinfo *spc[], *hptr;
extern char    ovstr[];		/* identification of overlay spectrum */

extern int     dalbrt(float);
extern int     getid(int);
int disp();

void           setmark(int,int,int,float);
void           hfill(int, int);
void           displa(), xy_axis(), axisbox(), hplot();

int logdis()
{
  if(ntrs)
    deca = fabs(atof(wptr[1]));
  else
    deca = 0.0;
  mlog = TRUE;
  ntrs = 0;
  disp();
  return(0);
}

int lindis()
{
  if(mlog)
    mlog = FALSE;
  ntrs = 0;
  disp();
  return(0);
}
void ylimits (int beg,int end)
{
  register float *p;
  float   *dlen;

  p = pdat + beg;
  dlen = p + end;
  ymax = 0.0;
  ymin = 0.0;
  while (p < dlen) {
    if (*p > ymax)
      ymax = *p;
    else if (*p < ymin)
      ymin = *p;
    p++;
  }
  ymin = fabsf(ymin);	        /* needed for yscale tics */
}

float yscale (float max,float min,float hght)
{
  float    ym, yscf;

  ym = (mlog) ? (LOGD (max)-deca) : max + min;
  ym = (ym < 1.0) ? 1.1 : ym * 1.1;                 /* add 10% */
  yscf = hght / ym;
  yzero = (min && !mlog) ? ceilf(min)*yscf : 0; /* offset in pixels */
  return(yscf);
}

void clear_screen()
{
    XSetForeground (dpy,gc,color_vec[0].pixel);
    XFillRectangle (dpy,pixmap,gc,0,0,width,height);
    XCopyArea(dpy,pixmap,gwin,gc,0,0,width,height,0,0);
    XSetForeground (dpy,gc,color_vec[1].pixel);
}


void displa ()
{
    register int i;
    extern void bkgd(),bgdplt(),fplot();
    extern void intgrt(int);
    extern void gfit();
    char str[32];
    float tsum;

    if (!yfix) {
      ylimits(fchan,len);
      ysmax[0] = ymax;
      ysmin[0] = ymin;
      yinc = yscale(ymax,ymin,(float)ylen);
    }
    clear_screen();
    XSetFont (dpy,gc,font2->fid);
    sprintf(str,"%d:%s:%d",idx,hptr->title,hptr->id);
    XDrawString (dpy,pixmap,gc,xzero+3,ytop+13,str,strlen(str));

    xinc = (float)xlen /(float)len;        /* X-scale factor */
    pmarks = peakmode ? &bktg[npeak][0] : marks;

    hplot();
    xy_axis();
    axisbox();

    for (mrks = 0; mrks < (peakmode ? 10 : NMARKS); mrks++)
      if(pmarks[mrks] == -1)
	break;

    for(i=0;i<mrks;i++)
      if(INSIDE(pmarks[i]))
	setmark(pmarks[i],i,WP,0.0);
    XCopyArea(dpy,pixmap,gwin,gc,0,0,width,height,0,0);
    picture = 1;

    if(peakmode) {
      if(pmarks[10] != -1) {	/* no bkgd */
	bkgd();
	bgdplt();
      }
      if (pmarks[6] != -1) {
	npks = 0;			/* get number of peaks for gfit */
	for (i = 6; i < 10; i++) {
	  if (pmarks[i] == -1)
	    break;
	  if (pmarks[i] > pmarks[2] && 
	      pmarks[i] < pmarks[3])
	    ++npks;
	}
	if (npks) {
	  gfit();
	  fplot();
	}
      }
      else {
	intgrt(0);
      }
    }
    else {
      tsum = 0;
      for(i=0; i<len; i++)
        tsum += pdat[fchan+i];
      sprintf(sxstr,"sum: %d  -  %d %10.1f",fchan,len+fchan-1,tsum);
      XmTextSetString(wdg_arr[k_mess],sxstr);
    }
}

void axisbox()
{
    register int x;
    static char   chn[] = {"Channels"};
    static char   eng[] = {"Energy (keV)"};

    XSetForeground (dpy,gc,color_vec[1].pixel);

    XDrawRectangle(dpy,pixmap,gc,xzero,ytop,xlen,ylen);

    XSetFont (dpy,gc,font1->fid);
    if(hptr->cflag) {			/* energy scale */
	x = (width - XTextWidth (font1,chn, strlen(chn)))/2;
	XDrawString (dpy,pixmap,gc,x,height-8,eng,strlen(eng));
    }
    else {
	x = (width - XTextWidth (font1,eng,strlen(eng)))/2;
	XDrawString (dpy,pixmap,gc,x,height-8,chn,strlen(chn));
    }
    XSetFont (dpy,gc,font2->fid);    
}

float mkround (float val)

/*
 * This routine rounds up a given positive number such that
 * it is some power of ten times either 1, 2, or 5.
 */
{
    int exponent, ix;

    exponent = (int) floor(log10(val));
    if (exponent < 0) {
	for (ix = exponent;  ix < 0; ix++) {
	    val *= 10.0;
	}
    } else {
	for (ix = 0;  ix < exponent; ix++) {
	    val /= 10.0;
	}
    }
    if (val > 5.0) val = 10.0;
    else if (val > 2.0) val = 5.0;
    else if (val > 1.0) val = 2.0;
    else val = 1.0;
    if (exponent < 0) {
	for (ix = exponent;  ix < 0;  ix++) {
	    val /= 10.0;
	}
    } else {
	for (ix = 0;  ix < exponent;  ix++) {
	    val *= 10.0;
	}
    }
    return val;
}

void xy_axis()
{
    register int x, y;
    int         ny,i;
    static char dot[] = {2,2};	        /* for dashed line */
    char        str[24];		/* character string */
    int         strl;
    int         textlength;
    float       fch;
    float       roff;		        /* round up: 1/2 of displayed channel*/
    int         deltx;
    float       delty;
    float       max;
    float       savexinc;
    float       tic, ytic;
    float       dlen, step;		/* temp length */
    static  int tick = 8;		/* length of tick mark */

    fch = fchan;
    dlen = (float)len;
    roff = xinc/2 ;
    if(hptr->cflag) {		/* use energy scale */
        savexinc = xinc;
	fch = fch*hptr->sl + hptr->zr;
	dlen *= hptr->sl;
	xinc /= hptr->sl;
    }
    step = mkround (dlen/11.0);
    if (step < 1.0) {
      step = 1.0;
    }
    deltx = fch / step;		/* round to full integer */
    deltx *= step;

    while ((x = (deltx - fch)*xinc + roff + xzero) < xmax) {

      if(x > xzero + 10) {
	XDrawLine (dpy,pixmap,gc,x,ytop,x,ytop+tick);
	XDrawLine (dpy,pixmap,gc,x,ybot,x,ybot-tick);

	sprintf(str,"%d",deltx);
	strl = strlen(str);
	textlength = XTextWidth (font2,str,strl);
	x -= textlength/2;
	XDrawString (dpy,pixmap,gc,x,ybot+13,str,strl);
      }
      deltx += step;
    }
    if(hptr->cflag)		/* restore scale for hplot */
      xinc = savexinc;

    tic = 0.1;
    max = (ymin > ymax) ? ymin : ymax;
    for (;;) {
	step = tic * 5.0;
	if (step > max)
	    break;
	tic = tic * 5.0;
	step = step * 5.0;
	if (step > max)
	    break;
	tic = tic * 2.0;
    }
    delty = 0.0;
    if (mlog) {
	tic = 1.0;
	delty = 1.0*pow(10.0,(double)deca);	/* deca = 0 is default */
    }
    ytic = tic*yinc;
    if(ytic < 20.0) ytic = 20.0;
    y = ybot + ylog;
    if (yzero) {
	y = ybot - yzero;
	XSetLineAttributes (dpy,gc,0,LineOnOffDash,CapButt,JoinRound);
	XSetDashes (dpy,gc,0,dot,2); 
	XDrawLine (dpy,pixmap,gc,xzero,y,xmax,y);
	XSetLineAttributes (dpy,gc,1,LineSolid,CapButt,JoinRound);
	while ((ybot - y) > ytic) {
	    y += ytic;
	    delty -= tic;
	}
    }
    do {
	XDrawLine (dpy,pixmap,gc,xmax-tick,y,xmax,y);
	XDrawLine (dpy,pixmap,gc,xzero,y,xzero+tick,y);
	if(fabsf(delty) > 0.05 && fabsf(delty) < 5.0 && !mlog)
	  sprintf(str,"%.1f",delty);
	else
	  sprintf(str,"%.0f",delty);
	strl = strlen(str);
	textlength = XTextWidth (font2,str,strl);
	x = xzero - textlength - 3;
	XDrawString (dpy,pixmap,gc,x,y+6,str,strl);
	delty = mlog ? delty*10 : delty + tic;
	if(mlog) {		/* extra tick marks */
	  for(i=2;i<10;i++) {
	    ny = y - ytic*log10((double)i);
	    if(ny > ytop) {
	      XDrawLine (dpy,pixmap,gc,xmax-tick+3,ny,xmax,ny);
	      XDrawLine (dpy,pixmap,gc,xzero,ny,xzero+tick-3,ny);
	    }
	  }
	}
    } while ((y -= ytic) > ytop);
}


void hplot ()				/* show histogram
					 only the visible lines are
					 put on the screen */
{
    register int  i,j;
    int       x, y;		/* current coordinates */
    int       xbf,ybf;		/* x-before & y-before */
    int       npoints;		/* number od XPoints */
    float     tmp;
    float    *pd;		/* temp data pointer */
    int       yp;		/* plot value of y */

    extern  XColor        SkyBlue;

    XPoint    his[5120];

    ylog = mlog ? LOGD(0.0)*yinc : 0;   /* ylog is negative ! */
    yset = ybot - yzero + ylog;

    if(over)
      pd = opdat + fchan;
    else
      pd = pdat + fchan;
    npoints = 0;

    tmp = (mlog ? (LOGD (pd[0])-deca) : pd[0]) * yinc;
    ybf = yset - (int) tmp;
    yp = ybot+1;
    xbf = 0;
    his[npoints].x = xzero+1;
    his[npoints].y = LIM(ybf);
    npoints++;

    for(i=0;i<len;i++) {

      x = i*xinc;
      tmp = (mlog ? (LOGD(pd[i])-deca) : pd[i]) * yinc;
      y = yset - (int)tmp;

      if(x == xbf) {
	if(y < yp)
	  yp = y;
	continue;
      }
      if(yp != ybot+1){
	j = y;
	y = yp;
	yp = j;
      }
      else {
	yp = ybot+1;
      }
      if(y == ybf) {
	xbf = x;
	continue;
      }
      his[npoints].x = xzero + xbf;
      his[npoints].y = LIM(ybf);
      npoints++;
      his[npoints].x = his[npoints-1].x;
      his[npoints].y = LIM(y);
      if(npoints++ > 5100){
	printf("xpoints limit exceeded\n");
	break;
      }
      ybf = y;
      xbf = x;
    }

    his[npoints].x = xzero + xbf;
    his[npoints].y = LIM(ybf);
    npoints++;
    his[npoints].x = his[npoints-1].x;
    his[npoints].y = LIM(yp);
    npoints++;
    his[npoints].x = xmax;
    his[npoints].y = his[npoints-1].y;
    npoints++;

    if(!over) {
      his[npoints].x = xmax;
      his[npoints++].y = ybot;
      his[npoints].x = xzero;
      his[npoints++].y = ybot;
      XSetForeground(dpy,gc,SkyBlue.pixel);
      XFillPolygon(dpy,pixmap,gc,his,npoints,Complex,CoordModeOrigin);
      XSetForeground (dpy,gc,color_vec[1].pixel);
      npoints -= 2;
    }
    if(peakmode) {
      if(pmarks[2] != -1 && pmarks[3] != -1)
	hfill(pmarks[2],pmarks[3]);
      else if (pmarks[0] != -1 && pmarks[1] != -1)
	hfill(pmarks[0],pmarks[1]);
    }
    XDrawLines(dpy,pixmap,gc,his,npoints,CoordModeOrigin);
}

void hfill(int b, int e)	/* fill part of histogram */
{
    register int  i;
    int       x, y;		/* current coordinates */
    int       ybf;		/* y-before */
    int       npoints;
    float     tmp;

    extern  XColor aquamarine1;

    XPoint    his[1024];

    x = (b - fchan)*xinc;
    npoints = 0;
    ybf = ybot;

    for(i=b;i<e+2;i++) {

	x = (i-fchan)*xinc;
	tmp = (mlog ? (LOGD (pdat[i])-deca) : pdat[i]) * yinc;
	if((y = yset - (int)tmp) == ybf)
	  continue;

	his[npoints].x = xzero + x;
	his[npoints].y = LIM(ybf);
	npoints++;
	his[npoints].x = his[npoints-1].x;
	his[npoints].y = LIM(y);
	npoints++;
	if(npoints > 1020)
	  break;
	ybf = y;
    }
    his[npoints].x = his[npoints-1].x;
    his[npoints].y = ybot;
    npoints++;
    XSetForeground(dpy,gc,aquamarine1.pixel);
    XFillPolygon(dpy,pixmap,gc,his,npoints,Complex,CoordModeOrigin);
    XSetForeground (dpy,gc,color_vec[1].pixel);
}

int disp_d()
{
    register int fch, lch;
    int diff;

    pmarks = marks;
    do_index = 0;

    if (ntrs == 1) {		/* input is spectrum id */
	idx = getid(atoi(wptr[1]));
	numbs[0] = idx;
	hptr = spc[idx];
	pdat = hdat + hptr->offset;
    }                        /* show same region in an other spectrum */

    if (ntrs > 1) {	    /* input is fch and lch plus optional max */
      peakmode = 0;
	fch = kilo(wptr[1]);
	fch = dalbrt((float)fch);
	diff = hdat - (pdat + fch);      /* check begin of data array */
	if (diff  > 0)
	  fch += diff;
	lch = kilo(wptr[2]);
	lch = dalbrt((float)lch);
	if (fch >= lch)
	  lch += fch;
	len = lch - fch;
	                           /* make sure to stay in data array */
	if (hptr->offset + fch + len > HSIZ)
	  len = HSIZ - hptr->offset - fch;
	if(len <= 0)
	  return(-1);
	lexp = 0;
	fchan = fch;
	sfchan[0] = fchan;
	slen[0] = len;
	if(ntrs > 2) {
	  ymax = fabs(atof(wptr[3]));
	  yfix = 1;
	}
    }
    displa();			/* redisplay */
    yfix = 0;
    return(0);
}

int disp()			   /* display up to 8 spectra or K */
{

    register int i, k, j;
    int yp;
    int narr[10];
    int savytop, savybot;
    float fylen;
    float tsum;
    char str[32];
    XmString text;

    peakmode = FALSE;
    ymax = 0.0;
    pmarks = marks;
    udflag = FALSE;		/* any display command switches ud off */
    doflag = FALSE;		/* overlay display off */
    yfix = 0;

    do_index = 0;
    if(ntrs < 1) {
	ntrs = sntrs;
    }
    else {
	k = 0;
	for (i = 1; i <= ntrs; i++) {	   /* fill number array */
	    if(strcmp(wptr[i],"-") == 0) {
		do {
		    narr[k] = k ? narr[k-1]+ 1 : 1;
		    if(k > 8) break;       /* upper limit 8 */
		} while (narr[k++] < atoi(wptr[i+1]));
		k--;
		continue;
	    }
	    narr[k++] = atoi(wptr[i]);
	    if(k > 10) break;
	}
	if (narr[k-1] < 0) {	  /* if neg. last entry treated as YMAX */
	    k--;
	    ymax =  -narr[k];
	    ymin = 0.0;
	    if(narr[k-1] < 0) {	/* if two neg entries ymin and ymax asigned*/
	      k--;
	      ymin = -narr[k];
	    }
	    yfix = 1;
	}
	j = 0;
	for(i=0;i<k;i++)
	  if(getid(narr[i]) >= 0)
	    numbs[j++] = narr[i];
	ntrs = j;
	if(ntrs > 8) ntrs = 8;           /* max of 8 spectra */
    }
    if (ntrs <= 0) {
	text = XmStringCreate("no valid input",XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }

    for (mrks=0;mrks < NMARKS;mrks++) {	   /* count marks */
	if(pmarks[mrks] == -1)
	  break;
    }
    savytop = ytop;			   /* save temporary display par */
    savybot = ybot;
    clear_screen();

    fylen = (float) ylen/ (float) ntrs;
    if (yfix)
      yinc = yscale(ymax,ymin,fylen);

    for (i = 0; i < ntrs; i++) {
	ytop = savytop + i*fylen;         /* to prevent round off */
	ybot = ytop + (int)fylen;
	idx = getid(numbs[i]);
	hptr = spc[idx];
	pdat = hdat + hptr->offset;       /* points to begin of
					     current spectrum */
	sprintf(str,"%d:%s:%d",idx,hptr->title,hptr->id);
	fchan = (dalbrt((float)dsr) <= 0) ? 0 : dalbrt((float)dsr);
	len = dsl ? dalbrt((float)dsl) : hptr->xma;
	if (len <= 0)
	  continue;
	lexp = 0;
	sfchan[0] = fchan;
	slen[0] = len;
	if(!yfix) {
	  ylimits(fchan,len);
	  yinc = yscale(ymax,ymin,fylen);
	}
	if(i == 0) {		/* save first pair for psfile */
	  ysmax[0] = ymax;
	  ysmin[0] = ymin;
	}
	xinc = (float)xlen /(float)len;        /* X-scale factor */
	hplot();
	xy_axis();
	if(i+1 != ntrs)			 /* draw dividing line */
	  XDrawLine(dpy,pixmap,gc,xzero,ybot,xmax,ybot);
	for(k=0;k<mrks;k++)
	  if(INSIDE(marks[k]))
	    setmark(marks[k],k,P,0.0);
	XSetForeground (dpy,gc,color_vec[7].pixel); /* black */
	yp = i ? ytop+26 : ytop+13;
	XDrawString (dpy,pixmap,gc,xzero+3,yp,str,strlen(str));
	XSetForeground (dpy,gc,color_vec[1].pixel);
    }
    ytop = savytop;		/* restore display parameters */
    ybot = savybot;
    axisbox();
    XCopyArea(dpy,pixmap,gwin,gc,0,0,width,height,0,0);
    picture = TRUE;
    sntrs = ntrs;
    idx = getid(numbs[0]);	/* set current id to first entry */
    hptr = spc[idx];
    pdat = hdat + hptr->offset;
    tsum = 0;
    for (i=0;i<len;i++)
      tsum += pdat[fchan+i];
    sprintf(str,"sum: %d  -  %d %10.1f",fchan,fchan+len-1, tsum);
    XmTextSetString(wdg_arr[k_mess],str);
    return(0);
}

int disp_o()
{
    register int nx;

    int      addon;
    static int co=1;
    XmString text;
    float *savpdat;
    char str[64];
    extern int colorof(char*);

    savpdat = pdat;
    if(co > 7) co = 0;
    addon = 0;

    if(ntrs < 1 || (nx = getid(atoi(wptr[1]))) == -1) {
	sprintf(str,"unknown id %s",wptr[1]);
	text = XmStringCreate(str,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }
    if (++do_index < 20)
      numbs[do_index] = nx;
    if(wptr[1][1] == '+') {
        strcpy(str,&wptr[1][2]);
	addon = kilo(str);
    }
    opdat = hdat + spc[nx]->offset + addon;

    if(ntrs > 1) {
	co = colorof(wptr[2]);
	XSetForeground (dpy,gc,color_vec[co].pixel);
    }
    else {
	XSetForeground (dpy,gc,color_vec[++co].pixel);
    }
    if(ntrs > 2) {
        if(atoi(wptr[3]) == 0) {
	  ov_flag = TRUE;
	  return(0);
	}
	else {
	    ymax = fabs(atof(wptr[3]));
	    ymin = 0.0;
	}
	yinc = yscale(ymax,ymin,(float)ylen);
    }
    if(do_index < 10) {
      ysmax[do_index] = ymax;
      ysmin[do_index] = ymin;
    }
    sprintf(ovstr,"%d: %s:%d",nx,spc[nx]->title,spc[nx]->id);
    over = TRUE;
    hplot();
    over = FALSE;
    XDrawString (dpy,pixmap,gc,xzero+3,ytop+23,ovstr,strlen(ovstr));
    XCopyArea(dpy,pixmap,gwin,gc,0,0,width,height,0,0);
    doflag = TRUE;
    return(0);
}

int overlay()
{
    register int i, k, j;
    int narr[50];
    int num[50];
    char str[32];
    XmString text;

    if (ntrs < 1) {
	text = XmStringCreate("no valid input",XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }
    k = 0;
    for (i = 1; i <= ntrs; i++) {	   /* fill number array */
      if(strcmp(wptr[i],"-") == 0) {
	do {
	  narr[k] = k ? narr[k-1]+ 1 : 1;
	  if(k>48) break;
	} while (narr[k++] < atoi(wptr[i+1]));
	k--;
	continue;
      }
      narr[k++] = atoi(wptr[i]);
      if(k>48) break;
    }
    j = 0;
    for(i=0;i<k;i++)
      if(getid(narr[i]) >= 0) {
	num[j++] = narr[i];
	if(j > 48) break;
      }
    if (j <= 0) {
	text = XmStringCreate("no valid input",XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }
    ntrs = 1;
    for (i = 0; i<j; i++) {
      sprintf(wptr[1],"%d",num[i]);
      disp_o();
    }
    return(0);
}

int peak()
{
    extern void search();
    int lchan;

    if(ntrs >0) {
      npeak = atoi (wptr[1]);
      if (ntrs > 1)
	pwidth = atoi (wptr[2]);
    }
    if (npeak <0 || npeak > NMARKS || marks[npeak] == -1) {
      XmTextSetString(wdg_arr[k_mess],"Peak not defined!");
      putchar(07);
      fflush(stdout);
      return(-1);
    }
    fchan = marks[npeak] - pwidth/2;
    if (fchan < 0)
      fchan = 0;
    lchan = fchan + pwidth;
    /*
    if (lchan > hptr->xma)
	lchan = hptr->xma;
    */
    if((len = lchan - fchan) == 0)
      return(-1);

    peakmode = TRUE;
    displa ();
    if(wptr[0][0] == 'c')
      search();
    return(0);
}

int set_dsr()
{
    dsr = 0;
    dsl = 0;
    if(ntrs == 1) {
	dsl = kilo(wptr[1]);
    }
    if(ntrs == 2) {
	dsr = kilo(wptr[1]);
	dsl = kilo(wptr[2]);
    }
    ntrs = 0;
    disp();
    return(0);
}

int set_mark()
{
    extern int checkmarks();
    register int i;

    for(i=0;i<ntrs;i++) {
	if(checkmarks()) return(-1);
	pmarks[mrks-1] = dalbrt(atof(wptr[i+1]));
	setmark(pmarks[mrks-1],mrks-1,WP,0.0);
    }
    return(0);
}

int set_id()			/* toogle selection id calling */
{
    noid = 1 - noid;
				/* set state in widget */
    if(wdg_arr[k_setid] != NULL) 
      XmToggleButtonSetState(wdg_arr[k_setid],noid,False);
    return(0);
}

int chid()
{
   char    mesg[80];
   XmString text;

   if(ntrs == 0) {
     sprintf(mesg,"Current id %dk",idx);
     text = XmStringCreate(mesg,XmSTRING_DEFAULT_CHARSET);
     XmCommandError(wdg_arr[k_com_win], text);
     XmStringFree(text);
     return(0);
   }
  hptr->id = atoi(wptr[1]);
  printf("new id = %d\n",hptr->id);
  return(0);
}

int checkmarks()		/* increments number of marks */
{				/* and checks against limit */
    int limit;

    limit = peakmode ? 10 : NMARKS;
    if(++mrks > limit) {
	printf("marker array full\n");
	mrks = limit;
	return(-1);
    }
    return(0);
}

void setmark (int nchan,int i,int mod,float en) /* set mark */
{
    register int   x, y;
    char str[16];

    int strl;
    float  tmp;
    extern int ycoor;		/* position of y cursor */

    if(mod == 2)		/* "erase" marker */
      XSetForeground (dpy,gc,color_vec[0].pixel);
    else
      XSetForeground (dpy,gc,color_vec[4].pixel);

    tmp = (mlog ? (LOGD (pdat[nchan])-deca) : pdat[nchan])* yinc;
    y = ybot - yzero - (int)tmp;
    if(mlog)
      y += LOGD(0.0)*yinc;
    y -= 5;
    if (i < 0 && ycoor < y)
      y = ycoor;		/* draw at cursor pos */
    if (y < ytop + 30)
	y = ytop + 30;

    x = (nchan - fchan + 0.5) * xinc + xzero;

    if(mod)
      XDrawLine (dpy,gwin,gc,x,y,x,y-10);
    XDrawLine (dpy,pixmap,gc,x,y,x,y-10);

    if (i < 0)			/* floating point number */
	sprintf (str, "%.2f", en);
    else			/* integer */
	sprintf (str, "%d", i);
    strl = strlen (str);
    x -= XTextWidth(font2,str,strl)/2;
    if(mod)
      XDrawString (dpy,gwin,gc,x,y-15,str,strl);
    XDrawString (dpy,pixmap,gc,x,y-15,str,strl);
    XSetForeground (dpy,gc,color_vec[1].pixel);
}

int escale()
{

    hptr->cflag = 1 - hptr->cflag;
				/* set state in widget */
    if (wdg_arr[k_esc] != NULL)
      XmToggleButtonSetState(wdg_arr[k_esc],hptr->cflag,False);
    return(0);
}
