/* 
 * Spectrum Analysis using Xlib
 *
 * The user interface is written in UIL
 * xsa.uil
 *
 * Gerfried Kumbartzki
 * Rutgers University
 * Dept. of Physics
 *
 * Jan. 1993
 * 
 * version 1.1 (implementation of workarea at end of data array)
 * version 1.2 added up/down
 * version 2.0 pointer version (shm version xsa -s)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdarg.h>
#include <ctype.h>
#include <time.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <Xm/Xm.h>
#include <Mrm/MrmAppl.h>
#include <Xm/Command.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/ToggleB.h>
#include <Xm/Label.h>
#include <Xm/SelectioB.h>
#include <Xm/List.h>

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/utsname.h>

#include "dacf.h"
#include "saicon.xb"

#define    MAXCOL                  9        /* NUMBER OF REQUESTED COLORS */

#define    k_menu_bar              0
#define    k_file_pdme             1
#define    k_file_menu             2
#define    k_graph_win             3
#define    k_open                  4
#define    k_write                 5
#define    k_com_win               6
#define    k_com_box               7
#define    k_wfilename             8
#define    k_staddr                9
#define    k_length               10
#define    k_bmode                11
#define    k_int                  12
#define    k_float                13
#define    k_question             14
#define    k_opt_pdme             15
#define    k_opt_menu             16
#define    k_rand                 17
#define    k_esc                  18
#define    k_setid                19
#define    k_mess                 20
#define    k_listing              21
#define    k_list_box             22
#define    k_anis_box             23
#define    k_coef1                24
#define    k_coef2                25
#define    k_theta1               26
#define    k_theta2               27
#define    k_theta3               28
#define    k_slope                29
#define    k_opt                  30
#define    k_eta                  31

#define    kmax                   32    /* one more than k_xxx */

#define    k_exit                100
#define    k_quit                101
#define    k_read                102
#define    k_cancel              103
#define    k_list_cancel         107
#define    k_help1               108
#define    k_help2               109
#define    k_help3               110
#define    k_print               111
#define    k_read_anis           112
#define    k_b1                  113
#define    k_b2                  114
#define    k_b3                  115
#define    k_apply_anis          116
#define    k_cancel_anis         117

char sccsid[]="@(#)Spectrum Analysis 2.6.7 (Apr.2015)";

Widget        toplevel;
Widget        help_widget;
Widget        wdg_arr[kmax];		/* all other widgets */

Display      *dpy;		        /* for drawing area */
Window        gwin,tracker;
Pixmap        pixmap,icon;
GC            gc;

Colormap      cmap;
XColor        exact,black,white,SkyBlue,aquamarine1;
XColor        red,yellow,lightgrey,cyan,blue,brown,green;
Cursor        cursor;
XGCValues     xgcv;
XFontStruct  *font1,*font2;

unsigned int  width, height;
struct colorlist color_vec[MAXCOL];

int           screen;
int           depth;
Arg           al[15];			/* global Argument array */
int           ac;
char          sxstr[80];	        /* Message text line */
int           initdone,picture;	        /* flags */

MrmHierarchy s_MrmHierarchy;            /* Mrm database hierarchy ID */
MrmType  dummy_class;                   /* and class variable. */

char *db_filename_vec[] =               /* Mrm hierarchy file list. */
  {"xsa"                                /* There is only one UID file for */
  };                                    /* this application. */
static int db_filename_num =
                (sizeof db_filename_vec / sizeof db_filename_vec [0]);

void s_error(char *s);
void showfile();
static void expose_proc();
static void activate_proc();
static void create_proc();
static int  quit_proc();
static void pull_proc();
static void help_proc();
static void resize_proc();
static void inout_proc();
static void dispatch();
static void open_proc();
static void show_hide_proc();
static void write_file_proc();
static void selection_proc();

static MrmRegisterArg reglist[] = {
    {"expose_proc", (XtPointer) expose_proc},
    {"activate_proc", (XtPointer) activate_proc},
    {"create_proc", (XtPointer) create_proc},
    {"pull_proc", (XtPointer)pull_proc},
    {"show_hide_proc", (XtPointer)show_hide_proc},
    {"quit_proc", (XtPointer)quit_proc},
    {"help_proc", (XtPointer)help_proc},
    {"resize_proc",(XtPointer)resize_proc},
    {"open_proc", (XtPointer)open_proc},
    {"write_file_proc", (XtPointer)write_file_proc},
    {"selection_proc", (XtPointer)selection_proc},
    {"inout_proc", (XtPointer)inout_proc},
    {"dispatch", (XtPointer)dispatch}
};
static int reglist_num = (sizeof reglist / sizeof reglist [0]);
 

extern char *getenv();
extern int uname(struct utsname *buf);
extern int getopt();

char *pspath;
char psname[64];
char timestr[32];		/* time string */
char sadatfile[24];		/* sa.dat file name */

struct command			/* command window processing */
{
   char  *name;
   int   (*function)();
};

float *hdat;              	        /* data array (all data) */
float *pdat, *opdat;                    /* pointer to histogram data */
int    shmflg = 0666;                   /* shared mem permission */
int    fchan = 0;            		/* first channel to display*/
int    lexp = 0;			/* level of expansion count */
int    sfchan[10],slen[10];	        /* stored levels of expansion */
int    len = 1024;		        /* display length */
int    idx;			        /* index of spectrum */
int    xzero, xmax, ytop, ybot;		/* corners of axisbox */
int    yzero;                           /* zero y in pixels */
int    yset;			        /* highest y coord in plot */
int    ylog;			        /* offset for log scale */
int    leftmar  = 60;			/* left margin */
int    rightmar = 20;			/* right margin */
int    topmar   = 20;			/* top margin */
int    botmar   = 40;			/* bottom margin */
int    xlen, ylen;			/* x-pixels, y-pixels */
int    do_index;		        /* index of overlay spectrum */
int    yfix = 0;			/* yscale is given */
float  ymin, ymax;			/* minimum maximum of y-data */
float  ysmax[10],ysmin[10];		/* y for overlay spectra */
int    over = FALSE;		        /* overlay plot flag */
int    nchan;			        /* channel number */
int    xcoor, ycoor;		        /* cursor coordinates */
int    numbs[20];		        /* general number array */
float  xinc = 1.0, yinc = 1.0;		/* scale factors */
float  deca = 0.0;		        /* subtract dekade in log display */

 /* various flags */

int     repeat;
int     noid;   		         /* switch between index and id */
int     npeak = -1;			 /* peak # */
int     peakmode;		         /* peak analysis */
int     calmode;			 /* calibration */
int     tcnt;				 /* temporary counter used in cal */
int     udflag;			         /* updown mode */
int     mv_flag;			 /* move marks */
int     ov_flag = FALSE;                 /* overlay at cross hair cursor */
int     wmode; 		                 /* write mode: datatype */
int     doflag = FALSE;			 /* overlay inactiv */
int     fixedslope = FALSE;	         /* slope typed in */

 /* up/down variables */
int     det;    		         /* detector # */
int     randoms = 1;			 /* switch randoms on/off */
int     bmode = 0;		         /* individual or averaged BKGD */

 /* anisotropy variables */
double Anm[3][2] = { 0.7143, -1.7143, 0.5102, -0.3673, 0.0, 0.0};
double dAnm[3][2] = {0.0, 0.0, 0.0, 0.0, 0.0,0.0};
double anm[2];
int    ispin = 0;                        /* preset can be changed in Options */
double thet1 = 59.0;		         /* 1. anisotropy angle */
double thet2 = 75.0;		         /* 2. anisotropy angle */
double thet3 = 67.0;		         /* precession angle */
double q[2] = {1.0, 1.0};                /* qs detecter attenuations */
double slope = 0.0;		         /* calculated or provided slope */
double dslope = 0.0;		         /* error in slope */
double eta = 0.0;		         /* eta parameter for A2/A4 */
double deta = 0.0;		         /* error in eta */
double creff=1.0,dcreff=0.0;    	 /* efficiency corection for ratio */

 /* marker variables */
int     mrks;   			 /* marker number */
int     marks[NMARKS];                   /* current marker arrays */
int     bktg[NMARKS][11];		 /* peak marker */
int    *pmarks;			         /* pointer to current marker array */

 /* display variables */
int     pwidth = 120;  		         /* width of peak window */
int     nhist = 1;   			 /* # of spectra in memory */
int     knr;			         /* next free element in data array */
int     dsl, dsr;		         /* display length and range */
int     mlog;			         /* log scale */
int     npks;				 /* number of peaks in gaussfit */
int     co = 4;				 /* color index red */
int     shmversion = 0;		         /* shm version flag */
float   en[2], cen[2];			 /* calibration parameters */

double  bp[6];				 /* background fit parameters */
double  shft = 0.0, gmfac = 1.0;         /* shift & gain match factor */

struct  hinfo *spc[NSPEC+1], *hptr;      /* histogram information */


int        exitus(), inputparser(), fit(), integrate(), mv_marks();
extern int forge(), add(), sub(), divide(), reset();
extern int intout(), peak(), pridata(), chid(), anis();
extern int g_match(), set_gmf(), rebin(), shift(), mcal(), gmfile();
extern int wrtf0(), wrtf2(), calibrate(), wcal(),rcal(), wmarks(), rmarks();
extern int input (), shspec(), trans(), pso_psfile(),lpso_psfile();
extern int ps1_psfile(), lps1_psfile(), ps2_psfile(), lps2_psfile();
extern int ps_psfile(), lps_psfile(), escale(), clear_all(), runfile();
extern int readfile(), readapnd(), readspec(), readcol(), readrow();
extern int disp_d(), disp(), disp_o(), set_dsr(), logdis(), lindis();
extern int overlay(), peakmar(), set_knr();
extern int set_mark(), get_fln(), set_id(), zero_data(), data_set();
extern int sel_id(), hisout(), chpath(), prpath();
extern int set_flhd(), set_hist(), cpcal(), set_aps(), w_win(), r_win();
extern int udselect(), dratio2(),dratio4(),subran(), readvars();
extern int genful(), sumlines(), multiply(), smooth();
extern int dosystem(), scale(), wspe(), wasci();
extern int psfile(int), search(),checkname();

void          sa_init(), graphinit(), icon_init();
extern  void  fplot(), bgdplt(), output(), gfit(), hplot();
extern  void  displa();
extern  void  setmark(int nchan,int i,int mod,float en);
extern  void  ylimits(int beg,int end);
extern  float yscale(float max,float min,float hght);
extern  void  clear_screen(), getime();
extern  void  calcoeff(), calslope(),legendre(double),legendre_deriv(double);
extern  void  corslope();

struct command cmd[] =
{
    {  "ds"       , disp       }, /* display one or several spectra */
    {  "do"       , disp_o     }, /* overlay spectrum */
    {  "r"        , readfile   }, /* read histgram file */
    {  "ra"       , readapnd   }, /* read histgram file and append */
    {  "ls"       , shspec     }, /* list all spectra in memory */
    {  "w"        , wrtf0      }, /* write histogram file out */
    {  "sel"      , sel_id     }, /* copy selected spectra to work space */
    {  "rs"       , readspec   }, /* read a spectrum from open file */
    {  "rc"       , readcol    }, /* read column(s) of matrix file */
    {  "rr"       , readrow    }, /* read row(s) of matrix file */
    {  "wf"       , wrtf2      }, /* write histogram file (float) */
    {  "clear"    , clear_all  }, /* clear all data and his-info */
    {  "cal"      , calibrate  }, /* calibrate spectra */
    {  "cpcal"    , cpcal      }, /* copy calibration to other spectrum */
    {  "mcal"     , mcal       }, /* set calibration for multiple spectra */
    {  "rcal"     , rcal       }, /* read calibration info from file */
    {  "wcal"     , wcal       }, /* write calibration info to file */
    {  "cd"       , chpath     }, /* change path of input data files */
    {  "d"        , disp_d     }, /* display slected data region */
    {  "dsr"      , set_dsr    }, /* set display range (from to) */
    {  "ov"       , overlay    }, /* overlay one or more spectra */
    {  "file"     , get_fln    }, /* open data file for reading */
    {  "open"     , get_fln    }, /* open data file for reading */
    {  "m"        , set_mark   }, /* type channel or energy to be marked */
    {  "mm"       , mv_marks   }, /* move marks */
    {  "p"        , peak       }, /* select peak for integration */
    {  "c"        , peak       }, /* find peak center */
    {  "i"        , integrate  }, /* sum displayed peak area */
    {  "s"        , integrate  }, /* sum displayed peak area */
    {  "f"        , fit        }, /* make gauss fit of selected peak */
    {  "add"      , add        }, /* add spectra */
    {  "+"        , add        }, /* add spectra */
    {  "addc"     , forge      }, /* add constant to spectra */
    {  "sub"      , sub        }, /* subtract spectra */
    {  "-"        , sub        }, /* subtract spectra */
    {  "sran"     , subran     }, /* subract random spectrum and remember */
    {  "mulf"     , forge      }, /* multiply spctrum with a factor */
    {  "div"      , divide     }, /* divide spectra */
    {  "mul"      , multiply   }, /* multiply spectra */
    {  "tr"       , trans      }, /* transfer spectra in memory */
    {  "ud"       , udselect   }, /* select for up/down analysis */
    {  "ps"       , ps_psfile  }, /* make postscript file sagraph.ps */
    {  "lps"      , lps_psfile }, /* send postsript file to default printer */
    {  "pso"      , pso_psfile }, /* overlay spectra in postscript file */
    {  "lpso"     , lpso_psfile}, /* send to lp */
    {  "ps1"      , ps1_psfile }, /* make multiple spctra in 1 column */
    {  "lps1"     , lps1_psfile}, /* send to lp */
    {  "ps2"      , ps2_psfile }, /* make multiple spctra in 2 columns */
    {  "lps2"     , lps2_psfile}, /* send to lp */
    {  "log"      , logdis     }, /* switch to log display (- decades) */
    {  "lin"      , lindis     }, /* switch to lin display */
    {  "pwd"      , prpath     }, /* show current data file path */
    {  "pd"       , pridata    }, /* print channel content to sa.dat */
    {  "pdg"      , pridata    }, /* print channel content to sa.dat */
    {  "pm"       , peakmar    }, /* copy peak marks over from other peak */
    {  "nspec"    , set_flhd   }, /* set in out flhd number of spectra */
    {  "nhist"    , set_hist   }, /* set or adjust number of his in mem */
    {  "knr"      , set_knr    }, /* set or adjust begin of free mem */
    {  "id"       , chid       }, /* set his id for current display */
    {  "esc"      , escale     }, /* toggle energy scale channel display */
    {  "reset"    , reset      }, /* reset marker (all) */
    {  "setid"    , set_id     }, /* toggle display selection index/specid */
    {  "rm"       , rmarks     }, /* read mks file (stored marker) */
    {  "wm"       , wmarks     }, /* write mks file (store marker) */
    {  "rmarks"   , rmarks     }, /* read mks file (stored marker) */
    {  "wmarks"   , wmarks     }, /* write mks file (store marker) */
    {  "ww"       , w_win      }, /* write win file (selected regions) */
    {  "rw"       , r_win      }, /* read win file (checking regions) */
    {  "rv"       , readvars   }, /* read variable */
    {  "gmf"      , set_gmf    }, /* set/get gain match factors */
    {  "gm"       , g_match    }, /* gainmatch spectrum */
    {  "pp"       , gmfile     }, /* gain match file for clover detectors */
    {  "aps"      , set_aps    }, /* set automatic peak search parameters */
    {  "smooth"   , smooth     }, /* smooth spectrum */
    {  "run"      , runfile    }, /* run script file (command file) */
    {  "sqrt"     , forge      }, /* make square root spectrum */
    {  "scale"    , scale      }, /* scale spectra (regions) */
    {  "cut"      , forge      }, /* cut negative counts */
    {  "rat"      , dratio4    }, /* make double ratio for up/down anal. */
    {  "rat2"     , dratio2    }, /* make double ratio 2 det. */
    {  "anis"     , anis       }, /* make anisotropy ratios. */
    {  "ful"      , genful     }, /* generate flat-upper-limit background */
    {  "sum"      , sumlines   }, /* sum peaks between marker pairs */
    {  "shift"    , shift      }, /* shift spectrum */
    {  "rebin"    , rebin      }, /* re bin spectrum eg 2048 -> 1024*/
    {  "system"   , dosystem   }, /* execute system call */
    {  "exit"     , exitus     }, /* exit program, keep log files */
    {  "quit"     , quit_proc  }, /* exit program unlink log files */
    {  "set"      , data_set   }, /* set data in channel */
    {  "wspe"     , wspe       }, /* write his file in radware mode */
    {  "wasci"    , wasci      }, /* write his file in ascii mode */
    {  "zero"     , zero_data  }  /* zero histogram data */
};
int command_num = sizeof(cmd) / sizeof(struct command);

					/* input line parser */
int    ntrs;				/* number of entries */
char   cmd_line[256];			/* typed command line */
char  *wptr[16];			/* word pointer */
char   strPS[128];		        /* text string for psfile */
char   filNam[80];		        /* generic output file name */
char   fname[128];       		/* data file name */
char   ovstr[32];			/* identification for overlay sp. */
char   cdirec[64];		        /* data file pathname - current dir */
char   direc[64];		        /* data file pathname */




int main(argc,argv)
int argc;
char *argv[];
{
    int i;
    int c;
    extern int optind, opterr;
    extern char *optarg;
    XtAppContext  app;
    Widget        main_window;


    while((c = getopt(argc, argv, "sv")) != EOF) {
      switch(c)
        {
          case 's':
            shmversion = 1;
            break;
	  case 'v':
	    printf("%s\n",sccsid);
	    exit(0);
	    break;
          default:
            break;
        }
    }
    XtSetLanguageProc (NULL,NULL,NULL);

    MrmInitialize();		/* Initialize Mrm before initializing */
				/* the X Toolkit. */

    toplevel = XtVaAppInitialize (&app,"Xsa",NULL,0,
				  &argc,argv,
				  NULL,NULL);

    if( MrmOpenHierarchyPerDisplay (XtDisplay(toplevel),
				    db_filename_num, /* Number of files. */
				    db_filename_vec, /* Array of file names.  */
				    NULL,            /* Default OS extenstion. */
				    &s_MrmHierarchy) /* Pointer to returned MrmID */
	!= MrmSUCCESS)
      s_error("can't open hierarchy -- set UIDPATH environment");
  
    /* Register the items Mrm needs to bind */
    MrmRegisterNames(reglist, reglist_num);

    /* Initialize widget array */
    for (i = 0; i < kmax; i++)
        wdg_arr[i] = NULL;

    /* Go get the main part of the application. */
    if( MrmFetchWidget(s_MrmHierarchy, "S_MAIN_WINDOW", toplevel,
		       &main_window, &dummy_class) != MrmSUCCESS)
      s_error("can't fetch main window");

    /* Manage the main part and realize everything.  The interface comes up
     * on the display now. */
    /* set window mangager hints */
    
    XtManageChild(main_window);
    XtRealizeWidget(toplevel);

    icon_init();		/* register icon with WM */
    graphinit();		/* initialize drawing area */
    sa_init();			/* initialize various variables in sa */

    if(argc > optind) {		/* with given filename dispay spectra */
      strcpy(fname,argv[optind]);
      if (open(fname,0) < 0) {
	perror(" ");
      }
      else
	showfile();
    }
    else {
      if (MrmFetchWidget(s_MrmHierarchy, "com_box",
			 toplevel, &wdg_arr[k_com_box],
			 &dummy_class) != MrmSUCCESS) {
	s_error("can't fetch command widget");
      }
      XtManageChild(wdg_arr[k_com_box]);
    }
    XtAppMainLoop(app);

}

void icon_init()
{
				/* icon is in bitmap file saicon.xb */
    XWMHints  *wm_hints;

    icon = XCreateBitmapFromData ( XtDisplay(toplevel),
				  XtWindow(toplevel),
				  (char *) saicon_bits,
				  saicon_width, saicon_height);
    wm_hints = XAllocWMHints();
    wm_hints->flags = InputHint | IconPixmapHint;
    wm_hints->input = True;
    wm_hints->icon_pixmap = icon;
    XSetWMHints(XtDisplay(toplevel),XtWindow(toplevel),wm_hints);
    XFree (wm_hints);
}
    /* Prepare for graphics - get necessary information*/

void graphinit()
{
    register int i;
    Dimension wi, he;

    dpy = XtDisplay(wdg_arr[k_graph_win]);
    gwin = XtWindow(wdg_arr[k_graph_win]);
    /*
    tracker = XtWindow(wdg_arr[k_mess]);
    */
    screen = DefaultScreen (dpy);
    font1 = XLoadQueryFont(dpy,
		"-*-helvetica-medium-r-normal-*-18-*-*-*-*-*-iso8859-1");
    if( font1 == (XFontStruct *)NULL ) {
	printf("Error could not load font 18\n");
	if((font1 = XLoadQueryFont(dpy,"fixed")) == (XFontStruct *)NULL ) {
	    printf("Error: could not load any font\n");
	    exit(1);
	}
    }
    font2 = XLoadQueryFont(dpy,
	       "-*-helvetica-medium-r-normal-*-12-*-*-*-*-*-iso8859-1");
    if( font2 == (XFontStruct *)NULL ) {
	printf("Error could not load font 12\n");
	if((font2 = XLoadQueryFont(dpy,"fixed")) == (XFontStruct *)NULL ) {
	    printf("Error: could not load any font\n");
	    exit(1);
	}
    }

    depth = XDefaultDepth (dpy,screen);

    if(depth > 1) {		/* assume color */
	cmap = XDefaultColormap (dpy,screen);
	XAllocNamedColor(dpy,cmap,"red",&exact,&red);
	XAllocNamedColor(dpy,cmap,"blue",&exact,&blue);
	XAllocNamedColor(dpy,cmap,"yellow",&exact,&yellow);
	XAllocNamedColor(dpy,cmap,"cyan",&exact,&cyan);
	XAllocNamedColor(dpy,cmap,"lightgrey",&exact,&lightgrey);
	XAllocNamedColor(dpy,cmap,"black",&exact,&black);
	XAllocNamedColor(dpy,cmap,"green",&exact,&green);
	XAllocNamedColor(dpy,cmap,"brown",&exact,&brown);
	XAllocNamedColor(dpy,cmap,"white",&exact,&white);
	XAllocNamedColor(dpy,cmap,"SkyBlue",&exact,&SkyBlue);
	XAllocNamedColor(dpy,cmap,"aquamarine1",&exact,&aquamarine1);

	strcpy(color_vec[0].name,"cyan"); /* common background */
	color_vec[0].pixel = cyan.pixel;
	strcpy(color_vec[1].name,"blue");
	color_vec[1].pixel = blue.pixel;
	strcpy(color_vec[2].name,"green");
	color_vec[2].pixel = green.pixel;
	strcpy(color_vec[3].name,"brown");
	color_vec[3].pixel = brown.pixel;
	strcpy(color_vec[4].name,"red");
	color_vec[4].pixel = red.pixel;
	strcpy(color_vec[5].name,"yellow");
	color_vec[5].pixel = yellow.pixel;
	strcpy(color_vec[6].name,"lightgrey");
	color_vec[6].pixel = lightgrey.pixel;
	strcpy(color_vec[7].name,"black");
	color_vec[7].pixel = black.pixel;
	strcpy(color_vec[8].name,"white");
	color_vec[8].pixel = white.pixel;
    }
    else {
      printf("Black and White\n");
	color_vec[0].pixel = WhitePixel(dpy,screen);
	for(i=1;i<MAXCOL;i++)
	  color_vec[i].pixel = BlackPixel(dpy,screen);
    }

    ac = 0;				/* get geometry */
    XtSetArg(al[ac],XmNwidth, &wi); ac++;
    XtSetArg(al[ac],XmNheight, &he); ac++;
    XtGetValues (wdg_arr[k_graph_win], al, ac);
    width = wi;
    height = he;
    xzero = leftmar;
    xmax = width -rightmar;
    ytop = topmar;
    ybot = height - botmar;
    xlen = xmax - xzero;
    ylen = ybot - ytop;
    xgcv.line_width = 1;
    xgcv.foreground = color_vec[0].pixel;

    pixmap = XCreatePixmap(dpy,gwin,width,height,depth);
    gc = XCreateGC(dpy, pixmap, GCForeground | GCBackground |
		   GCLineWidth,&xgcv);
				/* set background of pixmap */
    XFillRectangle (dpy,pixmap,gc,0,0,width,height);

    cursor = XCreateFontCursor(dpy,34);
    XDefineCursor(dpy,gwin,cursor);

    XSelectInput (dpy,gwin,StructureNotifyMask |
		 ExposureMask | KeyPressMask | ButtonPressMask |
		 ButtonMotionMask | ButtonReleaseMask |
		 EnterWindowMask | LeaveWindowMask);

    initdone = 1;
}

void sa_init()	       /* Initialization of various things */
{
  register int i,k;
    int id[2];
    char *cwd, *getcwd();
    char confname[32];
    char syscallstr[64];
    time_t t;
    FILE *fp;

    anm[0] = Anm[0][0];
    anm[1] = Anm[0][1];
    t = time(0);
    strftime(sadatfile, sizeof(sadatfile),"sa%m%d%y_%H%M.dat",localtime(&t));
    sprintf(syscallstr,"date >> %s",sadatfile);
    if(system (syscallstr) != 0) 
	puts("\n No output saved *** cannot create sa.dat");

    strcpy(psname,"preamble.ps");
    if((fp=fopen(psname,"r")) == NULL) {
      if((pspath = getenv("XSAPS")) == NULL) {
        puts("Set XSAPS environment for ps to work");
	putchar(07);
	fflush(stdout);
      }
      else {
        strcpy(psname,pspath);
        strcat(psname,"/preamble.ps");
      }
    }      
		                /* initialize markers */
    for (i = 0; i < NMARKS; i++) {
	marks[i]  = -1;
	for (k = 0; k < 11; k++)
	    bktg[i][k] = -1;
    }
    mrks = 0;
    pmarks = marks;		/* initialize pointer */
    peakmode = 0;		/* initialize flags */
    picture = 0;
    mv_flag = 0;
    ov_flag = 0;
    for(i=0;i<16;i++)		/* set token pointers to NULL*/
	wptr[i] = NULL;
    wptr[1] = cmd_line;		/* for showfile to work */
    direc[0] = '\0';
    if ((cwd = getcwd((char *)NULL, 64)) == NULL) {
      perror("pwd");
    }
    strcpy(cdirec,cwd);

    if (shmversion) {
                                /* online stuff starts here */
                                /* initialize shared memory */
      if((pspath = getenv("HOME")) != NULL) {
        strcpy(confname,pspath);
        strcat(confname,"/SHM.conf");
      }
      else
	strcpy(confname,"SHM.conf");

      if((fp = fopen(confname, "r")) == NULL) {
        printf("\n <==  NO SHARED MEMORY CONFIGURATION FILE FOUND  ==> \n");
        exit(0);
      }
      fscanf(fp,"%*s %*s %*s %*s %*s %d %d", &id[0],&id[1]);
      fclose(fp);

      if(id[0] >= 0) {		/* can be zero in LINUX */
	if((spc[0] = (struct hinfo *) shmat(id[0], 0, shmflg)) ==
         (struct hinfo *) -1) {
        perror("attach hinfo segment");
        exit(1);
	}
	spc[0]->sl = 1.0;
	spc[0]->zr = 0.0;
	for(i=1;i <= NSPEC;i++) {	/* initialize pointer array */
	  spc[i] = spc[i-1] + 1;
	  if(spc[i]->sl < 0.0001) {
	    spc[i]->sl = 1.0;
	    spc[i]->zr = 0.0;
	  }
	}
      }
      if(id[1] >= 0) {
	if((hdat = (float *) shmat(id[1], 0, shmflg)) == (float *)-1) {
	  perror ("attach data segment");
	  exit(1);
	}
	knr =   (int)hdat[0];
	nhist = (int)hdat[1];
	hdat[0] = 0.0;
	hdat[1] = 0.0;
      }
    }      /* end of online stuff */
           /* normal version */
    else {
      if((hdat = (float *)calloc((MKNR+WRK)*1024,sizeof(float))) == NULL) {
        perror("Data space allocation failed");
        exit(1);
      }
      if((spc[0] = (struct hinfo *) calloc(NSPEC+1,sizeof(struct hinfo))) == 
         NULL) {
        perror("Spectra info not allocated");
        exit(1);
      };

      for(i=1;i <= NSPEC;i++)	/* initialize pointer array */
        spc[i] = spc[i-1] + 1;
      for (i=0;i <= NSPEC;i++) { /* initialize calibration default */
	spc[i]->sl = 1.0;
	spc[i]->zr = 0.0;
      }
    }

    pdat = hdat;
    hptr = spc[0];		/* set up work space */
    strcpy(hptr->title, "work");
    hptr->offset = MKNR*1024;	/* start at end of data area */
    hptr->xma = WRKSIZ;
    hptr->id = 0;
    printf("\n\n ******* %s  output window ***********\n",sccsid);
}

void showfile()
{
  extern int readin(int flag);

  ntrs = 0;
  readin('r');
  hptr = spc[idx];
  fchan = 0;
  len = hptr->xma;
  pdat = hdat + hptr->offset;
  displa();
}

void s_error(char *problem_string)
{
    printf("%s\n", problem_string);
    if (shmversion) {
      hdat[0] = knr;
      hdat[1] = nhist;
    }
    exit(0);
}

static int quit_proc( w, tag, reason)
     Widget w;
     int    *tag;
     unsigned long *reason;
{
    int out = *tag;

    if (shmversion) {
      hdat[0] = knr;
      hdat[1] = nhist;
    }
    if (out == k_exit )
      exit(0);
    printf("Quitting\n");

    unlink(sadatfile);
    unlink("results.dat");
    exit(0);
    return(0);
}

int exitus()			/* can`t use exit(0) in cmd list */
{
    exit(0);
    return(0);
}

static void set_something(w, resource, value)
    Widget w;
    char *resource;
    Widget value;
{
    XtSetArg(al[0], resource, value);
    XtSetValues(w, al, 1);
}

static void show_hide_proc(w, tag, reason)
    Widget w;
    int *tag;
    unsigned long *reason;
{
    int widget_num = *tag;

    if (XtIsManaged(wdg_arr[widget_num]))
        XtUnmanageChild(wdg_arr[widget_num]);
    else
        XtManageChild(wdg_arr[widget_num]);
}

static void create_proc(w, tag, reason)	/* register widget
					   for later reference */
    Widget w;
    int *tag;
    unsigned long *reason;
{
    int widget_num = *tag;
    /*
    printf(" created %d\n",widget_num);
    */
    wdg_arr[widget_num] = w;
}

static void pull_proc(w, tag, reason)
    Widget w;
    int *tag;
    unsigned long *reason;
{
    int widget_num = *tag;

    if(widget_num == k_file_pdme) {
        if (wdg_arr[k_file_menu] == NULL) {
	    if (MrmFetchWidget(s_MrmHierarchy, "file_menu",
			       wdg_arr[k_menu_bar], &wdg_arr[k_file_menu],
			       &dummy_class) != MrmSUCCESS)
	      s_error("can't fetch file pulldown menu widget");
	    set_something(wdg_arr[k_file_pdme], XmNsubMenuId,
			  wdg_arr[k_file_menu]);
	}
    }
    if(widget_num == k_opt_pdme) {
        if (wdg_arr[k_opt_menu] == NULL) {
	    if (MrmFetchWidget(s_MrmHierarchy, "options_menu",
			       wdg_arr[k_menu_bar], &wdg_arr[k_opt_menu],
			       &dummy_class) != MrmSUCCESS)
	        s_error("can't fetch options pulldown menu widget");
	    set_something(wdg_arr[k_opt_pdme], XmNsubMenuId,
			  wdg_arr[k_opt_menu]);
	}
	if(wdg_arr[k_esc] != NULL)
	  XmToggleButtonSetState(wdg_arr[k_esc],hptr->cflag,False);
    }
}

static void
activate_proc (widget, tag, call_data)
    Widget       widget;
    int         *tag;
    XtPointer    call_data;
{
    int status;
    int i;
    char chstr[64];
    char *param;
    XmString list[NSPEC];
    int widget_num = *tag;	/* Convert tag to widget number. */
    XmToggleButtonCallbackStruct *state;

    switch (widget_num) {
      case k_com_box:
	if (wdg_arr[k_com_box] == NULL) {
	    status = MrmFetchWidget(s_MrmHierarchy, "com_box",
				    toplevel, &wdg_arr[k_com_box],
				    &dummy_class);
	    if (status != MrmSUCCESS) {
		s_error("can't fetch command widget");
		return;
	    }
	}
	XtManageChild(wdg_arr[k_com_box]);
	break;
      case k_cancel:
	XtUnmanageChild(wdg_arr[k_com_box]);
	break;

      case k_open:
	if (wdg_arr[k_open] == NULL) {
	    status = MrmFetchWidget(s_MrmHierarchy, "fsb",
				    toplevel, &wdg_arr[k_open],
				    &dummy_class);
	    if (status != MrmSUCCESS) {
		s_error("can't fetch file selection widget");
		return;
	    }
	}
	XtManageChild(wdg_arr[k_open]);
	break;

      case k_write:
	if (wdg_arr[k_write] == NULL) {
	    status = MrmFetchWidget(s_MrmHierarchy, "write_file",
				    toplevel, &wdg_arr[k_write],
				    &dummy_class);
	    if (status != MrmSUCCESS) {
		s_error("can't fetch command widget");
		return;
	    }
	}
	XtManageChild(wdg_arr[k_write]);
	break;

      case k_int:
	state = (XmToggleButtonCallbackStruct *) call_data;
	if(state->set)
	  wmode = 0;
	else
	  wmode = 2;
	break;

      case k_float:		/* kind of redundant */
	state = (XmToggleButtonCallbackStruct *) call_data;
	if(state->set)
	  wmode = 2;
	else
	  wmode = 0;
	break;

      case k_print:
	if(!picture)
	  break;
	ntrs = 0;
	psfile(1);
	system("lpr sagraph.ps");
	unlink("sagraph.ps");
	break;

      case k_anis_box:
	if (wdg_arr[k_anis_box] == NULL) {
	    status = MrmFetchWidget(s_MrmHierarchy, "anisotropy",
				    toplevel, &wdg_arr[k_anis_box],
				    &dummy_class);
	    if (status != MrmSUCCESS) {
		s_error("can't fetch anisotropy widget");
		return;
	    }
	}
	sprintf(chstr,"%7.4f(%.4f)",Anm[ispin][0] ,dAnm[ispin][0]);
	XmTextFieldSetString(wdg_arr[k_coef1],chstr);
	sprintf(chstr,"%7.4f(%.4f)",Anm[ispin][1],dAnm[ispin][1]);
	XmTextFieldSetString(wdg_arr[k_coef2],chstr);
	sprintf(chstr,"%7.3f",thet1);
	XmTextFieldSetString(wdg_arr[k_theta1],chstr);
	sprintf(chstr,"%7.3f",thet2);
	XmTextFieldSetString(wdg_arr[k_theta2],chstr);
	sprintf(chstr,"%7.3f",thet3);
	XmTextFieldSetString(wdg_arr[k_theta3],chstr);
	sprintf(chstr,"%8.5f(%.5f)",eta,deta);
	XmTextFieldSetString(wdg_arr[k_eta],chstr);
	sprintf(chstr,"%7.3f(%.3f)",slope, dslope);
	XmTextFieldSetString(wdg_arr[k_slope],chstr);

        if (!XtIsManaged(wdg_arr[k_anis_box]))
          XtManageChild(wdg_arr[k_anis_box]);

	break;

      case k_cancel_anis:
	if (XtIsManaged(wdg_arr[k_anis_box]))
	  XtUnmanageChild(wdg_arr[k_anis_box]);
	break;
	
      case k_read_anis:
	if (XtIsManaged(wdg_arr[k_anis_box]))
	  XtUnmanageChild(wdg_arr[k_anis_box]);

      case k_apply_anis:

	param = XmTextFieldGetString(wdg_arr[k_coef1]);
	sscanf(param,"%lf(%lf)",&Anm[ispin][0], &dAnm[ispin][0]);
	XtFree(param);
	param = XmTextFieldGetString(wdg_arr[k_coef2]);
	sscanf(param,"%lf(%lf)",&Anm[ispin][1], &dAnm[ispin][1]);
	XtFree(param);
	param = XmTextFieldGetString(wdg_arr[k_theta1]);
	thet1 = atof(param);
	XtFree(param);
	param = XmTextFieldGetString(wdg_arr[k_theta2]);
	thet2 = atof(param);
	XtFree(param);
	param = XmTextFieldGetString(wdg_arr[k_theta3]);
	thet3 = atof(param);
	XtFree(param);
	param = XmTextFieldGetString(wdg_arr[k_eta]);
	sscanf(param,"%lf(%lf)",&eta, &deta);
	XtFree(param);
	legendre(thet3);
	legendre_deriv(thet3);
	if(!fixedslope)
	  corslope();
	sprintf(chstr,"%7.3f(%.3f)",slope, dslope);
	XmTextFieldSetString(wdg_arr[k_slope],chstr);
	break;

      case k_b1:
	ispin = 0;
	sprintf(chstr,"%7.4f",Anm[ispin][0]);
	XmTextFieldSetString(wdg_arr[k_coef1],chstr);
	sprintf(chstr,"%7.4f",Anm[ispin][1]);
	XmTextFieldSetString(wdg_arr[k_coef2],chstr);
	break;

      case k_b2:
	ispin = 1;
	sprintf(chstr,"%7.4f",Anm[ispin][0]);
	XmTextFieldSetString(wdg_arr[k_coef1],chstr);
	sprintf(chstr,"%7.4f",Anm[ispin][1]);
	XmTextFieldSetString(wdg_arr[k_coef2],chstr);
	break;

      case k_b3:
	ispin = 2;
	sprintf(chstr,"%7.4f",Anm[ispin][0]);
	XmTextFieldSetString(wdg_arr[k_coef1],chstr);
	sprintf(chstr,"%7.4f",Anm[ispin][1]);
	XmTextFieldSetString(wdg_arr[k_coef2],chstr);
	break;

      case k_eta:
	param = XmTextFieldGetString(wdg_arr[k_eta]);
	sscanf(param,"%lf(%lf)",&eta, &deta);
	XtFree(param);
	calcoeff();
	calslope();
	sprintf(chstr,"%7.3f(%.3f)",slope, dslope);
	XmTextFieldSetString(wdg_arr[k_slope],chstr);
        break;

      case k_slope:
	param = XmTextFieldGetString(wdg_arr[k_slope]);
	sscanf(param,"%lf(%lf)",&slope, &dslope);
	if(slope == 0.0)
	  fixedslope = FALSE;
	break;

      case k_theta3:
	param = XmTextFieldGetString(wdg_arr[k_theta3]);
	thet3 = atof(param);
	XtFree(param);
	legendre(thet3);
	legendre_deriv(thet3);
	if(!fixedslope)
	  calslope();
	sprintf(chstr,"%7.3f(%.3f)",slope, dslope);
	XmTextFieldSetString(wdg_arr[k_slope],chstr);
	break;

      case k_rand:
	state = (XmToggleButtonCallbackStruct *) call_data;
	randoms = state->set;
	break;

      case k_esc:
	state = (XmToggleButtonCallbackStruct *) call_data;
	hptr->cflag = state->set;
	displa();
	break;

      case k_setid:
	state = (XmToggleButtonCallbackStruct *) call_data;
	noid = state->set;
	break;

      case k_bmode:
	state = (XmToggleButtonCallbackStruct *) call_data;
	bmode = state->set;
	break;

      case k_list_cancel:
        if (XtIsManaged(wdg_arr[k_list_box]))
          XtUnmanageChild(wdg_arr[k_list_box]);
        break;

      case k_list_box:
        if(wdg_arr[k_list_box] == NULL) {
            if (MrmFetchWidget (s_MrmHierarchy,"listing",
                                toplevel, &wdg_arr[k_list_box],
                                &dummy_class) != MrmSUCCESS)
              s_error("can't fetch list box widget");

        }
	for (i=1;i<nhist;i++){
	  sprintf(chstr,"%3d %-4d  %-12s %4dk",
		  i,spc[i]->id,spc[i]->title,spc[i]->offset/1024);
	  list[i-1] = XmStringCreateLocalized(chstr);
        }
	i--;
        XtVaSetValues (wdg_arr[k_listing],XmNitems, list,
		       XmNitemCount, i,
                       XmNmustMatch, True,
                       NULL);
        while (--i >= 0)
          XmStringFree(list[i]);

        if (!XtIsManaged(wdg_arr[k_list_box]))
          XtManageChild(wdg_arr[k_list_box]);
        break;

    case k_exit:
      XtUnmanageChild(help_widget);
      break;

      default:
	break;
    }
}

static void
resize_proc (widget,tag, callback_data)
     Widget     widget;
     int *      tag;
     XmDrawingAreaCallbackStruct  *callback_data;
{
    int widget_num = *tag;	/* Convert tag to widget number. */
    Dimension wi, he;

    if(widget_num  != k_graph_win)
      return;
				/* get Geometry */
    ac = 0;
    XtSetArg(al[ac],XmNwidth, &wi); ac++;
    XtSetArg(al[ac],XmNheight, &he); ac++;
    XtGetValues (wdg_arr[k_graph_win], al, ac);
    width = wi;
    height = he;
    xmax = width -rightmar;
    ybot = height - botmar;
    xlen = xmax - xzero;
    ylen = ybot - ytop;
				/* can run only after initialisation */
    if(initdone){
	XFreePixmap (dpy,pixmap);
	pixmap = XCreatePixmap (dpy,gwin,width,height,depth);
	XSetForeground (dpy,gc,color_vec[0].pixel);
	XFillRectangle(dpy,gwin,gc,0,0,width,height);
	if(picture)
	  displa();
    }
    return;
}
     
static void
help_proc(widget, tag, reason)
    Widget                  widget;
    int *                   tag;
    XmAnyCallbackStruct *  reason;
{
    int                 status;
    int widget_num = *tag;	/* Convert tag to widget number. */

    struct utsname osname;	/* get operating system */
    /*
    * If this is the first time that the user has selected help, then
    * fetch the help widget from the widget hierarchy.
    */
    uname(&osname);

    switch (widget_num) {
      
      case k_help3:
	if(!strcmp(osname.sysname,"Darwin"))   /* OSX */
	  system("open -a Safari http://www.physics.rutgers.edu/~kum/sa.html");
	else if (!strcmp(osname.sysname,"Linux"))
	  system("htmlview http://www.physics.rutgers.edu/~kum/sa.html&");
	break;

      case k_help1:
	if (help_widget == NULL) {
	  status = MrmFetchWidget(s_MrmHierarchy, "help_widget",
				  toplevel, &help_widget, &dummy_class);
	  if (status != MrmSUCCESS) {
	    s_error("can't fetch help widget");
	    return;
	  }
	}
	XtManageChild(help_widget);
	break;

      case k_help2:
	if(!strcmp(osname.sysname,"Darwin"))   /* OSX */
	  system("open -a Safari http://www.physics.rutgers.edu/~kum/sa.html#A1.1.28");
	else if (!strcmp(osname.sysname,"Linux"))
	  system("htmlview http://www.physics.rutgers.edu/~kum/sa.html#A1.1.28&");
	break;

      default:
	break;
    }
}

static void dispatch(w,tag,callback_data)
     Widget   w;
     char    *tag;
     XmCommandCallbackStruct  *callback_data;
{
    register int i;
    char errstr[16];
    XmString text;
    char *cmdp;

    XmStringGetLtoR(callback_data->value,XmSTRING_DEFAULT_CHARSET,&cmdp);
    /*
    cmdp = XmStringUnparse(callback_data->value,NULL,0,XmCHARSET_TEXT,
			   NULL,0,0);
    */
    if(strncmp(cmdp,"mk",2)== 0) {    /* prepend "run" as convenience */
      strcpy(cmd_line,"run ");
      strcat(cmd_line, cmdp);
    }
    else
      strcpy(cmd_line, cmdp);
 
    XtFree(cmdp);

    for(i=0;i<16;i++)		/* reset token pointers to NULL*/
	wptr[i] = NULL;

    ntrs = inputparser(cmd_line);	/* parses input line */

    for(i=0;i < command_num;i++)
      if (strcmp(wptr[0],cmd[i].name) == 0) {
	  if ( (*cmd[i].function) () > 0 ) {
	      sprintf(errstr,"%s failed",wptr[0]);
	      text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	      XmCommandError(wdg_arr[k_com_win],text);
	      XmStringFree(text);
	      return;
	  }
	  break;
      }
    if (i == command_num) {
	text =  XmStringCreate("unknown cmd",XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
    }
}

int inputparser(char *line)
{
    register int i;
    int c, cnt;
    char *chptr;

    cnt = 0;
    i = 0;
    strPS[0] ='\0';		        /* comment string in quotes */
    filNam[0] = '\0';		        /* redirected output file name  */

    while(line[i++] == ' ');		/* skip leading blanks */
    wptr[0] = &line[i-1];		/* set code word */
    while((c = line[i++])) {
      if(c == ' ') {			/* separate on 'blanks' */
	line[i-1] = '\0';
	wptr[++cnt] = &line[i];
      }
      if(*wptr[cnt] == ' ') cnt--;	/* skip multiple blanks */
      if(c == '"') {			/* extract quote text */
	chptr = strPS;
	while(line[i++] != '"')
	  *chptr++ = line[i-1];
	*chptr = '\0';
	cnt--;
      }
      if(c == '>') {		       /* signal next is filename */
	chptr = filNam;
       	while(line[i] == ' ')          /* skip next blanks */
	  i++;
	while(line[i] && line[i] != ' ') {
	  *chptr++ = line[i];
	  i++;
	}
	*chptr = '\0';
	cnt--;
      }
    }
    if(!*wptr[cnt]) cnt--;		/* cut trailing blank */
    return(cnt);
}

static void expose_proc(w,tag,callback_data)
     Widget w;
     int *tag;
     XmDrawingAreaCallbackStruct  *callback_data; /* 3-354 */
{
    int widget_num = *tag;	/* Convert tag to widget number. */
    if(widget_num  != k_graph_win)
      return;

    XCopyArea (dpy,pixmap,gwin,gc,
	       callback_data->event->xexpose.x,
	       callback_data->event->xexpose.y,
	       callback_data->event->xexpose.width,
	       callback_data->event->xexpose.height,
	       callback_data->event->xexpose.x,
	       callback_data->event->xexpose.y);
}

static void
inout_proc (widget,tag, callback_data)
     Widget     widget;
     int *      tag;
     XmDrawingAreaCallbackStruct  *callback_data;
{
    register int i,j;
    int nch;			/* number of char */
    int ichar;			/* character pressed */
    char text[10];
    char xstr[80];
    KeySym key;

    static int expand = 0;
    static int lchan;		/* temporary channel number */
    static int dir = 0;		/* scrolling direction */
    static int qflag = 0;	/* q for fill channels */
    static int sflag = 0;	/* peakmode without peak select */
    static int bflag = 0;	/* bkgd */
    static int mm = 0;		/* mark to be "dragged" */

    int wrap;
    int diff, nl, nf;
    int min, start, stop;
    float center;
    float temp;
    float dat, tsum;
    FILE   *op, *fopen ();	 /* sa.dat */

    extern float cent(int n), calbrt(double cem);
    extern int checkmarks();
    extern void docal();

    int widget_num = *tag;	/* Convert tag to widget number. */
    if(widget_num  != k_graph_win)
      return;
    if(!picture)
      return;

    switch (callback_data->event->type) {

      case  KeyPress:

	nch = XLookupString ((XKeyEvent *)callback_data->event,
			     text,10,&key, 0);
	text[nch] = '\0';
	ichar = text[0];
	xcoor = callback_data->event->xkey.x;
	ycoor = callback_data->event->xkey.y; 
	nchan = fchan + (xcoor - xzero)/xinc;
	break;

      case ButtonPress:

	xcoor = callback_data->event->xbutton.x;
	ycoor = callback_data->event->xbutton.y; 

	nchan = fchan + (xcoor - xzero)/xinc;
	if (nchan < fchan)		/* stay in histogram range */
	  nchan = fchan;
	else if (nchan > fchan + len)
	  nchan = fchan + len;

	if(callback_data->event->xbutton.button == 1) {

	  sprintf(xstr,"X: %4d Y: %8.1f",nchan, pdat[nchan]);
	  XmTextSetString(wdg_arr[k_mess],xstr);

	  if(mv_flag) {	/* move marks */
	    diff = nchan - pmarks[numbs[0]];
	    for(i=0;i<ntrs;i++) {
	      j = numbs[i];
	      if(pmarks[j] != -1) {
		setmark (pmarks[j], j, E, 0.0);
		pmarks[j] += diff;
		setmark (pmarks[j], j, WP, 0.0);
	      }
	    }
	    mv_flag = 0;
	    return;
	  }
	  if(ov_flag) {		/* click for overlay with scale at crosshair */
	    dat = opdat[nchan] ? opdat[nchan] : 1;
	    if(mlog)
	      dat = LOGD(dat);
	    ymax = dat * (float)ylen /(1.1 * (float)(ybot - ycoor + ylog));

	    if(mlog)
	      ymax = pow((double)10.0,(double)ymax);
	    yinc = yscale(ymax,ymin,(float)ylen);
	    ysmax[do_index] = ymax;
	    ysmin[do_index] = ymin;	         /* save for psfile */
	    over = TRUE;
	    hplot();
	    over = FALSE;
	    XDrawString (dpy,pixmap,gc,xzero+3,ytop+23,ovstr,strlen(ovstr));
	    XCopyArea(dpy,pixmap,gwin,gc,0,0,width,height,0,0);
	    ov_flag = 0;
	    return;
	  }
	  if(peakmode && nchan > pmarks[2] && nchan < pmarks[3]){
	    integrate();	/* click on peak */
	    return;
	  }
	  for(mm = 0; mm < mrks; mm++){	/* drag mark */
	    if(pmarks[mm] == nchan)
	      break;
	  }
	  if(mm<mrks && pmarks[mm] != -1){
	    setmark (pmarks[mm],mm,E,0.0);
	    mv_flag = 1;
	  }
	  return;
	}
	else if(callback_data->event->xbutton.button == 2)
	  ichar = 'm';		/* set mark */

	else if(callback_data->event->xbutton.button == 3)
	  ichar = 'z';		/* mark channel */

	break;

      case ButtonRelease:

	if(callback_data->event->xbutton.button == 1) {

	  xcoor = callback_data->event->xbutton.x;
	  ycoor = callback_data->event->xbutton.y; 
	  XmTextSetString(wdg_arr[k_mess],sxstr);

	  lchan = fchan + (xcoor - xzero)/xinc;
	  if (ycoor > ybot)	/* move spectrum left or right */
	    diff = lchan;

	  if (lchan < fchan)	/* expand spectrum */
	    lchan = fchan;
	  else if (lchan > fchan + len)
	    lchan = fchan + len;

	  if (abs(lchan - nchan) < 1)
	    return;
	  if(mv_flag) {
	    pmarks[mm] = lchan;
	    setmark(pmarks[mm],mm,WP,0.0);
	    mv_flag = 0;
	    return;
	  }
	  sfchan[lexp] = fchan;	/* save former parameters */
	  slen[lexp] = len;
	  if(lexp++ > 9)
	    lexp = 9;

	  if (ycoor > ybot)
	    fchan += (nchan-diff);
	  else
	    fchan = nchan;
	  if (lchan < fchan) {	/* interchange numbers */
	    fchan ^= lchan;
	    lchan ^= fchan;
	    fchan ^= lchan;
	  }
	  if(ycoor < ybot)
	    len = lchan - fchan;
	  if(peakmode) pwidth = len;
	  displa();
	}
	return;

      default:
	return;
    }

    if(ichar != 'e' && ichar != 'x') expand = FALSE;

    switch (ichar) {

      case 'e':			/* set expand mode */
      case 'x':

	if(!expand) {
	    expand = TRUE;
	    lchan = nchan;
	    return;
	}
	/* the second coor. is set */
	sfchan[lexp] = fchan;
	slen[lexp] = len;
	if(lexp++ > 9)		/* save former parameters */
	  lexp = 9;
	if(nchan == lchan)
	  return;
	if (lchan > nchan) {
	    fchan = nchan;
	}
	else {
	    fchan = lchan;
	    lchan = nchan;
	}
	len = lchan - fchan;
	if(peakmode) pwidth = len;
	expand = FALSE;
	break;

      case '.':				/* print chan # and content */
	sprintf (sxstr,"channel: %4d content: %g", nchan, pdat[nchan]);
	XmTextSetString(wdg_arr[k_mess],sxstr);
	return;

      case 'y':				/* set new y-scale */
	if(ycoor > ybot || ycoor < ytop) return;
	dat = pdat[nchan] ? pdat[nchan] : 10.0;
	if(mlog)
	  dat = LOGD(dat);
	yinc = (yset - ycoor)/dat;
	ymax = height/yinc;
	ysmax[0] = ymax;
	ysmin[0] = ymin;	        /* save for psfile reentry */
	yfix = 1;
	break;

      case 'o':				/* set new y-scale */
	if(ycoor > ybot || ycoor < ytop) return;
	if(doflag == FALSE) return;
	displa();		        /* redisplay spectrum */
	temp = ymax;
	for(i = 0; i < mrks; i++){
	  if(INSIDE(pmarks[i]))
	    setmark(pmarks[i],i,WP, 0.0);
	}
	dat = opdat[nchan] ? opdat[nchan] : 1;
	if(mlog)
	  dat = LOGD(dat);
	ymax = dat * (float)ylen /(1.1 * (float)(ybot - ycoor + ylog));
	if(mlog)
	  ymax = pow((double)10.0,(double)ymax);
	yinc = yscale(ymax,ymin,(float)ylen);
	ysmax[do_index] = ymax;
	ysmin[do_index] = ymin;	                   /* save for psfile */
	sprintf(sxstr,"Ymax = %.2f Scale = %f",ymax,temp/ymax);
	XmTextSetString(wdg_arr[k_mess],sxstr);
	XSetForeground (dpy,gc,color_vec[4].pixel);
	over = TRUE;
	hplot();
	over = FALSE;
	XDrawString (dpy,pixmap,gc,xzero+3,ytop+23,ovstr,strlen(ovstr));
	XCopyArea(dpy,pixmap,gwin,gc,0,0,width,height,0,0);
	return;

      case 'r':
	if(--lexp < 0) {	        /* reset display */
	  if(!yfix) {
	    putchar(07);
	    fflush(stdout);
	    if(peakmode)
	      peakmode = 0;
	  }
	  lexp = 0;
	  yfix = 0;
	}
	fchan = sfchan[lexp];
	len   = slen[lexp];
	if(peakmode) pwidth = len;
	break;

      case 'l':				/* toggle linear - log */
	mlog = 1 - mlog;
	yfix = 0;
	break;

      case ';':				/* toggle escale */
      case 27:
	hptr->cflag = 1 - hptr->cflag;
	yfix = 0;
	break;

      case 'p':				/* previous region*/
	if(peakmode && udflag) {
	  det--;
	  if(det < 1) det = 1;
	  ntrs = 0;
	  udselect();
	  return;
	}
	if(peakmode){		        /* previous peak */

	  npeak--;
	  ntrs = 0;
	  peak();
	  return;
	}
	dir = 1;			/* reverse scroll */

      case 'n':				/* next region*/
	if(peakmode && udflag) {
	  det++;
	  if(det < 1 || det > 4) det = 1;
	  ntrs = 0;
	  udselect();
	  return;
	}
	if(peakmode){		        /* next peak */
	  npeak++;
	  ntrs = 0;
	  peak();
	  return;
	}
	wrap = (len < 256) ? len*0.05 : 0; /*  a 5% overlap */
	if (dir)	/* backward */
	  fchan -= len - wrap;
	else 		/* forward */
	  fchan += len - wrap;
	dir = 0;
	if (fchan < 0)
	  fchan = 0;
/*	if (fchan+len > hptr->xma)
	  fchan = hptr->xma - len; */
	if(fchan + len > HSIZ)
	  fchan = HSIZ - len;
	break;

      case 'j':				/* next k */
	fchan += 1024;
	if(fchan + len > HSIZ)
	  fchan = HSIZ - len;
	break;

      case 'h':				/* previous k */
	fchan -= 1024;
	if(fchan - len < 0)
	  fchan = 0;
	break;

      case 'm':				/* set marker */
	if(checkmarks()) return;
	pmarks[mrks-1] = nchan;
	setmark(nchan,mrks-1,WP,0.0);
	return;

      case 'k':
	if(mrks == 0) return;
	setmark(pmarks[mrks-1],mrks-1,E,0.0);
	pmarks[mrks-1] = -1;
	mrks--;
	return;

      case 'c':				/* clear all marker */
	if (!mrks)
	  return;
	if(!peakmode) {
	    reset();
	}
	else {
	    for (i = 0; i < 11; i++)
	      pmarks[i] = -1;
	    mrks = 0;
	}
	break;

      case 'a':
	search ();			/* find and mark peaks */
	return;
	
      case 'd':			        /* just redisplay */
	yfix = 0;
	break;

      case 't':
	for (i = 0; i < mrks; i++)
	  if (INSIDE(pmarks[i]))
            printf ("%2d  %5d %10.2f    %7.1f\n",
                    i, pmarks[i], 
                    hptr->cflag  ? cent(pmarks[i]) : 
		    calbrt ((double) pmarks[i]),
                    pdat[pmarks[i]]);
        if ((op = fopen (sadatfile, "a")) != NULL) {
            fputs ("\ntag   ch#     energy    counts\n", op);
            for (i = 0; i < mrks; i++)
              if (INSIDE(pmarks[i]))
                fprintf (op, "%2d  %5d %10.2f    %7.1f\n",
                         i,pmarks[i],
                         hptr->cflag  ? cent(pmarks[i]) : 
			 calbrt ((double) pmarks[i]),
                         pdat[pmarks[i]]);
	    repeat = 1;
	    fclose (op);
	}
	return;

      case '0': 
      case '1': 		/* input single digit */
      case '2': 
      case '3': 
      case '4': 
      case '5': 
      case '6': 
      case '7': 
      case '8': 
      case '9': 
      case '-':
	i = ichar - 48;
	if(i < 0)
	  i = -1;		/*  '-' */
	if(bflag) {		/* set bkgd polynomian */
	  pmarks[10] = i;
	  goto bkcal;		/* see case 'b' */
	}
	if (i == -1) return;
	if(qflag) {		/* insert number between marks */
	    tsum = (float)i;
	    min = abs (nchan - pmarks[0]);
	    nf = 0;
	    for (i = 0; i < mrks; i++) {
		diff = abs (nchan - pmarks[i]);
		if (diff < min) {
		    min = diff;
		    nf = i;
		}
	    }
	    if (pmarks[nf] > nchan)
	      nf--;
	    for (i = pmarks[nf]; i <= pmarks[nf + 1]; i++)
	      pdat[i] = tsum;
	    qflag = 0;
	    return;
	}
	if (i > mrks) return;	/* marker does not exist */
				/* shift marker */
	setmark (pmarks[i], i, E, 0.0);
	pmarks[i] = nchan;
	setmark (nchan, i, WP, 0.0);
	if(peakmode)
	  break;
	return;

      case 'z':

	center = cent(nchan);
	if (calmode == 1) {	          /* show calibration point */
	  cen[tcnt] = center;
	  center = en[tcnt++];
	  setmark (nchan, -1, WP, center);
	  if (tcnt > 1) {
	    docal();
	    calmode = FALSE;
	    tcnt = 0;
	  }
	  return;
	}
	center = calbrt (center); /* general use set position */
	setmark (nchan, -1, WP, center);

	if (calmode == 2) {	/* used for making peak position file */
	  cen[tcnt++] = center;
	  if(tcnt > 1) {
	    ntrs = 3;
	    gmfile();
	    tcnt = 0;
	  }
	}	    
	return;

      case 'v':
	if (!peakmode)
	  return;
	ntrs = 0;
	peakmar();
	return;

      case 'V': 
	if (!peakmode)
	  return;
	ntrs = 1;
	strcpy(wptr[1],"0");
	peakmar();
	return;

      case 'q':
	qflag = 1;
	return;

      case ',': 		/* set equidistant markers */
	if (mrks < 1 || checkmarks())
	  return;
	diff = abs (pmarks[1] - pmarks[0]);
	pmarks[mrks-1] = pmarks[mrks-2] + diff;
	setmark (pmarks[mrks-1], mrks-1, WP, 0.0);
	return;

      case '"': 		/* set window */
	if (mrks < 2)
	  return;
	diff = abs (pmarks[mrks-3] - pmarks[mrks-2]);
	if(checkmarks()) return;
	pmarks[mrks-1] = pmarks[mrks-2] + diff;
	setmark (pmarks[mrks-1], mrks-1, WP, 0.0);
	return;

      case '=':
        tsum = 0;
        min = abs (nchan - pmarks[0]);
        nf = 0;
        for (i = 0; i < mrks; i++) {
            diff = abs (nchan - pmarks[i]);
            if (diff < min) {
                min = diff;
                nf = i;
            }
        }
        if (pmarks[nf] > nchan)
          nf--;
	if(nf == -1)
	  start = fchan;
	else
	  start = (INSIDE(pmarks[nf])) ? pmarks[nf]   : fchan;
	stop = (INSIDE(pmarks[nf+1]))  ? pmarks[nf+1] : fchan + len;

	for (i = start; i < stop; i++)
              tsum += pdat[i];

	sprintf(sxstr,"sum: %d  -  %d %10.1f",start,stop-1, tsum);
	XmTextSetString(wdg_arr[k_mess],sxstr);
        if((op = fopen (sadatfile, "a")) != NULL) {
	    fprintf(op,"sum: %d  -  %d %10.1f\n",start,stop, tsum);
	    fclose(op);
	}
	return;

      case '+':

        if((op = fopen (sadatfile, "a")) != NULL) {
            fputs ("\nSums between marks\n", op);
            for (i = 0; i < mrks; i+=2) {
                nf = pmarks[i];
                nl = pmarks[i+1];
                if (nf > nl) {
                    nf ^= nl;
                    nl ^= nf;
                    nf ^= nl;
                }
                if(nf == -1) break;
                tsum = 0;
                for (j = nf; j < nl; j++)
                  tsum += pdat[j];
                fprintf (op, "%2d  %2d   %d  -  %d %10.1f\n",
                         i, i+1, nf, nl, tsum);
            }
	    fclose (op);
	    repeat = 1;
        }

        puts ("Sums between marks");

        for (i = 0; i < mrks; i+=2) {
            nf = pmarks[i];
            nl = pmarks[i+1];
            if (nf > nl) {
                nf ^= nl;
                nl ^= nf;
                nf ^= nl;
            }
            tsum = 0;
            if(nf == -1) break;
            for (j = nf; j < nl; j++)
              tsum += pdat[j];
            printf ("%5d  %5d %10.1f\n", nf, nl, tsum);
        }
	return;

      case 'i': 
      case 's': 

	integrate();
	return;

      case 'f': 

	fit();
	return;

      case 'b': 

	if(!peakmode) {
	  XmTextSetString(wdg_arr[k_mess],"No peak selected!");
	  putchar(07);
	  fflush(stdout);
	  return;
	}
	if(mrks == 2)		/* 2 mark mode without bkgd calculation */
	  break;
	if (mrks < 6) {
	    XmTextSetString(wdg_arr[k_mess],"Check marks!");
	    putchar(07);
	    fflush(stdout);
	    return;
	}
	if(bflag) {
	  bkcal:
	    if(pmarks[10] < -1 || pmarks[10] > 5) {
	      XmTextSetString(wdg_arr[k_mess],"b-, b0 to b5 only!");
	      bflag = FALSE;
	      putchar(07);
	      fflush(stdout);
	      return;
	    }
	    for (i = 0; i < 6; i++)
	      bp[i] = 0;
	    displa();		/* calculates bp's and plots bkgd */
	    bflag = FALSE;
	    return;
	}
	bflag = TRUE;		/* go again get order of polynomian */
	return;

      case 'g':
	if(peakmode) {		/* if in peakmode return to display */
	    fchan = sfchan[lexp];
	    len   = slen[lexp];
	    peakmode = 0;
	    if(sflag) {		/* clear all marker */
		reset();
		sflag = 0;
	    }
	    printf("%d %d\n",fchan,len);
	    break;
	}
	if(!mrks) {		/* no marks set */
	    marks[0] = nchan;
	    npeak = 0;
	    sflag = 1;
	}
	else {
	    nf = 300;		/* arbitrary distance */
	    for(i=0;i<mrks;i++) {	/* pick nearest peak*/
		if ((nl = abs(nchan - marks[i])) < nf) {
		    nf = nl;
		    npeak = i;
		}
	    }
	}
	sfchan[lexp] = fchan;
	slen[lexp] = len;
	ntrs = 0;
	peak();
	return;

      case 'S':
	tsum = 0.0;
	for(i=0;i<len;i++)
	  tsum += pdat[fchan+i];
	sprintf(sxstr,"Total = %.0f",tsum);
	XmTextSetString(wdg_arr[k_mess],sxstr);
	return;

      case 'u':
	if(!peakmode)	              /* update: repeat same peak */
	  break;
	ntrs = 0;
	yfix = 0;
	peak();
	return;

      case ' ':
	if(peakmode){
	  fchan -= 4;
	  len += 8;
	  pwidth = len;
	}
	else {
	  fchan -= (len*0.05 < 1) ? 1 : len*0.05;
	  if(fchan < 0)
	    fchan = 0;
	  len += (len*0.1 < 2) ? 2 : len*0.1;
	  if(len > hptr->xma)
	    len = hptr->xma;
	}
	break;

      case '>':
	fchan -= (len*0.05 < 1) ? 1 : len*0.05;
	break;

      case '<':
	fchan += (len*0.05 < 1) ? 1 : len*0.05;
	break;

      default:
	return;
    }
    displa();
    return;
}

static void open_proc(w,tag,callback_data)
     Widget w;
     int *tag;
     XmFileSelectionBoxCallbackStruct  *callback_data;
{
    extern int opfile(char *s);
    extern int readin(int flag);
    char *filename;
    int fd;

    int  k = *tag;

    XmStringGetLtoR(callback_data->value,XmSTRING_DEFAULT_CHARSET,&filename);
				/* in Motif 2.2 depreciated */
    /*
    filename = XmStringUnparse(callback_data->value,NULL,0,XmCHARSET_TEXT,
			       NULL, 0, 0);
    */
    XtUnmanageChild(wdg_arr[k_open]);
    strcpy(fname,filename);
    XtFree(filename);
    if ((fd = opfile(fname)) != -1) {
      if(k == k_read) {
	ntrs = 0;
	readin('r');
	hptr = spc[idx];
	fchan = 0;
	len = hptr->xma;
	pdat = hdat + hptr->offset;
	displa();
      }
      else {
	close(fd);
      }
    }
}

void write_file_proc(w,tag,callback_data)
     Widget w;
     int *tag;
     XmAnyCallbackStruct  *callback_data;
{
  extern int wfile(int m);
  register int i;
  int widget_num = *tag;	/* Convert tag to widget number. */

  if(*tag == k_wfilename) {
      int begin = k_wfilename - 1;
      wptr[1] = cmd_line;

      for(i = 1; i < 4; i++) {
	  wptr[i] = XmTextFieldGetString(wdg_arr[i + begin]);
	  wptr[i+1] = wptr[i] + strlen(wptr[i]) + 1;
      }
      ntrs = 3;
      checkname(wptr[1], wmode);
  }
  else {
    wfile(wmode);
  }
}

void selection_proc(w,tag,reason)
     Widget w;
     int *tag;
     XmListCallbackStruct *reason;
{
  extern int dalbrt(float a);
  extern int getid(int n);

  int num;
  int *pos;

  if(XmListGetSelectedPos(w,&pos,&num)== 0)
    return;
  idx = getid(pos[0]);
  XtFree((char *) pos);
  hptr = spc[idx];
  pdat = hdat + hptr->offset;       /* points to begin of
				       current spectrum */
  if (!peakmode) {
    fchan = 0;
    fchan += dalbrt((float)dsr);
    len = dsl ? dalbrt((float)dsl) : hptr->xma;
    lexp = 0;
    sfchan[0] = fchan;
    slen[0] = len;
  }
  ntrs =1;
  numbs[0] = idx;
  displa();
  return;
}

int mv_marks()
{
    register int i,k;

    if(ntrs) {
	k = 0;
	for (i = 1; i <= ntrs; i++) {	   /* fill number array */
	    if(strcmp(wptr[i],"-") == 0) {
		do {
		    numbs[k] = k ? numbs[k-1]+ 1 : 1;
		    if(k > 19) break;       /* upper limit */
		} while (numbs[k++] < atoi(wptr[i+1]));
		k--;
		continue;
	    }
	    numbs [k++] = atoi(wptr[i]);
	}
	ntrs = k;
	mv_flag = 1;
	return(0);
    }
    mv_flag = 0;
    return(0);
}

int integrate()
{
  extern int intgrt(int i);
  extern void dsp(int m);
  int mode;

    if(!peakmode) return(-1);

    if(udflag){
	dsp (1);
    }
    else {
	intgrt(1);
    }
    return(0);
}

int fit()
{
    extern void dsp(int m);
    extern double chi;
    register int i;

    if(!peakmode) return(-1);

    npks = 0;			/* get number of peaks for gfit */
    for (i = 6; i < 10; i++) {
     if (pmarks[i] == -1)
       break;
     if (pmarks[i] > pmarks[2] && 
	  pmarks[i] < pmarks[3])
	++npks;
    }
    if (npks == 0) {
      XmTextSetString(wdg_arr[k_mess],"Peak center is missing");
      putchar(07);
      fflush(stdout);
      return(-1);
    }

    if (udflag) {
	dsp(2);			/* is fit mode */
    }
    else {
	gfit ();
	sprintf(sxstr,"chi sqr = %.2f",chi);
	XmTextSetString(wdg_arr[k_mess],sxstr);
	fplot ();
	output ();
    }
    return(0);
}
int dosystem()
{
  system(strPS);
  return(0);
}

int colorof(char *str)
{
    register int i;
    int cn[6][2] = {{'b',3},{'g',2},{'r',4},{'y',5},
		     {'g',6},{'w',8}};
    int co;

    if(strlen(str) == 1) {
      if(isdigit(str[0]))
	co = atoi(str);
      else {
	for(i=0;i<6;i++)
	  if(str[0] == (char)cn[i][0]) {
	    co = cn[i][1];
	    break;
	  }
      }
    }
    else {
	for(i=1;i<MAXCOL;i++)
	  if(!strcmp(color_vec[i].name,str)) break;
	co = i;
    }	
    if(co < 0 || co > MAXCOL) co = MAXCOL - 1;
    return(co);
}

