/* Input/output of data for SA
 * 
 * includes print-data, rmarks, wmarks,
 * transfer, add, sub, add-constant, mul, div, zero_data,
 * reset, shspec, runfile, kilo, get_fln, subran, read variables
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <math.h>
#include <Mrm/MrmAppl.h>
#include <X11/Xlib.h>
#include <Xm/Xm.h>
#include <Xm/Command.h>
#include <Xm/List.h>
#include <Xm/TextF.h>

#include "dacf.h"

#define    k_com_win               6
#define    k_listing              21
#define    k_list_box             22
#define    k_anis_box             23
#define    k_coef1                24
#define    k_coef2                25
#define    k_theta1               26
#define    k_theta2               27
#define    k_theta3               28
#define    k_slope                29
#define    k_eta                  31

struct command			         /* command window processing */
{
   char  *name;
   int   (*function)();
};
int prtf;			/* print flag for peak pos file */
extern MrmHierarchy s_MrmHierarchy;      /* Mrm database hierarchy ID */
extern MrmType  dummy_class;             /* and class variable. */
extern Widget   toplevel;
extern Widget   wdg_arr[];		 /* all other widgets */

extern  char   *wptr[];		         /* token word pointers */
extern  int     ntrs;			 /* number of entries */
extern  int     idx;		         /* current spectra id */
extern  int     repeat;
extern  int     numbs[];
extern  int     marks[NMARKS];	         /* display marker array */
extern  int     bktg[NMARKS][11];        /* peak marker array */
extern  int     npeak;			 /* current peak number */
extern  int     mrks;		         /* number of marks */
extern  int    *pmarks;
extern  int     nhist;			 /* number of histograms in memory */
extern  int     fchan;
extern  int     len;
extern  int     noid;		         /* select by  array order */
extern  int     knr;		         /* next free adress in data array */
extern  int     ispin;		         /* angular cor */
extern  int     command_num;
extern  char    strPS[];
extern  char    filNam[];                /* file name from command line */
extern  float  *pdat, *hdat;		 /* data */
extern  struct  hinfo *spc[], *hptr;     /* histogram information */
extern  struct  command cmd[];

extern  char    direc[];	         /* data file pathname */
extern  char    fname[];     		 /* data file name */
extern  char    timestr[];	         /* date and time string */
extern  char    sadatfile[];	         /* sa.dat file name */
extern  double  bp[];  
extern  double  Anm[][2];		/* angcor parameters A2 and A4 */
extern  double  anm[],danm[];

int     ran;			         /* ran subtration flag */
float  *pdr;			         /* pointer to randoms spectrum */
int     inda = 0;			         /* anis index */

extern int  psfile(int mode);
extern int  kilo(char*);
extern int  dalbrt(float);
extern void displa();
extern int  inputparser(char*);
extern void s_error(char*);
extern int  opfile(char*);
extern int  peak();
extern int  disp();
extern int  calcoeff();

int getid(int n)
{
    int j;

    if(noid == 0)
      return (n < nhist ? n : -1);
	
    if(n == 0)
      return(0);
    for (j = 0; j < nhist; j++)
      if (n == spc[j]->id)
	return(j);
    if (j == nhist) {
	printf("Cannot find histogram %d\n",n);
	return(-1);
    }
    return(0);
}


int kilo(char *str)
{
    int knr,kl;

    kl = strlen(str);
    if(str[kl-1] == 'k' || str[kl-1] == 'K') {
	str[kl-1] = '\0';
	knr = atoi(str) * 1024;
    }
    else
      knr = atoi(str);
    return(knr);
}

				/* printout of his-data */
int pridata ()
{
    register int    i, j;
    static int  fch = 0, lch = 100;
    int  sto;			/* stderr used */
    float calbrt(double);
    FILE *fps;

    sto = 0;
    if(filNam[0] == '\0') {	/* no output file specified */
      fps = stderr;		/* use stderr */
      sto = 1;
    }
    else {
      if((fps = fopen(filNam, "w")) == NULL) {
	fprintf(stderr,"cannot open data  output file %s\n",wptr[i]);
	return(1);
      }
    }
    if(ntrs) {
      fch = kilo(wptr[1]);
      fch = dalbrt((float)fch);
    }
    if (ntrs > 1) {
      lch = kilo(wptr[2]);
      lch = dalbrt((float)lch);
    }
    if (lch <= fch)
      lch += fch;
    if (lch > hptr->xma)
      lch = hptr->xma;

    if (strcmp(wptr[0],"pdg") == 0) { /* output for grace plots */
      if(sto)
	fprintf(stderr,"Warning: filename missing\n");
      fprintf(fps," @type xy\n");
      for(i=fch;i<lch;i++) 
          fprintf(fps,"%7.2f  %8.2f\n",calbrt((double)i),pdat[i-1]);
      if (!sto) fclose (fps);
      return(0);
    }
    for (i = fch; i < lch; i += 10) {
      fprintf (fps,"%4d", i);
      for (j = 0; j < 10; j++)
	fprintf (fps,"%6.1f%c", pdat[i + j], (j % 10 == 9) ? '\n' : ' ');
    }
    if (!sto) fclose(fps);
    return(0);
}

int peakmar() {
  register int i;
  int old;

  if(ntrs > 0) 
    old = atoi(wptr[1]);
  else old = npeak - 1;		/* former peak */
  if (old < 0 || marks[old] == -1)
    return(-1);
  for(i=0; i< 11; i++)
    bktg[npeak][i] = bktg[old][i];
  ntrs = 0;
  peak();
  return(0);
}

int rmarks () {
    int     fd;
    char    str[8];
    XmString text;

    if (ntrs <1) 
      strcpy(wptr[1],"sa");
    strcat (wptr[1], ".mks");
    if ((fd = open (wptr[1], 0)) < 0) {
      text =XmStringCreate("No such file",
			   XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(1);
    }
    read (fd, marks, sizeof (marks));
    read (fd, bktg, sizeof (bktg));
    close (fd);
    displa();
    return(0);
}

int wmarks () {
    int     fd;

    if (ntrs <1) 
      strcpy(wptr[1],"sa");
    strcat (wptr[1], ".mks");
    if ((fd = creat (wptr[1], 0664)) < 0) {
	perror (" ");
	return(1);
    }
    write (fd, marks, sizeof (marks));
    write (fd, bktg, sizeof (bktg));
    close (fd);
    return(0);
}

int w_win()
{
    register int i;

    char fname[32];
    FILE *op;

    strcpy(fname, wptr[1]);
    strcat(fname,".win");

    if ((op = fopen (fname, "w")) != 0) {
	for (i = 0; i < mrks; i+=2)
	  fprintf (op, "%5d %5d\n",pmarks[i],pmarks[i+1]);
	fclose (op);
	return (0);
    }
    else
      return(-1);
}

int r_win()
{
    register int i;
    char fname[32];
    XmString text;
    FILE *op;

    strcpy(fname, wptr[1]);
    strcat(fname,".win");
    if ((op = fopen (fname, "r")) != NULL) {
	for (i = 0; i < NMARKS; i+=2)
	  if(fscanf (op, "%5d %5d\n",&pmarks[i],&pmarks[i+1]) == EOF)
	    break;
	fclose (op);
	displa();
	return (0);
    }
    else {
      text =XmStringCreate("No such file",
			   XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(-1);
    }
}

int reset () {				 /* reset and clear all markers */
    register int    i, k;

    for (i = 0; i < NMARKS; i++) {
	marks[i]  = -1;
	for (k = 0; k < 11; k++)
	    bktg[i][k] = -1;
    }
    mrks = 0;
    displa();
    return(0);
}

/******************* transfer data ***********************************/

int trans ()			/* tr transfer data in memory */
{
    register int    i;
    int nid;			/* new id */

    XmString text;
    char str[8];

    static int  ich;		/* initial channel */
    static int  dch;		/* destination */
    static int  dlen = 1024;	/* number of channels */

    char    chstr[64];
    XmString list[NSPEC];

    if(ntrs < 1) {
      text =XmStringCreate("start addr, dest.addr, length, id, title",
			   XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(-1);
    }
    if(wptr[1][0] == 'w') {	/* work space */
      ich = MKNR*1024;
      dlen = spc[0]->xma;
      if(wptr[1][1] == '+') {	/* within work space */
	i = kilo(&wptr[1][2]);
	ich += i;
	dlen -= i;
      }
    }
    else {
      ich = kilo(wptr[1]);
    }
    if(ntrs == 1) {		/* destination */
      dch = knr;		/* append to data array */
      knr = dch + dlen;
    }
    else {			/* put into work space */
      if(wptr[2][0] == 'w') {
        dch = MKNR*1024;
        if(wptr[2][1] == '+') {
          i = kilo(&wptr[2][2]);
          dch += i;
        }
      }
      else {
	dch = kilo(wptr[2]);	/* other destination */
	knr = dch + dlen;
      }
    }
    if (ntrs > 2)		/* total number of channels transfered  */
      dlen = kilo(wptr[3]);
                        /* assign an identification */
    nid = (ntrs > 3) ? atoi(wptr[4]) : 0;

    if ((ich < 0 || ich > HSIZ) || 
	(dch < 0 || dch > HSIZ) ||
	(ich + dlen > HSIZ || dch + dlen > HSIZ)) {
      text =XmStringCreate("Problem, check numbers",
			   XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(-1);
    }
    if (ich > dch) {
	for (i = 0; i < dlen; i++)
	  hdat[dch + i] = hdat[ich + i]; /* always absolut addresses */
    }
    else if (ich < dch) {
	for (i = dlen; i > 0; i--)
	  hdat[dch + i] = hdat[ich + i];
    }
    for(i=1;i<nhist;i++)	/* find end of data array */
      if(spc[i]->offset == dch)
	break;
    if(i == nhist) {
      nhist++;
      knr = dch + dlen;
    }
    hptr = spc[i];
    hptr->offset = dch;
    hptr->xma = dlen;
    hptr->id = (nid) ? nid : i;
    strcpy(spc[i]->title,strPS);

    if(wdg_arr[k_list_box] != NULL && XtIsManaged(wdg_arr[k_list_box])) {
      for (i=1;i<nhist;i++){
	sprintf(chstr,"%3d %-4d %-12s %4dk",
		i,spc[i]->id,spc[i]->title,spc[i]->offset/1024);
	list[i-1] = XmStringCreateLocalized(chstr);
      }
      i--;
      XtVaSetValues (wdg_arr[k_listing],XmNitems, list,
		     XmNitemCount, i,
		     XmNmustMatch, True,
		     NULL);
      while (--i >= 0)
	XmStringFree(list[i]);
    }
    return(0);
}

int addsub (int flag)
{
    register int i, k, j;
    extern int disp();
    float  *pf;
    float  *pd;
    int  dlen;
    int narr[NSPEC];
    int arr[NSPEC];
    XmString text;
    char errstr[20];

    k = 0;
    if(ntrs > 10) ntrs = 10;           /* max of numbs array */
    for (i = 1; i <= ntrs; i++) {	   /* fill number array */
      if(strcmp(wptr[i],"-") == 0) {
	do {
	  narr[k] = k ? narr[k-1]+ 1 : 1;
	  if(k > NSPEC-1) break;       /* upper limit NSPEC */
	} while (narr[k++] < atoi(wptr[i+1]));
	k--;
	continue;
      }
      narr[k++] = atoi(wptr[i]);
      if(k > NSPEC) break;
    }
    j = 0;
    for(i=0;i<k;i++)
      if(getid(narr[i]) >= 0)	/* check for valid entries */
	arr[j++] = narr[i];
    ntrs = j;
    if (ntrs <= 0) {
	text = XmStringCreate("no valid input",XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }
    pf = hdat + MKNR*1024;	/* work area */
    spc[0]->xma = spc[arr[0]]->xma;
    for(j=0;j<ntrs;j++){
      k = arr[j];
      dlen = spc[k]->xma;
      if(dlen > spc[0]->xma) 
	spc[0]->xma = dlen;
      pd = hdat + spc[k]->offset;

      switch (flag) {

      case '+':
	for (i = 0; i < dlen; i++)
	  pf[i] +=  pd[i];	 /* add to current address */
	pf[0] /= 2.0;
	break;

      case '*':
	for (i = 0; i < dlen; i++)
	  if(pd[i] != 0)
	    pf[i] *=  pd[i];
	break;

      case '/':
	for (i = 0; i < dlen; i++)
	  if(pd[i] != 0)
	    pf[i] /=  pd[i];
	break;

      case '-':
	for (i = 0; i < dlen; i++)
	  pf[i] -= pd[i];
	break;

      default:
	break;
      }
    }
    ntrs = 1;
    strcpy(wptr[1],"0");
    disp();
    return(0);
}

int add() { return(addsub('+'));}
int sub() { return(addsub('-'));}
int divide() { return(addsub('/'));}
int multiply() {return(addsub('*'));}

     
int forge()			/* modify data */
{
    register int i;
    float   *pd;
    int      dlen;
    double   tmp;
    XmString text;
    char     errstr[20];
    static   float value;


    switch (ntrs) {
    case 1:
      value = atof (wptr[1]);
      break;
    case 2:
      if((idx = getid(atoi(wptr[1]))) < 0) {
	sprintf(errstr,"%s unknown",wptr[1]);
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win],text);
	XmStringFree(text);
	return(-1);
      }
      value = atof (wptr[2]);
      break;
    default:
      break;
    }
    pd = hdat + spc[idx]->offset;
    dlen = spc[idx]->xma;

    if(!strcmp(wptr[0],"mulf")){ /* scale */
      for (i = 0; i < dlen; i++)
	pd[i] *= value;
    }
    else if(!strcmp(wptr[0],"addc")){ /* add/sub constant */
      for (i = 0; i < dlen; i++)
	pd[i] += value;
    }
    else if(!strcmp(wptr[0],"sqrt")){ /* square root of data */
      for (i = 0; i < dlen; i++) 
	if(pd[i] > 0.0) {
	  tmp = sqrt((double)pd[i]);
	  pd[i] = (float)tmp;
	}
    }
    else if(!strcmp(wptr[0],"cut")){ /* supress negative counts */
      if(value > 0.0)
	value = -value;
      for (i = 0; i < dlen; i++)
	if(pd[i] < value)
	  pd[i] = value;
    }
    ntrs = 0;
    disp();
    return(0);
}
/* no where used
int printout () {
  char syscallstr[64];
    sprintf(syscallstr,"lpr -h %s",sadatfile);
    system (syscallstr);
    return(0);
}
*/
int shspec()
{
    register int  i;
    char    chstr[64];
    XmString list[NSPEC];

    printf ("\nind  id  name          begin   length    calibration\n");
    if(ntrs) {
      if((i = getid(atoi(wptr[1]))) >= 0)
	printf("%3d %-4d  %-14s %8d %8d %8.3f %5.3f\n",
	       i,spc[i]->id,spc[i]->title,spc[i]->offset,spc[i]->xma,
	       spc[i]->sl,spc[i]->zr);
    }
    else {
      for (i=1;i<nhist;i++)
	printf("%3d %-4d  %-14s %8d %8d %8.3f %5.3f\n",
	       i,spc[i]->id,spc[i]->title,spc[i]->offset,spc[i]->xma,
	       spc[i]->sl,spc[i]->zr);

      if(wdg_arr[k_list_box] == NULL) {
	if (MrmFetchWidget (s_MrmHierarchy,"listing",
			    toplevel, &wdg_arr[k_list_box],
			    &dummy_class) != MrmSUCCESS)
	  s_error("can't fetch list box widget");
        }
      for (i=1;i<nhist;i++){
	sprintf(chstr,"%3d %-4d %-12s %4dk",
		i,spc[i]->id,spc[i]->title,spc[i]->offset/1024);
	list[i-1] = XmStringCreateLocalized(chstr);
      }
      i--;
      XtVaSetValues (wdg_arr[k_listing],XmNitems, list,
		     XmNitemCount, i,
		     XmNmustMatch, True,
		     NULL);
      while (--i >= 0)
	XmStringFree(list[i]);

      if (!XtIsManaged(wdg_arr[k_list_box]))
	XtManageChild(wdg_arr[k_list_box]);
    }
    return(0);
}

int sel_id()
{
    register int i, k, j, tl;
    float *pd;
    XmString text;
    int narr[10];
    int arr[10];
    int limit;
    int out;
    FILE *op;

    k = 0;
    if(ntrs > 10) ntrs = 10;           /* max of numbs array */
    for (i = 1; i <= ntrs; i++) {	   /* fill number array */
      if(strcmp(wptr[i],"-") == 0) {
	do {
	  narr[k] = k ? narr[k-1]+ 1 : 1;
	  if(k > 8) break;       /* upper limit 8 */
	} while (narr[k++] < atoi(wptr[i+1]));
	k--;
	continue;
      }
      narr[k++] = atoi(wptr[i]);
    }
    j = 0;
    for(i=0;i<k;i++)
      if(getid(narr[i]) >= 0)
	arr[j++] = narr[i];
    ntrs = j;
    if (ntrs <= 0) {
	text = XmStringCreate("no valid input",XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }
    limit = 0;
    pd = hdat + MKNR*1024;	/* begin of work space */
    out = 0;
    for(j=0;j<ntrs;j++){
      hptr = spc[getid(arr[j])];
      limit += hptr->xma;		/* transfer length */
      tl = hptr->xma;
      if (limit > WRKSIZ) {
	tl =  WRKSIZ + tl - limit; /* cut at WRKSIZ */
	limit = WRKSIZ;
	out++;
      }
      for(i=0;i<tl;i++)
	*pd++ = hdat[hptr->offset+i];
      if (out)
	break;
    }
    spc[0]->xma = limit;
    strcpy(spc[0]->title,"w-");
    if(idx !=0)
      strncat(spc[0]->title,hptr->title,13);
    spc[0]->id = hptr->id;
    if(prtf && (op=fopen(sadatfile,"a")) != NULL){
      fprintf(op,"\n%3d",arr[0]);
      fclose(op);
    }
    ntrs = 1;
    strcpy(wptr[1],"0");
    disp();
    return(0);
}

	
int get_fln()
{
    int fd;
    strcpy(fname,direc);
    strcat(fname,wptr[1]);
    if((fd=opfile(fname)) < 0)
      return(-1);
    close(fd);
    return(0);
}

#include <sys/stat.h>

int chpath()
{
    struct stat status;
    direc[0] = '\0';
    if(ntrs == 1) {
	strcpy(direc,wptr[1]);
	strcat(direc,"/");
	if(stat(direc,&status) == -1)
	  perror(" ");
	return(-1);
    }
    return(0);
}

int prpath()
{
    char *cwd, *getcwd();
    XmString text;

    if(direc[0] == '\0'){
	if ((cwd = getcwd((char *)NULL, 64)) == NULL) {
	    perror("pwd");
	    return(-1);
	}
	text = XmStringCreate(cwd,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win],text);
	XmStringFree(text);
    }
    else {
	text = XmStringCreate(direc,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win],text);
	XmStringFree(text);
    }
    return(0);
}
      
int runfile()			/* read commands from file */
{
    register int i;
    FILE *fpp;
    char *cmdpath;		/* environment for script files */
    char buf[80];
    char cmdpathname[64];
    char errstr[80];
    XmString text;
    int  index,narg,j,k;
    char subst[5][32];
    char nbuf[120];
    char numb[2];
    FILE *op;

    if(ntrs < 1) {		/* file name necessary */
	text = XmStringCreate("file name ?",XmSTRING_DEFAULT_CHARSET);
    	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }
    if((cmdpath = getenv("XSACMD")) == NULL)
      strcpy(&cmdpathname[0],"./");	/* optional location for cmd files */
    else 
      strcpy(&cmdpathname[0],cmdpath);
    strcat(cmdpathname,wptr[1]);

    if ((fpp = fopen(wptr[1],"r")) != (FILE *)NULL ) {/* first look in current dir */
      strcpy(&cmdpathname[0],wptr[1]); /* needed for sa.dat file */
    }
    else if((fpp = fopen(cmdpathname,"r")) == (FILE *)NULL) {
	sprintf(errstr,"Can't open %s or %s",wptr[1],cmdpathname);
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
    	XmCommandError(wdg_arr[k_com_win], text);
	XmStringFree(text);
	return(-1);
    }
    if (ntrs > 1) {		/* substitutions for $i */
      for(i=2;i<=ntrs;i++)	/* subst[0] not used */
	strcpy(&subst[i-1][0],wptr[i]);
    }
    narg = ntrs - 1;		/* number of replacements */

    while (fgets(buf,81,fpp) != (char *)NULL) {

	for(i=0;i<16;i++)		/* reset token pointers to NULL*/
	  wptr[i] = NULL;

	for(i=0;i<80;i++) {	/* needed for fgets to clean up \n */
	    if(buf[i] == '\n') {
		buf[i] = '\0';
		break;
	    }
	}
	j = 0;
	for(i=0;i<80;i++) {	/* do command substitutions */
	  if(buf[i] != '$') {
	    nbuf[j++] = buf[i];
	    if(buf[i] == '\0')
	      break;
	  }
	  else {
	    numb[0] = buf[i+1];
	    i++;
	    numb[1] = '\0';
	    if((index = atoi(numb)) > narg) {
	      text = XmStringCreate("missing argument!",
				    XmSTRING_DEFAULT_CHARSET);
	      XmCommandError(wdg_arr[k_com_win], text);
	      XmStringFree(text);
	      fclose(fpp);
	      return(-1);
	    }
	    for(k=0;k<strlen(subst[index]);k++)
	      nbuf[j++] = subst[index][k];
	  }
	}
	nbuf[j] = '\0';
	/*
	printf("%s\n",nbuf);
	*/
	ntrs = inputparser(nbuf);	/* parses input line */

	for(i=0;i < command_num;i++)
	  if (strcmp(wptr[0],cmd[i].name) == 0) {
	      if ( (*cmd[i].function) () )
		i = command_num;
	      break;
	  }
	if(i == command_num) {
	    sprintf(errstr,"%s caused abort",wptr[0]);
	    text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	    XmCommandError(wdg_arr[k_com_win], text);
	    XmStringFree(text);
	    return(-1);
	}
    }
    return(0);
}

int clear_all()
{
  extern double pa[8],dpa[8];
  extern double tru[UDPNR][4][2];	/* results of ud analysis */
  extern double truerr[UDPNR][4][2];

  register int i,j,n;
  XmString list[NSPEC];

  for (i=0; i<HSIZ; i++)
    hdat[i] = 0;
  nhist = 1;
  knr = 0;
  reset();
  for(i=0;i<8;i++) {
    pa[i] = 0.0;
    dpa[i] = 0.0;
  }
  for(i=0;i<UDPNR;i++) {
    for (j=0;j<4;j++) {
      for (n=0;n<2;n++) {
	tru[i][j][n] = 0.0;
	truerr[i][j][n] = 0.0;
      }
    }
  }
  if(wdg_arr[k_list_box] != NULL && XtIsManaged(wdg_arr[k_list_box]))
    XtVaSetValues (wdg_arr[k_listing],XmNitems, list,
		   XmNitemCount, 0,
		   XmNmustMatch, True,
		   NULL);
  return (0);
}

int zero_data()
{
  register int i,j, k;
  int  narr[NSPEC+1];
  static int arr[NSPEC+1];
  XmString text;
  static int entries;

    if (ntrs == 1 && strcmp(wptr[1],"all") == 0) {
      for (i=0; i<HSIZ; i++)
	hdat[i] = 0;
      return (0);
    }
    if (ntrs == 1 && strcmp(wptr[1],"reset") == 0) {
      clear_all();
      return (0);
    }
    if (ntrs == 1 && strcmp(wptr[1],"w") == 0) {
      hptr = spc[0];
      for (i=0; i<WRKSIZ; i++)
	hdat[hptr->offset+i] = 0;
      return (0);
    }
    k = 0;
    if(ntrs > 16) ntrs = 16;           /* max of pointer array */
    for (i = 1; i <= ntrs; i++) {	   /* fill number array */
      if(strcmp(wptr[i],"-") == 0) {
	do {
	  narr[k] = k ? narr[k-1]+ 1 : 1;
	} while (narr[k++] < atoi(wptr[i+1]));
	k--;
	continue;
      }
      narr[k++] = atoi(wptr[i]);
    }
    for(j=0,i=0;i<k;i++)
      if(getid(narr[i]) >= 0)
	arr[j++] = narr[i];
    entries = j;
    if (entries == 0) {
      text = XmStringCreate("no valid input",XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win], text);
      XmStringFree(text);
      return(-1);
    }
    for (i=0;i<entries;i++) {
      hptr = spc[arr[i]];
      for (j=0; j<hptr->xma; j++)
	hdat[hptr->offset+j] = 0;
    }
    return(0);
}

int data_set()
{
  register int i;
  extern int nchan;
  int   min,diff,nf;
  float ndat;


    if(!ntrs)
      return(1);

    if(ntrs == 1) {
      ndat = atof(wptr[1]);
      min = abs (nchan - pmarks[0]);
      nf = 0;
      for (i = 0; i < mrks; i++) {
	diff = abs (nchan - pmarks[i]);
	if (diff < min) {
	  min = diff;
	  nf = i;
	}
      }
      if (pmarks[nf] > nchan)
	nf--;
      for (i = pmarks[nf]; i <= pmarks[nf + 1]; i++)
	pdat[i] = ndat;
    }
    else if (ntrs == 2) {
      nchan = atoi(wptr[1]);
      if (nchan > hptr->xma)
	return(1);
      ndat = atof(wptr[2]);
      pdat[nchan] = ndat;
    }
    return (0);
}
int ps_psfile()
{
  psfile(0);
  return(0);
}
				/* one or several spectra */
int lps_psfile()
{
  psfile(0);
  system("lpr sagraph.ps");
  return(0);
}

int ps1_psfile()
{
  psfile(2);			/* several spectra separated */
  return(0);
}

int lps1_psfile()
{
  psfile(2);
  system("lpr sagraph.ps");
  return(0);
}

int ps2_psfile()
{
  psfile(3);			/* several spectra separated */
  return(0);
}

int lps2_psfile()
{
  psfile(3);
  system("lpr sagraph.ps");
  return(0);
}

int pso_psfile()
{
  psfile(1);			/* overlay mode */
  return(0);
}

int lpso_psfile()
{
  psfile(1);			/* overlay mode & print */
  system("lpr sagraph.ps");
  return(0);
}

int smooth()
{
    float          temp;
    float          min;
    float          new;
    int            n;
    int            i,j;
    int            max;
    int            nx;
    XmString       text;
    char           errstr[20];

    n = 1;			/* smoth over 3 channnels */
    if (ntrs) {
      n = atoi(wptr[1]);
      if (n < 1 || n > 5) {
	sprintf(errstr,"%d should be 1 - 5 only", n);
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win],text);
	XmStringFree(text);
	return(-1);
      }
      if(ntrs > 1) {
	if((nx = getid(atoi(wptr[1]))) < 0) { /* input spec id optional */
	sprintf(errstr,"%s unknown",wptr[1]);
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win],text);
	XmStringFree(text);
	return(-1);
	}
      pdat = hdat + spc[nx]->offset;
      fchan = 0;
      len = spc[nx]->xma;
      }
    }
    max = fchan + len - 1;
    min = 0.0;

    for (i=fchan;i<max;i++)
      if(pdat[i] < min)
	min = pdat[i];
    if(min < 0.0)		/* if data below zero  */
      for (i=fchan;i<max;i++)
	pdat[i] -= min;

    for(j = 0;j < n; j++) {	/* more than 1 smooth */
      temp = pdat[fchan];
      for(i = fchan+1; i < max; i++) {
	new = (temp + 2.0*pdat[i] + pdat[i+1])/4.0;
	temp = pdat[i];
	pdat[i] = new;
      }
      pdat[max] = (temp + 3.0*pdat[max])/4.0;
    }

    if(min < 0.0)
      for (i=fchan;i<max;i++)
	pdat[i] += min;

    ntrs = 0;
    disp();
    return(0);
}

int scale()			/* scale spectra; built a/b for marker pairs */
{			/* result in workspace */
    register int   i,j,n;
    int            nx,mx;
    int            nf,nl;
    int            max;
    float         *pdatb;
    double         suma,sumb;
    double         y[64];
    double         x[64], z[64][64];
    double         yv, tmp;
    double         s,m;
    char           errstr[20];
    XmString       text;

    if (ntrs) {
      if((nx = getid(atoi(wptr[1]))) < 0) { /* spec a */
	sprintf(errstr,"%s unknown",wptr[1]);
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win],text);
	XmStringFree(text);
	return(-1);
      }
      if((mx = getid(atoi(wptr[2]))) < 0) { /* spec b */
	sprintf(errstr,"%s unknown",wptr[1]);
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win],text);
	XmStringFree(text);
	return(-1);
      }
    }
    if(mrks < 2) 
      return(-1);
    pdat = hdat + spc[nx]->offset;
    pdatb = hdat + spc[mx]->offset;
    len = spc[nx]->xma;
    n = 0;
    for (i = 0; i < mrks; i+=2) {
      nf = pmarks[i];
      nl = pmarks[i+1];
      if (nf > nl) {
	nf ^= nl;
	nl ^= nf;
	nf ^= nl;
      }
      if(nf == -1) break;
      suma = 0.0;
      sumb = 0.0;
      for (j = nf; j <= nl; j++) {
	suma += (double)pdat[j];
	sumb += (double)pdatb[j];
      }
      x[n]   = (nl + nf)/2;
      y[n++] = suma/sumb;
    }

    pdat = hdat + spc[0]->offset; /* results in work space */
    spc[0]->xma = len;

    j = 0;			/* fill spectrum */
    for(i=0;i<mrks;i++) {
      max = pmarks[i+1];
      if(i%2 == 0){
	for( ;j<max;j++) {
	  pdat[j] = (float)y[i/2];
	}
      }
      else {			/* linear interpolation */
	s = (y[i/2+1] - y[i/2])/(pmarks[i+1] - pmarks[i]);
	m = y[i/2] - s*pmarks[i];
	for( ;j<max;j++) {
	  pdat[j] = s*j + m;
	}
      }
    }
    for(;j<len;j++)
      pdat[j] = (float)y[n-1];

    /*  spline doesn't work yet */
    /*
    max = n;
    for(i=0;i<max;i++)
      z[0][i] = y[i];
    for(j=1;j<max;j++)
      for(i=0;i<max-j;i++)
        z[j][i] = (z[j-1][i+1] - z[j-1][i])/(x[i+j] - x[i]);

    for(i=0;i<len;i++){
      yv = z[0][0];
      tmp = 1.0;
      for(j=1;j<max;j++) {
        tmp *= ((double)i - x[j-1]);
        yv += z[j][0] * tmp;
      }
      pdat[i] = (float) yv;
    }
    */

    return(0);
}

int subran()
{
  register int i;
  float  *pdw;		/* pointer to work area */
  int     nx;
  int     pkselect;
  XmString text;
  char errstr[20];

    pkselect = 0;
    if(ntrs == 0){
      sprintf(errstr,"need spec id");
      text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(-1);
    }
    if((nx = getid(atoi(wptr[1]))) < 0) {
      sprintf(errstr,"%s unknown",wptr[1]);
      text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
      XmCommandError(wdg_arr[k_com_win],text);
      XmStringFree(text);
      return(-1);
    }
    nx = nx > 64 ? nx - 64 : nx; /* sran 1 starts at id 65 */
    hptr = spc[nx];
    pdw = hdat + MKNR*1024;	     /* work area */
    for(i=0;i<hptr->xma;i++)
      pdw[i] = hdat[hptr->offset+i]; /* fill in work space */
    spc[0]->xma = hptr->xma;
    strcpy(spc[0]->title,"w-");
    strcat(spc[0]->title,hptr->title);
    spc[0]->id = hptr->id;
    hptr = spc[nx+1];		/* default: randoms are next spectrum */
    ran = nx < 16 ? nx : 1;     /* index for anis */
    if(ntrs > 1 && strcmp(wptr[2],"p") == 0) { /* peak selected */
      pkselect = ntrs < 3 ? 1 : 2;
    }
    else if(ntrs > 2 && strcmp(wptr[3],"p") == 0) { /* peak selected */
      pkselect = ntrs < 4 ? 1 : 3;
    }
    else if(ntrs == 2) {	/* id of randoms spectrum provided */
      if((nx = getid(atoi(wptr[2]))) < 0) {
	sprintf(errstr,"%s unknown",wptr[2]);
	text = XmStringCreate(errstr,XmSTRING_DEFAULT_CHARSET);
	XmCommandError(wdg_arr[k_com_win],text);
	XmStringFree(text);
	return(-1);
      }
      hptr = spc[nx];		    /* points to randoms spectrum */
    }
    pdr = hdat + hptr->offset;	    /* pointer to random spectrum */

    for(i=0;i<spc[0]->xma;i++)
      pdw[i] -= pdr[i];		    /* subtract randoms */
    ntrs = 1;
    strcpy(wptr[1],"0");
    disp(); 
    if (pkselect) {	            /* display work area */
      if(pkselect == 2)  	    /* goto peak */
	strcpy(wptr[1],wptr[3]);
      else if (pkselect == 3)
	strcpy(wptr[1],wptr[4]);
      else if (pkselect == 1)	    /* goto last peak */
	ntrs = 0;
      pkselect = 0;
      peak();			/* call peak */
    }
    return(0);
}

int readvars()			/* sa variabels can be read in */
{
  extern  double  thet1,thet2,thet3,q[],creff,dcreff;
  extern  void    calslope(),legendre(double),legendre_deriv(double);
  extern  void    caleta();
  extern  double  slope, dslope, eta, deta, fixedslope;
  char chstr[80];

  if (ntrs > 0) {
    if(strncmp(wptr[1],"thet3",5) == 0){
      thet3 = atof(wptr[2]);
      legendre(thet3);
      legendre_deriv(thet3);
      calslope();
      if(wdg_arr[k_anis_box] != NULL && XtIsManaged(wdg_arr[k_anis_box])) {
        sprintf(chstr,"%7.3f",thet3);
        XmTextFieldSetString(wdg_arr[k_theta3],chstr);
        sprintf(chstr,"%8.5f(%.5f)",eta,deta);
        XmTextFieldSetString(wdg_arr[k_eta],chstr);
        sprintf(chstr,"%7.3f(%.3f)",slope, dslope);
        XmTextFieldSetString(wdg_arr[k_slope],chstr);
      }
      return(0);
    }
    else if(strncmp(wptr[1],"thet1",5) == 0){
      thet1 = atof(wptr[2]);
      if(wdg_arr[k_anis_box] != NULL && XtIsManaged(wdg_arr[k_anis_box])) {
        sprintf(chstr,"%7.3f",thet1);
        XmTextFieldSetString(wdg_arr[k_theta1],chstr);
      }
      return(0);
    }
    else if(strncmp(wptr[1],"thet2",5) == 0){
      thet2 = atof(wptr[2]);
      if(wdg_arr[k_anis_box] != NULL && XtIsManaged(wdg_arr[k_anis_box])) {
        sprintf(chstr,"%7.3f",thet2);
        XmTextFieldSetString(wdg_arr[k_theta2],chstr);
      }
      return(0);
    }
    else if(strncmp(wptr[1],"qs",2) == 0){
      q[0] = atof(wptr[2]);
      q[1] = atof(wptr[3]);
      /*
      caleta();
      sprintf(chstr,"%8.5f(%.5f)",eta,deta);
      XmTextFieldSetString(wdg_arr[k_eta],chstr);
      */
      anm[0] = Anm[ispin][0]*q[0];
      sprintf(chstr,"%7.4f",anm[0]);
      if(wdg_arr[k_coef1] != NULL)
	XmTextFieldSetString(wdg_arr[k_coef1],chstr);
      anm[1] = Anm[ispin][1]*q[1];
      sprintf(chstr,"%7.4f",anm[1]);
      if(wdg_arr[k_coef2] != NULL)
	XmTextFieldSetString(wdg_arr[k_coef2],chstr);
      calslope();
      sprintf(chstr,"%7.3f(%.3f)",slope, dslope);
      if(wdg_arr[k_slope] != NULL)
	XmTextFieldSetString(wdg_arr[k_slope],chstr);
      return(0);
    }
    else if(strncmp(wptr[1],"a2",2) == 0){
      anm[0] = atof(wptr[2]);
      sprintf(chstr,"%7.4f",anm[0]);
      if(wdg_arr[k_coef1] != NULL)
	XmTextFieldSetString(wdg_arr[k_coef1],chstr);
      if(wptr[3] != NULL)
	danm[0] = atof(wptr[3]);
      else
	danm[0] = 0.0;
      eta = (1.0 - anm[0]/(Anm[ispin][0]*q[0]))/6.0;
      deta = danm[0]/fabs((Anm[ispin][0]*q[0]))/6.0;
      sprintf(chstr,"%8.5f(%.5f)",eta,deta);
      XmTextFieldSetString(wdg_arr[k_eta],chstr);
      calcoeff();
      sprintf(chstr,"%7.4f",anm[1]);
      if(wdg_arr[k_coef2] != NULL)
	XmTextFieldSetString(wdg_arr[k_coef2],chstr);
      calslope();
      sprintf(chstr,"%7.3f(%.3f)",slope, dslope);
      if(wdg_arr[k_slope] != NULL)
	XmTextFieldSetString(wdg_arr[k_slope],chstr);
      return(0);
    }
    else if(strncmp(wptr[1],"eff",3) == 0){
      creff  = atof(wptr[2]);
      dcreff = atof(wptr[3]);
      dcreff /= creff;		/* calculate relative error^2 */
      dcreff *= dcreff;
      return(0);
    }
    else if(strncmp(wptr[1],"slope",3) == 0){
      slope  = atof(wptr[2]);
      dslope = atof(wptr[3]);
      fixedslope = (slope == 0) ? FALSE : TRUE;
      return(0);
    }
    else if(strncmp(wptr[1],"ud1",3) == 0){
      inda  = 0;
      return(0);
    }
    else if(strncmp(wptr[1],"ud2",3) == 0){
      inda  = 1;
      return(0);
    }
    else if(strncmp(wptr[1],"pr",2) == 0){
      prtf  = 1 - prtf;
      return(0);
    }
  }
  printf("only thet[1-3],qs,a2,eff,ud1,ud2,slope,pr\n");
  return(1);
}

